@@ -4,15 +4,13 @@ $EM_CONF[$_EXTKEY] = [
     'description' => 'Shows info about TYPO3, installed extensions and a separate module for available modules.',
     'category' => 'module',
     'state' => 'stable',
-    'uploadfolder' => 0,
-    'createDirs' => '',
     'clearCacheOnLoad' => 0,
     'author' => 'TYPO3 Core Team',
     'author_email' => 'typo3cms@typo3.org',
-    'version' => '9.5.14',
+    'version' => '10.3.0',
     'constraints' => [
         'depends' => [
-            'typo3' => '9.5.14',
+            'typo3' => '10.3.0',
         ],
         'conflicts' => [],
         'suggests' => [],
