@@ -4,16 +4,14 @@ $EM_CONF[$_EXTKEY] = [
     'description' => 'The TYPO3 admin panel provides a panel with additional functionality in the frontend (Debugging, Caching, Preview...)',
     'category' => 'fe',
     'state' => 'stable',
-    'uploadfolder' => 0,
-    'createDirs' => '',
     'clearCacheOnLoad' => 1,
     'author' => 'TYPO3 Core Team',
     'author_email' => 'typo3cms@typo3.org',
     'author_company' => '',
-    'version' => '9.5.14',
+    'version' => '10.3.0',
     'constraints' => [
         'depends' => [
-            'typo3' => '9.5.14',
+            'typo3' => '10.3.0',
         ],
         'conflicts' => [],
         'suggests' => [],
