@@ -134,7 +134,7 @@ class ShortcutRepository
     }
 
     /**
-     * runs through the available shortcuts an collects their groups
+     * runs through the available shortcuts and collects their groups
      *
      * @return array Array of groups which have shortcuts
      */
@@ -233,7 +233,7 @@ class ShortcutRepository
         // Check if given id is a combined identifier
         if (!empty($queryParameters['id']) && preg_match('/^[\d]+:/', $queryParameters['id'])) {
             try {
-                $resourceFactory = ResourceFactory::getInstance();
+                $resourceFactory = GeneralUtility::makeInstance(ResourceFactory::class);
                 $resource = $resourceFactory->getObjectFromCombinedIdentifier($queryParameters['id']);
                 $title = trim(sprintf(
                     '%s (%s)',
@@ -484,7 +484,7 @@ class ShortcutRepository
         while ($row = $result->fetch()) {
             $shortcut = ['raw' => $row];
 
-            list($row['module_name'], $row['M_module_name']) = explode('|', $row['module_name']);
+            [$row['module_name'], $row['M_module_name']] = explode('|', $row['module_name']);
 
             $queryParts = parse_url($row['url']);
             // Explode GET vars recursively
@@ -601,6 +601,7 @@ class ShortcutRepository
      */
     protected function getShortcutIcon(array $row, array $shortcut): string
     {
+        $selectFields = [];
         switch ($row['module_name']) {
             case 'xMOD_alt_doc.php':
                 $table = $shortcut['table'];
@@ -626,8 +627,10 @@ class ShortcutRepository
                         $selectFields[] = $GLOBALS['TCA'][$table]['ctrl']['typeicon_column'];
                     }
 
-                    if ($GLOBALS['TCA'][$table]['ctrl']['versioningWS']) {
+                    if (BackendUtility::isTableWorkspaceEnabled($table)) {
                         $selectFields[] = 't3ver_state';
+                        $selectFields[] = 't3ver_wsid';
+                        $selectFields[] = 't3ver_oid';
                     }
 
                     $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
@@ -659,7 +662,7 @@ class ShortcutRepository
                 $moduleName = $row['module_name'];
 
                 if (strpos($moduleName, '_') !== false) {
-                    list($mainModule, $subModule) = explode('_', $moduleName, 2);
+                    [$mainModule, $subModule] = explode('_', $moduleName, 2);
                     $iconIdentifier = $this->moduleLoader->modules[$mainModule]['sub'][$subModule]['iconIdentifier'];
                 } elseif (!empty($moduleName)) {
                     $iconIdentifier = $this->moduleLoader->modules[$moduleName]['iconIdentifier'];
@@ -690,7 +693,7 @@ class ShortcutRepository
         if (strpos($moduleName, 'xMOD_') === 0) {
             $title = substr($moduleName, 5);
         } else {
-            list($mainModule, $subModule) = explode('_', $moduleName);
+            [$mainModule, $subModule] = explode('_', $moduleName);
             $mainModuleLabels = $this->moduleLoader->getLabelsForModule($mainModule);
             $title = $languageService->sL($mainModuleLabels['title']);
 
@@ -747,11 +750,6 @@ class ShortcutRepository
             }
         }
 
-        if (isset($parameters['M']) && empty($parameters['route'])) {
-            $parameters['route'] = $parameters['M'];
-            unset($parameters['M']);
-        }
-
         if (strpos($parsedUrl['path'], 'index.php') !== false && isset($parameters['route'])) {
             $routePath = $parameters['route'];
             /** @var \TYPO3\CMS\Backend\Routing\Router $router */
