@@ -14,20 +14,20 @@ namespace TYPO3\CMS\Backend\Backend\ToolbarItems;
  * The TYPO3 project - inspiring people to share!
  */
 
+use Psr\EventDispatcher\EventDispatcherInterface;
+use TYPO3\CMS\Backend\Backend\Event\SystemInformationToolbarCollectorEvent;
 use TYPO3\CMS\Backend\Routing\UriBuilder;
 use TYPO3\CMS\Backend\Toolbar\Enumeration\InformationStatus;
 use TYPO3\CMS\Backend\Toolbar\ToolbarItemInterface;
 use TYPO3\CMS\Core\Core\Environment;
-use TYPO3\CMS\Core\Core\SystemEnvironmentBuilder;
 use TYPO3\CMS\Core\Database\ConnectionPool;
+use TYPO3\CMS\Core\Information\Typo3Version;
+use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Page\PageRenderer;
 use TYPO3\CMS\Core\Routing\RouteNotFoundException;
 use TYPO3\CMS\Core\Utility\CommandUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\StringUtility;
-use TYPO3\CMS\Core\Utility\VersionNumberUtility;
-use TYPO3\CMS\Extbase\Object\ObjectManager;
-use TYPO3\CMS\Extbase\SignalSlot\Dispatcher;
 use TYPO3\CMS\Fluid\View\StandaloneView;
 
 /**
@@ -67,9 +67,9 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
     protected $systemMessages = [];
 
     /**
-     * @var Dispatcher
+     * @var EventDispatcherInterface
      */
-    protected $signalSlotDispatcher;
+    protected $eventDispatcher;
 
     /**
      * @var int
@@ -77,10 +77,14 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
     protected $maximumCountInBadge = 99;
 
     /**
-     * Constructor
+     * @var Typo3Version
      */
-    public function __construct()
+    protected $typo3Version;
+
+    public function __construct(EventDispatcherInterface $eventDispatcher = null)
     {
+        $this->eventDispatcher = $eventDispatcher ?? GeneralUtility::getContainer()->get(EventDispatcherInterface::class);
+        $this->typo3Version = GeneralUtility::makeInstance(Typo3Version::class);
         $this->getPageRenderer()->loadRequireJsModule('TYPO3/CMS/Backend/Toolbar/SystemInformationMenu');
         $this->highestSeverity = InformationStatus::cast(InformationStatus::STATUS_INFO);
     }
@@ -230,8 +234,7 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
         $this->getGitRevision();
         $this->getOperatingSystem();
 
-        $this->emitGetSystemInformation();
-        $this->emitLoadMessages();
+        $this->eventDispatcher->dispatch(new SystemInformationToolbarCollectorEvent($this));
 
         $this->severityBadgeClass = !$this->highestSeverity->equals(InformationStatus::STATUS_NOTICE) ? 'badge-' . (string)$this->highestSeverity : '';
     }
@@ -243,7 +246,7 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
     {
         $this->systemInformation[] = [
             'title' => 'LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:toolbarItems.sysinfo.typo3-version',
-            'value' => VersionNumberUtility::getCurrentTypo3Version(),
+            'value' => $this->typo3Version->getVersion(),
             'iconIdentifier' => 'information-typo3-version'
         ];
     }
@@ -294,7 +297,7 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
      */
     protected function getApplicationContext()
     {
-        $applicationContext = GeneralUtility::getApplicationContext();
+        $applicationContext = Environment::getContext();
         $this->systemInformation[] = [
             'title'  => 'LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:toolbarItems.sysinfo.applicationcontext',
             'value'  => (string)$applicationContext,
@@ -314,7 +317,7 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
 
         $this->systemInformation[] = [
             'title' => 'LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:toolbarItems.sysinfo.composerMode',
-            'value' => $GLOBALS['LANG']->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.enabled'),
+            'value' => $this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.enabled'),
             'iconIdentifier' => 'information-composer-mode'
         ];
     }
@@ -324,7 +327,7 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
      */
     protected function getGitRevision()
     {
-        if (!StringUtility::endsWith(TYPO3_version, '-dev') || SystemEnvironmentBuilder::isFunctionDisabled('exec')) {
+        if (!StringUtility::endsWith($this->typo3Version->getVersion(), '-dev') || $this->isFunctionDisabled('exec')) {
             return;
         }
         // check if git exists
@@ -370,22 +373,6 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
         ];
     }
 
-    /**
-     * Emits the "getSystemInformation" signal
-     */
-    protected function emitGetSystemInformation()
-    {
-        $this->getSignalSlotDispatcher()->dispatch(__CLASS__, 'getSystemInformation', [$this]);
-    }
-
-    /**
-     * Emits the "loadMessages" signal
-     */
-    protected function emitLoadMessages()
-    {
-        $this->getSignalSlotDispatcher()->dispatch(__CLASS__, 'loadMessages', [$this]);
-    }
-
     /**
      * Returns a new standalone view, shorthand function
      *
@@ -405,6 +392,21 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
         return $view;
     }
 
+    /**
+     * Check if the given PHP function is disabled in the system
+     *
+     * @param string $functionName
+     * @return bool
+     */
+    protected function isFunctionDisabled(string $functionName): bool
+    {
+        $disabledFunctions = GeneralUtility::trimExplode(',', ini_get('disable_functions'));
+        if (!empty($disabledFunctions)) {
+            return in_array($functionName, $disabledFunctions, true);
+        }
+        return false;
+    }
+
     /**
      * Returns the current BE user.
      *
@@ -415,6 +417,14 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
         return $GLOBALS['BE_USER'];
     }
 
+    /**
+     * @return LanguageService|null
+     */
+    protected function getLanguageService(): ?LanguageService
+    {
+        return $GLOBALS['LANG'] ?? null;
+    }
+
     /**
      * Returns current PageRenderer
      *
@@ -424,17 +434,4 @@ class SystemInformationToolbarItem implements ToolbarItemInterface
     {
         return GeneralUtility::makeInstance(PageRenderer::class);
     }
-
-    /**
-     * Get the SignalSlot dispatcher
-     *
-     * @return Dispatcher
-     */
-    protected function getSignalSlotDispatcher()
-    {
-        if (!isset($this->signalSlotDispatcher)) {
-            $this->signalSlotDispatcher = GeneralUtility::makeInstance(ObjectManager::class)->get(Dispatcher::class);
-        }
-        return $this->signalSlotDispatcher;
-    }
 }
