@@ -18,6 +18,7 @@ use TYPO3\CMS\Backend\Routing\UriBuilder;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Core\Database\ConnectionPool;
 use TYPO3\CMS\Core\Database\Query\Restriction\DeletedRestriction;
+use TYPO3\CMS\Core\Database\Query\Restriction\WorkspaceRestriction;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
 use TYPO3\CMS\Core\Resource\AbstractFile;
@@ -273,7 +274,7 @@ class Clipboard
             // Import / Export link:
             if (ExtensionManagementUtility::isLoaded('impexp')) {
                 $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
-                $url = $uriBuilder->buildUriFromRoute('xMOD_tximpexp', $this->exportClipElementParameters());
+                $url = $uriBuilder->buildUriFromRoute('tx_impexp_export', $this->exportClipElementParameters());
                 $optionArray[] = [
                     'label' => $this->clLabel('export', 'rm'),
                     'uri' => (string)$url
@@ -285,7 +286,7 @@ class Clipboard
                     'label' => $this->clLabel('edit', 'rm'),
                     'uri' => '#',
                     'additionalAttributes' => [
-                        'onclick' => htmlspecialchars('window.location.href=' . GeneralUtility::quoteJSvalue($this->editUrl() . '&returnUrl=') . '+top.rawurlencode(window.location.href);'),
+                        'onclick' => htmlspecialchars('window.location.href=' . GeneralUtility::quoteJSvalue($this->editUrl() . '&returnUrl=') . '+encodeURIComponent(window.location.href);'),
                     ]
                 ];
             }
@@ -337,7 +338,7 @@ class Clipboard
             'id' => 'normal',
             'number' => 0,
             'url' => GeneralUtility::linkThisScript(['CB' => ['setP' => 'normal']]),
-            'description' => 'normal-description',
+            'description' => 'labels.normal-description',
             'label' => 'labels.normal',
             'padding' => $this->padTitle('normal')
         ];
@@ -350,7 +351,7 @@ class Clipboard
                 'id' => 'tab_' . $a,
                 'number' => $a,
                 'url' => GeneralUtility::linkThisScript(['CB' => ['setP' => 'tab_' . $a]]),
-                'description' => 'cliptabs-description',
+                'description' => 'labels.cliptabs-description',
                 'label' => 'labels.cliptabs-name',
                 'padding' => $this->padTitle('tab_' . $a)
             ];
@@ -376,10 +377,10 @@ class Clipboard
         if (is_array($this->clipData[$pad]['el'] ?? false)) {
             foreach ($this->clipData[$pad]['el'] as $k => $v) {
                 if ($v) {
-                    list($table, $uid) = explode('|', $k);
+                    [$table, $uid] = explode('|', $k);
                     // Rendering files/directories on the clipboard
                     if ($table === '_FILE') {
-                        $fileObject = ResourceFactory::getInstance()->retrieveFileOrFolderObject($v);
+                        $fileObject = GeneralUtility::makeInstance(ResourceFactory::class)->retrieveFileOrFolderObject($v);
                         if ($fileObject) {
                             $thumb = [];
                             $folder = $fileObject instanceof \TYPO3\CMS\Core\Resource\Folder;
@@ -475,6 +476,8 @@ class Clipboard
     {
         $lines = [];
         $tcaCtrl = $GLOBALS['TCA'][$table]['ctrl'];
+        $workspaceId = (int)$this->getBackendUser()->workspace;
+
         if (BackendUtility::isTableLocalizable($table)) {
             $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($table);
             $queryBuilder->getRestrictions()
@@ -497,16 +500,13 @@ class Clipboard
                         'pid',
                         $queryBuilder->createNamedParameter(-1, \PDO::PARAM_INT)
                     )
-                );
+                )
+                ->orderBy($tcaCtrl['languageField']);
 
-            if (isset($tcaCtrl['versioningWS']) && $tcaCtrl['versioningWS']) {
-                $queryBuilder
-                    ->andWhere(
-                        $queryBuilder->expr()->eq(
-                            't3ver_wsid',
-                            $queryBuilder->createNamedParameter($parentRec['t3ver_wsid'], \PDO::PARAM_INT)
-                        )
-                    );
+            if (BackendUtility::isTableWorkspaceEnabled($table)) {
+                $queryBuilder->getRestrictions()->add(
+                    GeneralUtility::makeInstance(WorkspaceRestriction::class, $workspaceId)
+                );
             }
             $rows = $queryBuilder->execute()->fetchAll();
             if (is_array($rows)) {
@@ -662,7 +662,7 @@ class Clipboard
         // All records
         $elements = $this->elFromTable('');
         foreach ($elements as $tP => $value) {
-            list($table, $uid) = explode('|', $tP);
+            [$table, $uid] = explode('|', $tP);
             $parameters['edit[' . $table . '][' . $uid . ']'] = 'edit';
         }
         $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
@@ -757,15 +757,14 @@ class Clipboard
         // Init
         $pad = $this->current;
         $params = [];
-        $params['tx_impexp']['action'] = 'export';
         // Traverse items:
         if (is_array($this->clipData[$pad]['el'] ?? false)) {
             foreach ($this->clipData[$pad]['el'] as $k => $v) {
                 if ($v) {
-                    list($table, $uid) = explode('|', $k);
+                    [$table, $uid] = explode('|', $k);
                     // Rendering files/directories on the clipboard
                     if ($table === '_FILE') {
-                        $file = ResourceFactory::getInstance()->getObjectFromCombinedIdentifier($v);
+                        $file = GeneralUtility::makeInstance(ResourceFactory::class)->getObjectFromCombinedIdentifier($v);
                         if ($file instanceof AbstractFile) {
                             $params['tx_impexp']['record'][] = 'sys_file:' . $file->getUid();
                         }
@@ -827,7 +826,7 @@ class Clipboard
     {
         if (is_array($this->clipData[$this->current]['el'] ?? false)) {
             foreach ($this->clipData[$this->current]['el'] as $k => $v) {
-                list($table, $uid) = explode('|', $k);
+                [$table, $uid] = explode('|', $k);
                 if ($table !== '_FILE') {
                     if (!$v || !is_array(BackendUtility::getRecord($table, $uid, 'uid'))) {
                         unset($this->clipData[$this->current]['el'][$k]);
@@ -839,7 +838,7 @@ class Clipboard
                         $this->changed = 1;
                     } else {
                         try {
-                            ResourceFactory::getInstance()->retrieveFileOrFolderObject($v);
+                            GeneralUtility::makeInstance(ResourceFactory::class)->retrieveFileOrFolderObject($v);
                         } catch (\TYPO3\CMS\Core\Resource\Exception\ResourceDoesNotExistException $e) {
                             // The file has been deleted in the meantime, so just remove it silently
                             unset($this->clipData[$this->current]['el'][$k]);
@@ -859,12 +858,12 @@ class Clipboard
      */
     public function elFromTable($matchTable = '', $pad = '')
     {
-        $pad = $pad ? $pad : $this->current;
+        $pad = $pad ?: $this->current;
         $list = [];
         if (is_array($this->clipData[$pad]['el'] ?? false)) {
             foreach ($this->clipData[$pad]['el'] as $k => $v) {
                 if ($v) {
-                    list($table, $uid) = explode('|', $k);
+                    [$table, $uid] = explode('|', $k);
                     if ($table !== '_FILE') {
                         if ((!$matchTable || (string)$table == (string)$matchTable) && $GLOBALS['TCA'][$table]) {
                             $list[$k] = $pad === 'normal' ? $v : $uid;
@@ -908,7 +907,7 @@ class Clipboard
         if (!$table && !$uid) {
             $elArr = $this->elFromTable('');
             reset($elArr);
-            list($table, $uid) = explode('|', key($elArr));
+            [$table, $uid] = explode('|', key($elArr));
         }
         if ($this->isSelected($table, $uid)) {
             $selRec = BackendUtility::getRecordWSOL($table, $uid);
@@ -948,7 +947,7 @@ class Clipboard
      */
     public function makePasteCmdArray($ref, $CMD, array $update = null)
     {
-        list($pTable, $pUid) = explode('|', $ref);
+        [$pTable, $pUid] = explode('|', $ref);
         $pUid = (int)$pUid;
         // pUid must be set and if pTable is not set (that means paste ALL elements)
         // the uid MUST be positive/zero (pointing to page id)
@@ -959,7 +958,7 @@ class Clipboard
             $mode = $this->currentMode() === 'copy' ? 'copy' : 'move';
             // Traverse elements and make CMD array
             foreach ($elements as $tP => $value) {
-                list($table, $uid) = explode('|', $tP);
+                [$table, $uid] = explode('|', $tP);
                 if (!is_array($CMD[$table])) {
                     $CMD[$table] = [];
                 }
@@ -992,7 +991,7 @@ class Clipboard
         // all records
         $elements = $this->elFromTable('');
         foreach ($elements as $tP => $value) {
-            list($table, $uid) = explode('|', $tP);
+            [$table, $uid] = explode('|', $tP);
             if (!is_array($CMD[$table])) {
                 $CMD[$table] = [];
             }
@@ -1018,7 +1017,7 @@ class Clipboard
      */
     public function makePasteCmdArray_file($ref, $FILE)
     {
-        list($pTable, $pUid) = explode('|', $ref);
+        [$pTable, $pUid] = explode('|', $ref);
         $elements = $this->elFromTable('_FILE');
         $mode = $this->currentMode() === 'copy' ? 'copy' : 'move';
         // Traverse elements and make CMD array
