@@ -17,6 +17,8 @@ namespace TYPO3\CMS\Backend\ContextMenu\ItemProviders;
 
 use TYPO3\CMS\Backend\Routing\UriBuilder;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
+use TYPO3\CMS\Core\Domain\Repository\PageRepository;
+use TYPO3\CMS\Core\Routing\UnableToLinkToPageException;
 use TYPO3\CMS\Core\Type\Bitmask\Permission;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
@@ -121,6 +123,16 @@ class PageProvider extends RecordProvider
                     'iconIdentifier' => 'actions-pagetree-mountroot',
                     'callbackAction' => 'mountAsTreeRoot',
                 ],
+                'showInMenus' => [
+                    'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_misc.xlf:CM_showInMenus',
+                    'iconIdentifier' => 'actions-view',
+                    'callbackAction' => 'showInMenus',
+                ],
+                'hideInMenus' => [
+                    'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_misc.xlf:CM_hideInMenus',
+                    'iconIdentifier' => 'actions-ban',
+                    'callbackAction' => 'hideInMenus',
+                ],
             ],
         ],
         'divider3' => [
@@ -211,6 +223,12 @@ class PageProvider extends RecordProvider
             case 'disable':
                 $canRender = $this->canBeDisabled();
                 break;
+            case 'showInMenus':
+                $canRender = $this->canBeToggled('nav_hide', 1);
+                break;
+            case 'hideInMenus':
+                $canRender = $this->canBeToggled('nav_hide', 0);
+                break;
             case 'delete':
                 $canRender = $this->canBeDeleted();
                 break;
@@ -535,14 +553,42 @@ class PageProvider extends RecordProvider
     {
         $language = (int)$this->record['sys_language_uid'];
         $additionalParams = ($language > 0) ? '&L=' . $language : '';
-        return BackendUtility::getPreviewUrl(
-            $this->getPreviewPid(),
-            '',
-            null,
-            '',
-            '',
-            $additionalParams
-        );
+
+        try {
+            return BackendUtility::getPreviewUrl(
+                $this->getPreviewPid(),
+                '',
+                null,
+                '',
+                '',
+                $additionalParams
+            );
+        } catch (UnableToLinkToPageException $e) {
+            return '';
+        }
+    }
+
+    /**
+     * Checks if user has access to this column
+     * and the page doktype is lower than 200 (exclude sys_folder, ...)
+     * and it contains given value
+     *
+     * @param string $fieldName
+     * @param int $value
+     * @return bool
+     */
+    protected function canBeToggled(string $fieldName, int $value): bool
+    {
+        if (!$this->languageAccess) {
+            return false;
+        }
+        if (!empty($GLOBALS['TCA'][$this->table]['columns'][$fieldName]['exclude'])
+            && $this->record['doktype'] <= PageRepository::DOKTYPE_SPACER
+            && $this->backendUser->check('non_exclude_fields', $this->table . ':' . $fieldName)
+        ) {
+            return (int)$this->record[$fieldName] === $value;
+        }
+        return false;
     }
 
     /**
