@@ -16,18 +16,17 @@ namespace TYPO3\CMS\Backend\Controller;
 
 use Psr\Http\Message\ResponseInterface;
 use TYPO3\CMS\Backend\Domain\Repository\Module\BackendModuleRepository;
-use TYPO3\CMS\Backend\Module\ModuleLoader;
 use TYPO3\CMS\Backend\Routing\UriBuilder;
+use TYPO3\CMS\Backend\Template\ModuleTemplate;
 use TYPO3\CMS\Backend\Toolbar\ToolbarItemInterface;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
-use TYPO3\CMS\Core\Compatibility\PublicMethodDeprecationTrait;
+use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
 use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
-use TYPO3\CMS\Core\Database\ConnectionPool;
-use TYPO3\CMS\Core\Database\Query\Restriction\BackendWorkspaceRestriction;
-use TYPO3\CMS\Core\Database\Query\Restriction\DeletedRestriction;
 use TYPO3\CMS\Core\Http\HtmlResponse;
 use TYPO3\CMS\Core\Http\JsonResponse;
 use TYPO3\CMS\Core\Imaging\IconFactory;
+use TYPO3\CMS\Core\Information\Typo3Version;
+use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Page\PageRenderer;
 use TYPO3\CMS\Core\Type\Bitmask\Permission;
 use TYPO3\CMS\Core\Type\File\ImageInfo;
@@ -41,50 +40,16 @@ use TYPO3\CMS\Fluid\View\StandaloneView;
  */
 class BackendController
 {
-    use PublicMethodDeprecationTrait;
-
-    /**
-     * @var array
-     */
-    private $deprecatedPublicMethods = [
-        'render' => 'Using BackendController::render() is deprecated and will not be possible anymore in TYPO3 v10.0.',
-    ];
-
-    /**
-     * @var string
-     */
-    protected $content = '';
-
     /**
      * @var string
      */
     protected $css = '';
 
-    /**
-     * @var array
-     */
-    protected $cssFiles = [];
-
-    /**
-     * @var string
-     */
-    protected $js = '';
-
-    /**
-     * @var array
-     */
-    protected $jsFiles = [];
-
     /**
      * @var array
      */
     protected $toolbarItems = [];
 
-    /**
-     * @var bool
-     */
-    protected $debug;
-
     /**
      * @var string
      */
@@ -96,15 +61,10 @@ class BackendController
     protected $partialPath = 'EXT:backend/Resources/Private/Partials/';
 
     /**
-     * @var \TYPO3\CMS\Backend\Domain\Repository\Module\BackendModuleRepository
+     * @var BackendModuleRepository
      */
     protected $backendModuleRepository;
 
-    /**
-     * @var \TYPO3\CMS\Backend\Module\ModuleLoader Object for loading backend modules
-     */
-    protected $moduleLoader;
-
     /**
      * @var PageRenderer
      */
@@ -115,81 +75,67 @@ class BackendController
      */
     protected $iconFactory;
 
+    /**
+     * @var Typo3Version
+     */
+    protected $typo3Version;
+
+    /**
+     * @var UriBuilder
+     */
+    protected $uriBuilder;
+
     /**
      * Constructor
      */
     public function __construct()
     {
         $this->getLanguageService()->includeLLFile('EXT:core/Resources/Private/Language/locallang_misc.xlf');
-
         $this->backendModuleRepository = GeneralUtility::makeInstance(BackendModuleRepository::class);
         $this->iconFactory = GeneralUtility::makeInstance(IconFactory::class);
-        $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
-        // Set debug flag for BE development only
-        $this->debug = (int)$GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] === 1;
-        // Initializes the backend modules structure for use later.
-        $this->moduleLoader = GeneralUtility::makeInstance(ModuleLoader::class);
-        $this->moduleLoader->load($GLOBALS['TBE_MODULES']);
+        $this->uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
+        $this->typo3Version = GeneralUtility::makeInstance(Typo3Version::class);
         $this->pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
         // Add default BE javascript
-        $this->jsFiles = [
-            'md5' => 'EXT:backend/Resources/Public/JavaScript/md5.js',
-            'evalfield' => 'EXT:backend/Resources/Public/JavaScript/jsfunc.evalfield.js',
-            'backend' => 'EXT:backend/Resources/Public/JavaScript/backend.js',
-        ];
+        $this->pageRenderer->addJsFile('EXT:backend/Resources/Public/JavaScript/md5.js');
+        $this->pageRenderer->addJsFile('EXT:backend/Resources/Public/JavaScript/backend.js');
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/LoginRefresh', 'function(LoginRefresh) {
 			LoginRefresh.setIntervalTime(' . MathUtility::forceIntegerInRange((int)$GLOBALS['TYPO3_CONF_VARS']['BE']['sessionTimeout'] - 60, 60) . ');
-			LoginRefresh.setLoginFramesetUrl(' . GeneralUtility::quoteJSvalue((string)$uriBuilder->buildUriFromRoute('login_frameset')) . ');
-			LoginRefresh.setLogoutUrl(' . GeneralUtility::quoteJSvalue((string)$uriBuilder->buildUriFromRoute('logout')) . ');
+			LoginRefresh.setLoginFramesetUrl(' . GeneralUtility::quoteJSvalue((string)$this->uriBuilder->buildUriFromRoute('login_frameset')) . ');
+			LoginRefresh.setLogoutUrl(' . GeneralUtility::quoteJSvalue((string)$this->uriBuilder->buildUriFromRoute('logout')) . ');
 			LoginRefresh.initialize();
 		}');
 
-        // load module menu
+        $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/BroadcastService', 'function(service) { service.listen(); }');
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/ModuleMenu');
-
-        // load Toolbar class
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/Toolbar');
-
-        // load Utility class
-        $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/Utility');
-
-        // load Notification functionality
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/Notification');
-
-        // load Modals
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/Modal');
-
-        // load InfoWindow
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/InfoWindow');
-
-        // load ContextMenu
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/ContextMenu');
 
         // load the storage API and fill the UC into the PersistentStorage, so no additional AJAX call is needed
-        $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/Storage');
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/Storage/Persistent', 'function(PersistentStorage) {
             PersistentStorage.load(' . json_encode($this->getBackendUser()->uc) . ');
         }');
 
-        // load debug console
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/DebugConsole');
 
         $this->pageRenderer->addInlineLanguageLabelFile('EXT:core/Resources/Private/Language/locallang_core.xlf');
         $this->pageRenderer->addInlineLanguageLabelFile('EXT:core/Resources/Private/Language/locallang_misc.xlf');
         $this->pageRenderer->addInlineLanguageLabelFile('EXT:backend/Resources/Private/Language/locallang_layout.xlf');
         $this->pageRenderer->addInlineLanguageLabelFile('EXT:core/Resources/Private/Language/locallang_mod_web_list.xlf');
-
         $this->pageRenderer->addInlineLanguageLabelFile('EXT:core/Resources/Private/Language/debugger.xlf');
         $this->pageRenderer->addInlineLanguageLabelFile('EXT:core/Resources/Private/Language/wizard.xlf');
 
-        $this->pageRenderer->addInlineSetting('ShowItem', 'moduleUrl', (string)$uriBuilder->buildUriFromRoute('show_item'));
-        $this->pageRenderer->addInlineSetting('RecordHistory', 'moduleUrl', (string)$uriBuilder->buildUriFromRoute('record_history'));
-        $this->pageRenderer->addInlineSetting('NewRecord', 'moduleUrl', (string)$uriBuilder->buildUriFromRoute('db_new'));
-        $this->pageRenderer->addInlineSetting('FormEngine', 'moduleUrl', (string)$uriBuilder->buildUriFromRoute('record_edit'));
-        $this->pageRenderer->addInlineSetting('RecordCommit', 'moduleUrl', (string)$uriBuilder->buildUriFromRoute('tce_db'));
-        $this->pageRenderer->addInlineSetting('WebLayout', 'moduleUrl', (string)$uriBuilder->buildUriFromRoute('web_layout'));
-
-        $this->css = '';
+        $this->pageRenderer->addInlineSetting('ContextHelp', 'moduleUrl', (string)$this->uriBuilder->buildUriFromRoute('help_cshmanual'));
+        $this->pageRenderer->addInlineSetting('ShowItem', 'moduleUrl', (string)$this->uriBuilder->buildUriFromRoute('show_item'));
+        $this->pageRenderer->addInlineSetting('RecordHistory', 'moduleUrl', (string)$this->uriBuilder->buildUriFromRoute('record_history'));
+        $this->pageRenderer->addInlineSetting('NewRecord', 'moduleUrl', (string)$this->uriBuilder->buildUriFromRoute('db_new'));
+        $this->pageRenderer->addInlineSetting('FormEngine', 'moduleUrl', (string)$this->uriBuilder->buildUriFromRoute('record_edit'));
+        $this->pageRenderer->addInlineSetting('RecordCommit', 'moduleUrl', (string)$this->uriBuilder->buildUriFromRoute('tce_db'));
+        $this->pageRenderer->addInlineSetting('FileCommit', 'moduleUrl', (string)$this->uriBuilder->buildUriFromRoute('tce_file'));
+        $this->pageRenderer->addInlineSetting('WebLayout', 'moduleUrl', (string)$this->uriBuilder->buildUriFromRoute('web_layout'));
 
         $this->initializeToolbarItems();
         $this->executeHook('constructPostProcess');
@@ -230,56 +176,37 @@ class BackendController
     }
 
     /**
-     * Injects the request object for the current request or subrequest
-     * As this controller goes only through the render() method, it is rather simple for now
+     * Main function generating the BE scaffolding
      *
      * @return ResponseInterface the response with the content
      */
     public function mainAction(): ResponseInterface
-    {
-        $this->render();
-        return new HtmlResponse($this->content);
-    }
-
-    /**
-     * Main function generating the BE scaffolding
-     */
-    protected function render()
     {
         $this->executeHook('renderPreProcess');
 
         // Prepare the scaffolding, at this point extension may still add javascript and css
-        $view = $this->getFluidTemplateObject($this->templatePath . 'Backend/Main.html');
-
+        $moduleTemplate = GeneralUtility::makeInstance(ModuleTemplate::class);
+        $view = $moduleTemplate->getView();
+        $view->setPartialRootPaths([GeneralUtility::getFileAbsFileName('EXT:backend/Resources/Private/Partials')]);
+        $view->setTemplatePathAndFilename(GeneralUtility::getFileAbsFileName($this->templatePath . 'Backend/Main.html'));
         $view->assign('moduleMenuCollapsed', $this->getCollapseStateOfMenu());
         $view->assign('moduleMenu', $this->generateModuleMenu());
         $view->assign('topbar', $this->renderTopbar());
 
-        /******************************************************
-         * Now put the complete backend document together
-         ******************************************************/
-        foreach ($this->cssFiles as $cssFileName => $cssFile) {
-            $this->pageRenderer->addCssFile($cssFile);
-            // Load additional css files to overwrite existing core styles
-            if (!empty($GLOBALS['TBE_STYLES']['stylesheets'][$cssFileName])) {
-                $this->pageRenderer->addCssFile($GLOBALS['TBE_STYLES']['stylesheets'][$cssFileName]);
-            }
-        }
         if (!empty($this->css)) {
             $this->pageRenderer->addCssInlineBlock('BackendInlineCSS', $this->css);
         }
-        foreach ($this->jsFiles as $jsFile) {
-            $this->pageRenderer->addJsFile($jsFile);
-        }
         $this->generateJavascript();
-        $this->pageRenderer->addJsInlineCode('BackendInlineJavascript', $this->js, false);
 
-        // Set document title:
-        $title = $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename'] ? $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename'] . ' [TYPO3 CMS ' . TYPO3_version . ']' : 'TYPO3 CMS ' . TYPO3_version;
-        // Renders the module page
-        $this->content = $this->getDocumentTemplate()->render($title, $view->render());
-        $hookConfiguration = ['content' => &$this->content];
-        $this->executeHook('renderPostProcess', $hookConfiguration);
+        // Set document title
+        $typo3Version = 'TYPO3 CMS ' . $this->typo3Version->getVersion();
+        $title = $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename'] ? $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename'] . ' [' . $typo3Version . ']' : $typo3Version;
+        $moduleTemplate->setTitle($title);
+
+        // Renders the backend scaffolding
+        $content = $moduleTemplate->renderContent();
+        $this->executeHook('renderPostProcess', ['content' => &$content]);
+        return new HtmlResponse($content);
     }
 
     /**
@@ -321,7 +248,7 @@ class BackendController
         $view->assign('logoUrl', PathUtility::getAbsoluteWebPath($logoPath));
         $view->assign('logoWidth', $logoWidth);
         $view->assign('logoHeight', $logoHeight);
-        $view->assign('applicationVersion', TYPO3_version);
+        $view->assign('applicationVersion', $this->typo3Version->getVersion());
         $view->assign('siteName', $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename']);
         $view->assign('toolbar', $this->renderToolbar());
 
@@ -337,7 +264,7 @@ class BackendController
     {
         $toolbar = [];
         foreach ($this->toolbarItems as $toolbarItem) {
-            /** @var \TYPO3\CMS\Backend\Toolbar\ToolbarItemInterface $toolbarItem */
+            /** @var ToolbarItemInterface $toolbarItem */
             if ($toolbarItem->checkAccess()) {
                 $hasDropDown = (bool)$toolbarItem->hasDropDown();
                 $additionalAttributes = (array)$toolbarItem->getAdditionalAttributes();
@@ -407,7 +334,7 @@ class BackendController
         if (!$beUser->check('modules', $pageModule)) {
             $pageModule = '';
         } else {
-            $pageModuleUrl = (string)GeneralUtility::makeInstance(UriBuilder::class)->buildUriFromRoute($pageModule);
+            $pageModuleUrl = (string)$this->uriBuilder->buildUriFromRoute($pageModule);
         }
         $t3Configuration = [
             'username' => htmlspecialchars($beUser->user['username']),
@@ -416,34 +343,38 @@ class BackendController
             'inWorkspace' => $beUser->workspace !== 0,
             'showRefreshLoginPopup' => (bool)($GLOBALS['TYPO3_CONF_VARS']['BE']['showRefreshLoginPopup'] ?? false)
         ];
-        $this->js .= '
-	TYPO3.configuration = ' . json_encode($t3Configuration) . ';
-	/**
-	 * Frameset Module object
-	 *
-	 * Used in main modules with a frameset for submodules to keep the ID between modules
-	 * Typically that is set by something like this in a Web>* sub module:
-	 *		if (top.fsMod) top.fsMod.recentIds["web"] = "\'.(int)$this->id.\'";
-	 * 		if (top.fsMod) top.fsMod.recentIds["file"] = "...(file reference/string)...";
-	 */
-	var fsMod = {
-		recentIds: [],					// used by frameset modules to track the most recent used id for list frame.
-		navFrameHighlightedID: [],		// used by navigation frames to track which row id was highlighted last time
-		currentBank: "0"
-	};
 
-	top.goToModule = function(modName, cMR_flag, addGetVars) {
-		TYPO3.ModuleMenu.App.showModule(modName, addGetVars);
-	}
-	' . $this->setStartupModule();
-        // Check editing of page:
-        $this->handlePageEditing();
+        $this->pageRenderer->addJsInlineCode(
+            'BackendConfiguration',
+            '
+        TYPO3.configuration = ' . json_encode($t3Configuration) . ';
+        /**
+         * Frameset Module object
+         *
+         * Used in main modules with a frameset for submodules to keep the ID between modules
+         * Typically that is set by something like this in a Web>* sub module:
+         *		if (top.fsMod) top.fsMod.recentIds["web"] = "\'.(int)$this->id.\'";
+         * 		if (top.fsMod) top.fsMod.recentIds["file"] = "...(file reference/string)...";
+         */
+        var fsMod = {
+            recentIds: [],					// used by frameset modules to track the most recent used id for list frame.
+            navFrameHighlightedID: [],		// used by navigation frames to track which row id was highlighted last time
+            currentBank: "0"
+        };
+
+        top.goToModule = function(modName, cMR_flag, addGetVars) {
+            TYPO3.ModuleMenu.App.showModule(modName, addGetVars);
+        }
+        ' . $this->setStartupModule()
+          . $this->handlePageEditing(),
+            false
+        );
     }
 
     /**
      * Checking if the "&edit" variable was sent so we can open it for editing the page.
      */
-    protected function handlePageEditing()
+    protected function handlePageEditing(): string
     {
         $beUser = $this->getBackendUser();
         $userTsConfig = $this->getBackendUser()->getTSConfig();
@@ -452,50 +383,25 @@ class BackendController
         if ($editId) {
             // Looking up the page to edit, checking permissions:
             $where = ' AND (' . $beUser->getPagePermsClause(Permission::PAGE_EDIT) . ' OR ' . $beUser->getPagePermsClause(Permission::CONTENT_EDIT) . ')';
+            $editRecord = null;
             if (MathUtility::canBeInterpretedAsInteger($editId)) {
                 $editRecord = BackendUtility::getRecordWSOL('pages', $editId, '*', $where);
-            } else {
-                $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');
-                $queryBuilder->getRestrictions()
-                    ->removeAll()
-                    ->add(GeneralUtility::makeInstance(DeletedRestriction::class))
-                    ->add(GeneralUtility::makeInstance(BackendWorkspaceRestriction::class));
-
-                $editRecord = $queryBuilder->select('*')
-                    ->from('pages')
-                    ->where(
-                        $queryBuilder->expr()->eq(
-                            'alias',
-                            $queryBuilder->createNamedParameter($editId, \PDO::PARAM_STR)
-                        ),
-                        $queryBuilder->expr()->orX(
-                            $beUser->getPagePermsClause(Permission::PAGE_EDIT),
-                            $beUser->getPagePermsClause(Permission::CONTENT_EDIT)
-                        )
-                    )
-                    ->setMaxResults(1)
-                    ->execute()
-                    ->fetch();
-
-                if ($editRecord !== false) {
-                    BackendUtility::workspaceOL('pages', $editRecord);
-                }
             }
             // If the page was accessible, then let the user edit it.
             if (is_array($editRecord) && $beUser->isInWebMount($editRecord)) {
-                // Setting JS code to open editing:
-                $this->js .= '
-		// Load page to edit:
-	window.setTimeout("top.loadEditId(' . (int)$editRecord['uid'] . ');", 500);
-			';
                 // Checking page edit parameter:
                 if (!($userTsConfig['options.']['bookmark_onEditId_dontSetPageTree'] ?? false)) {
                     $bookmarkKeepExpanded = (bool)($userTsConfig['options.']['bookmark_onEditId_keepExistingExpanded'] ?? false);
                     // Expanding page tree:
                     BackendUtility::openPageTree((int)$editRecord['pid'], !$bookmarkKeepExpanded);
                 }
-            } else {
-                $this->js .= '
+                // Setting JS code to open editing:
+                return '
+		// Load page to edit:
+	window.setTimeout("top.loadEditId(' . (int)$editRecord['uid'] . ');", 500);
+			';
+            }
+            return '
             // Warning about page editing:
             require(["TYPO3/CMS/Backend/Modal", "TYPO3/CMS/Backend/Severity"], function(Modal, Severity) {
                 Modal.show("", ' . GeneralUtility::quoteJSvalue(sprintf($this->getLanguageService()->getLL('noEditPage'), $editId)) . ', Severity.notice, [{
@@ -508,8 +414,8 @@ class BackendController
                     }
                 }])
             });';
-            }
         }
+        return '';
     }
 
     /**
@@ -535,7 +441,7 @@ class BackendController
             // check if the start module has additional parameters, so a redirect to a specific
             // action is possible
             if (strpos($startModule, '->') !== false) {
-                list($startModule, $startModuleParameters) = explode('->', $startModule, 2);
+                [$startModule, $startModuleParameters] = explode('->', $startModule, 2);
             }
         }
 
@@ -596,7 +502,7 @@ class BackendController
     protected function generateModuleMenu()
     {
         // get all modules except the user modules for the side menu
-        $moduleStorage = $this->backendModuleRepository->loadAllowedModules(['user', 'help']);
+        $moduleStorage = $this->backendModuleRepository->loadAllowedModules(['user', 'help', 'dashboard']);
 
         $view = $this->getFluidTemplateObject($this->templatePath . 'ModuleMenu/Main.html');
         $view->assign('modules', $moduleStorage);
@@ -650,7 +556,7 @@ class BackendController
     /**
      * Returns LanguageService
      *
-     * @return \TYPO3\CMS\Core\Localization\LanguageService
+     * @return LanguageService
      */
     protected function getLanguageService()
     {
@@ -660,20 +566,10 @@ class BackendController
     /**
      * Returns the current BE user.
      *
-     * @return \TYPO3\CMS\Core\Authentication\BackendUserAuthentication
+     * @return BackendUserAuthentication
      */
     protected function getBackendUser()
     {
         return $GLOBALS['BE_USER'];
     }
-
-    /**
-     * Returns an instance of DocumentTemplate
-     *
-     * @return \TYPO3\CMS\Backend\Template\DocumentTemplate
-     */
-    protected function getDocumentTemplate()
-    {
-        return $GLOBALS['TBE_TEMPLATE'];
-    }
 }
