@@ -17,20 +17,18 @@ namespace TYPO3\CMS\Backend\Controller\ContentElement;
 
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
-use TYPO3\CMS\Backend\Template\DocumentTemplate;
+use TYPO3\CMS\Backend\Routing\UriBuilder;
 use TYPO3\CMS\Backend\Template\ModuleTemplate;
 use TYPO3\CMS\Backend\Tree\View\ContentCreationPagePositionMap;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Backend\View\BackendLayoutView;
 use TYPO3\CMS\Backend\Wizard\NewContentElementWizardHookInterface;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
-use TYPO3\CMS\Core\Compatibility\PublicPropertyDeprecationTrait;
 use TYPO3\CMS\Core\Http\HtmlResponse;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Service\DependencyOrderingService;
 use TYPO3\CMS\Core\Type\Bitmask\Permission;
-use TYPO3\CMS\Core\Utility\ArrayUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\HttpUtility;
 use TYPO3\CMS\Fluid\View\StandaloneView;
@@ -41,24 +39,6 @@ use TYPO3\CMS\Fluid\View\StandaloneView;
  */
 class NewContentElementController
 {
-    use PublicPropertyDeprecationTrait;
-
-    /**
-     * @var array
-     */
-    protected $deprecatedPublicProperties = [
-        'id' => 'Using $id of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-        'sys_language' => 'Using $sys_language of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-        'R_URI' => 'Using $R_URI of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-        'colPos' => 'Using $colPos of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-        'uid_pid' => 'Using $uid_pid of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-        'modTSconfig' => 'Using $modTSconfig of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-        'doc' => 'Using $doc of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-        'content' => 'Using $content of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-        'access' => 'Using $access of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-        'config' => 'Using $config of NewContentElementController from the outside is discouraged as this variable is used for internal storage.',
-    ];
-
     /**
      * Page id
      *
@@ -92,34 +72,6 @@ class NewContentElementController
      */
     protected $uid_pid;
 
-    /**
-     * Module TSconfig.
-     *
-     * @var array
-     */
-    protected $modTSconfig = [];
-
-    /**
-     * Internal backend template object
-     *
-     * @var DocumentTemplate
-     */
-    protected $doc;
-
-    /**
-     * Used to accumulate the content of the module.
-     *
-     * @var string
-     */
-    protected $content;
-
-    /**
-     * Access boolean.
-     *
-     * @var bool
-     */
-    protected $access;
-
     /**
      * config of the wizard
      *
@@ -132,16 +84,6 @@ class NewContentElementController
      */
     protected $pageInfo;
 
-    /**
-     * @var string
-     */
-    protected $onClickEvent;
-
-    /**
-     * @var array
-     */
-    protected $MCONF;
-
     /**
      * @var StandaloneView
      */
@@ -159,6 +101,11 @@ class NewContentElementController
      */
     protected $moduleTemplate;
 
+    /**
+     * @var UriBuilder
+     */
+    protected $uriBuilder;
+
     /**
      * Constructor
      */
@@ -168,32 +115,19 @@ class NewContentElementController
         $GLOBALS['SOBE'] = $this;
         $this->view = $this->getFluidTemplateObject();
         $this->menuItemView = $this->getFluidTemplateObject('MenuItem.html');
-
-        // @deprecated since TYPO3 v9, will be obsolete in TYPO3 v10.0 with removal of init()
-        $request = $GLOBALS['TYPO3_REQUEST'];
-        // @deprecated since TYPO3 v9, will be moved out of __construct() in TYPO3 v10.0
-        $this->init($request);
+        $this->uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
     }
 
     /**
      * Constructor, initializing internal variables.
      *
-     * @param ServerRequestInterface|null $request
+     * @param ServerRequestInterface $request
      */
-    public function init(ServerRequestInterface $request = null)
+    protected function init(ServerRequestInterface $request)
     {
-        if ($request === null) {
-            // Method signature in TYPO3 v10.0: protected function init(ServerRequestInterface $request)
-            trigger_error('NewContentElementController->init() will be set to protected in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-            $request = $GLOBALS['TYPO3_REQUEST'];
-        }
-
         $lang = $this->getLanguageService();
         $lang->includeLLFile('EXT:core/Resources/Private/Language/locallang_misc.xlf');
-        $LOCAL_LANG_orig = $GLOBALS['LOCAL_LANG'];
         $lang->includeLLFile('EXT:backend/Resources/Private/Language/locallang_db_new_content_el.xlf');
-        ArrayUtility::mergeRecursiveWithOverrule($LOCAL_LANG_orig, $GLOBALS['LOCAL_LANG']);
-        $GLOBALS['LOCAL_LANG'] = $LOCAL_LANG_orig;
 
         $parsedBody = $request->getParsedBody();
         $queryParams = $request->getQueryParams();
@@ -205,19 +139,11 @@ class NewContentElementController
         $colPos = $parsedBody['colPos'] ?? $queryParams['colPos'] ?? null;
         $this->colPos = $colPos === null ? null : (int)$colPos;
         $this->uid_pid = (int)($parsedBody['uid_pid'] ?? $queryParams['uid_pid'] ?? 0);
-        $this->MCONF['name'] = 'xMOD_db_new_content_el';
-        $this->modTSconfig['properties'] = BackendUtility::getPagesTSconfig($this->id)['mod.']['wizards.']['newContentElement.'] ?? [];
-        $config = BackendUtility::getPagesTSconfig($this->id);
-        $this->config = $config['mod.']['wizards.']['newContentElement.'];
-        // Starting the document template object:
-        // We keep this here in case somebody relies on it in a hook or alike
-        $this->doc = GeneralUtility::makeInstance(DocumentTemplate::class);
+        $this->config = BackendUtility::getPagesTSconfig($this->id)['mod.']['wizards.']['newContentElement.']['wizardItems.'] ?? [];
         // Setting up the context sensitive menu:
         $this->moduleTemplate->getPageRenderer()->loadRequireJsModule('TYPO3/CMS/Backend/ContextMenu');
         // Getting the current page and receiving access information (used in main())
-        $perms_clause = $this->getBackendUser()->getPagePermsClause(Permission::PAGE_SHOW);
-        $this->pageInfo = BackendUtility::readPageAccess($this->id, $perms_clause);
-        $this->access = is_array($this->pageInfo);
+        $this->pageInfo = BackendUtility::readPageAccess($this->id, $this->getBackendUser()->getPagePermsClause(Permission::PAGE_SHOW));
     }
 
     /**
@@ -229,8 +155,9 @@ class NewContentElementController
      */
     public function mainAction(ServerRequestInterface $request): ResponseInterface
     {
+        $this->init($request);
         $this->prepareContent('window');
-        $this->moduleTemplate->setContent($this->content);
+        $this->moduleTemplate->setContent($this->view->render());
         return new HtmlResponse($this->moduleTemplate->renderContent());
     }
 
@@ -243,78 +170,27 @@ class NewContentElementController
      */
     public function wizardAction(ServerRequestInterface $request): ResponseInterface
     {
+        $this->init($request);
         $this->prepareContent('list_frame');
-        return new HtmlResponse($this->content);
+        return new HtmlResponse($this->view->render());
     }
 
     /**
-     * Creating the module output.
+     * Create on-click event value.
      *
-     * @deprecated since TYPO3 v9, will be removed in TYPO3v10 without substitute
+     * @param string $clientContext
+     * @return string
      */
-    public function main()
+    protected function onClickInsertRecord(string $clientContext): string
     {
-        trigger_error('NewContentElementController->main() will be replaced by protected method prepareContent() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        $this->prepareContent('window');
-    }
-
-    /**
-     * Returns the array of elements in the wizard display.
-     * For the plugin section there is support for adding elements there from a global variable.
-     *
-     * @return array
-     */
-    public function wizardArray()
-    {
-        trigger_error('NewContentElementController->wizardArray() will be replaced by protected method getWizards() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        return $this->getWizards();
-    }
-
-    /**
-     * @param mixed $wizardElements
-     * @return array
-     */
-    public function wizard_appendWizards($wizardElements)
-    {
-        trigger_error('NewContentElementController->wizard_appendWizards() will be replaced by protected method getAppendWizards() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        return $this->getAppendWizards($wizardElements);
-    }
-
-    /**
-     * @param string $groupKey Not used
-     * @param string $itemKey Not used
-     * @param array $itemConf
-     * @return array
-     */
-    public function wizard_getItem($groupKey, $itemKey, $itemConf)
-    {
-        trigger_error('NewContentElementController->wizard_getItem() will be replaced by protected method getWizardItem() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        return $this->getWizardItem($groupKey, $itemKey, $itemConf);
-    }
-
-    /**
-     * @param string $groupKey Not used
-     * @param array $wizardGroup
-     * @return array
-     */
-    public function wizard_getGroupHeader($groupKey, $wizardGroup)
-    {
-        trigger_error('NewContentElementController->wizard_getGroupHeader() will be replaced by protected method getWizardGroupHeader() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        return $this->getWizardGroupHeader($wizardGroup);
-    }
-
-    /**
-     * Checks the array for elements which might contain unallowed default values and will unset them!
-     * Looks for the "tt_content_defValues" key in each element and if found it will traverse that array as fieldname /
-     * value pairs and check.
-     * The values will be added to the "params" key of the array (which should probably be unset or empty by default).
-     *
-     * @param array $wizardItems Wizard items, passed by reference
-     */
-    public function removeInvalidElements(&$wizardItems)
-    {
-        trigger_error('NewContentElementController->removeInvalidElements() will be replaced by protected method removeInvalidWizardItems() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        $this->removeInvalidWizardItems($wizardItems);
+        // $this->uid_pid can be negative (= pointing to tt_content record) or positive (= "page ID")
+        $location = (string)$this->uriBuilder->buildUriFromRoute('record_edit', [
+            'edit[tt_content][' . $this->uid_pid . ']' => 'new',
+            'defVals[tt_content][colPos]' => $this->colPos,
+            'defVals[tt_content][sys_language_uid]' => $this->sys_language,
+            'returnUrl' => GeneralUtility::_GP('returnUrl')
+        ]);
+        return $clientContext . '.location.href=' . GeneralUtility::quoteJSvalue($location) . '+document.editForm.defValues.value; return false;';
     }
 
     /**
@@ -323,38 +199,19 @@ class NewContentElementController
      * @param string $clientContext JavaScript client context to be used
      *        + 'window', legacy if rendered in current document
      *        + 'list_frame', in case rendered in global modal
-     *
      * @throws \UnexpectedValueException
      */
     protected function prepareContent(string $clientContext): void
     {
-        $hasAccess = true;
-        if ($this->id && $this->access) {
-
-            // Init position map object:
-            $posMap = GeneralUtility::makeInstance(
-                ContentCreationPagePositionMap::class,
-                null,
-                $clientContext
-            );
-            $posMap->cur_sys_language = $this->sys_language;
-            // If a column is pre-set:
+        // Setting up the buttons for docheader
+        $this->getButtons();
+        $hasAccess = $this->id && is_array($this->pageInfo);
+        if ($hasAccess) {
+            // If a column is pre-set
             if (isset($this->colPos)) {
-                if ($this->uid_pid < 0) {
-                    $row = [];
-                    $row['uid'] = abs($this->uid_pid);
-                } else {
-                    $row = '';
-                }
-                $this->onClickEvent = $posMap->onClickInsertRecord(
-                    $row,
-                    $this->colPos,
-                    '',
-                    $this->uid_pid,
-                    $this->sys_language
-                );
+                $onClickEvent = $this->onClickInsertRecord($clientContext);
             } else {
-                $this->onClickEvent = '';
+                $onClickEvent = '';
             }
             // ***************************
             // Creating content
@@ -380,8 +237,8 @@ class NewContentElementController
             $menuItems = [];
 
             $this->view->assignMultiple([
-                'hasClickEvent' => $this->onClickEvent !== '',
-                'onClickEvent' => 'function goToalt_doc() { ' . $this->onClickEvent . '}',
+                'hasClickEvent' => $onClickEvent !== '',
+                'onClickEvent' => 'function goToalt_doc() { ' . $onClickEvent . '}',
             ]);
 
             foreach ($wizardItems as $wizardKey => $wInfo) {
@@ -393,7 +250,7 @@ class NewContentElementController
                     ];
                     $key = count($menuItems) - 1;
                 } else {
-                    if (!$this->onClickEvent) {
+                    if (!$onClickEvent) {
                         // Radio button:
                         $wizardOnClick = 'document.editForm.defValues.value=unescape(' . GeneralUtility::quoteJSvalue(rawurlencode($wInfo['params'])) . '); window.location.hash=\'#sel2\';';
                         // Onclick action for icon/title:
@@ -402,10 +259,24 @@ class NewContentElementController
                         $aOnClick = "document.editForm.defValues.value=unescape('" . rawurlencode($wInfo['params']) . "');goToalt_doc();";
                     }
 
+                    // Go to DataHandler directly instead of FormEngine
+                    if ($wInfo['saveAndClose'] ?? false) {
+                        $urlParams = [];
+                        $id = uniqid('NEW');
+                        parse_str($wInfo['params'], $urlParams);
+                        $urlParams['data']['tt_content'][$id] = $urlParams['defVals']['tt_content'] ?? [];
+                        $urlParams['data']['tt_content'][$id]['colPos'] = $this->colPos;
+                        $urlParams['data']['tt_content'][$id]['pid'] = $this->uid_pid;
+                        $urlParams['data']['tt_content'][$id]['sys_language_uid'] = $this->sys_language;
+                        $urlParams['redirect'] = GeneralUtility::_GP('returnUrl');
+                        unset($urlParams['defVals']);
+                        $url = $this->uriBuilder->buildUriFromRoute('tce_db', $urlParams);
+                        $aOnClick = 'list_frame.location.href=' . GeneralUtility::quoteJSvalue((string)$url) . '; return false';
+                    }
                     $icon = $this->moduleTemplate->getIconFactory()->getIcon($wInfo['iconIdentifier'])->render();
 
                     $this->menuItemView->assignMultiple([
-                        'onClickEvent' => $this->onClickEvent,
+                        'onClickEvent' => $onClickEvent,
                         'aOnClick' => $aOnClick,
                         'wizardInformation' => $wInfo,
                         'icon' => $icon,
@@ -423,29 +294,35 @@ class NewContentElementController
             ));
 
             // If the user must also select a column:
-            if (!$this->onClickEvent) {
-
-                // Load SHARED page-TSconfig settings and retrieve column list from there, if applicable:
-                $colPosArray = GeneralUtility::callUserFunction(
-                    BackendLayoutView::class . '->getColPosListItemsParsed',
-                    $this->id,
-                    $this
-                );
-                $colPosIds = array_column($colPosArray, 1);
-                // Removing duplicates, if any
-                $colPosList = implode(',', array_unique(array_map('intval', $colPosIds)));
-                // Finally, add the content of the column selector to the content:
-                $this->view->assign('posMap', $posMap->printContentElementColumns($this->id, 0, $colPosList, 1, $this->R_URI));
+            if (!$onClickEvent) {
+                $this->definePositionMapEntries($clientContext);
             }
-        } else {
-            // In case of no access:
-            $hasAccess = false;
         }
         $this->view->assign('hasAccess', $hasAccess);
+    }
 
-        $this->content = $this->view->render();
-        // Setting up the buttons and markers for docheader
-        $this->getButtons();
+    /**
+     * User must select a column as well (when in "main mode"), so the position map is initialized and assigned to
+     * the view.
+     *
+     * @param string $clientContext
+     */
+    protected function definePositionMapEntries(string $clientContext): void
+    {
+        // Load SHARED page-TSconfig settings and retrieve column list from there, if applicable:
+        $colPosArray = GeneralUtility::makeInstance(BackendLayoutView::class)->getColPosListItemsParsed((int)$this->id);
+        $colPosIds = array_column($colPosArray, 1);
+        // Removing duplicates, if any
+        $colPosList = implode(',', array_unique(array_map('intval', $colPosIds)));
+        // Finally, add the content of the column selector to the content:
+        // Init position map object
+        $posMap = GeneralUtility::makeInstance(
+            ContentCreationPagePositionMap::class,
+            null,
+            $clientContext
+        );
+        $posMap->cur_sys_language = $this->sys_language;
+        $this->view->assign('posMap', $posMap->printContentElementColumns($this->id, 0, $colPosList, 1, $this->R_URI));
     }
 
     /**
@@ -477,41 +354,39 @@ class NewContentElementController
     protected function getWizards(): array
     {
         $wizardItems = [];
-        if (is_array($this->config)) {
-            $wizards = $this->config['wizardItems.'] ?? [];
-            $appendWizards = $this->getAppendWizards($wizards['elements.'] ?? []);
-            if (is_array($wizards)) {
-                foreach ($wizards as $groupKey => $wizardGroup) {
-                    $this->prepareDependencyOrdering($wizards[$groupKey], 'before');
-                    $this->prepareDependencyOrdering($wizards[$groupKey], 'after');
-                }
-                $wizards = GeneralUtility::makeInstance(DependencyOrderingService::class)->orderByDependencies($wizards);
+        $wizards = $this->config;
+        $appendWizards = $this->getAppendWizards($wizards['elements.'] ?? []);
+        if (is_array($wizards)) {
+            foreach ($wizards as $groupKey => $wizardGroup) {
+                $this->prepareDependencyOrdering($wizards[$groupKey], 'before');
+                $this->prepareDependencyOrdering($wizards[$groupKey], 'after');
+            }
+            $wizards = GeneralUtility::makeInstance(DependencyOrderingService::class)->orderByDependencies($wizards);
 
-                foreach ($wizards as $groupKey => $wizardGroup) {
-                    $groupKey = rtrim($groupKey, '.');
-                    $showItems = GeneralUtility::trimExplode(',', $wizardGroup['show'], true);
-                    $showAll = in_array('*', $showItems, true);
-                    $groupItems = [];
-                    if (is_array($appendWizards[$groupKey . '.']['elements.'])) {
-                        $wizardElements = array_merge((array)$wizardGroup['elements.'], $appendWizards[$groupKey . '.']['elements.']);
-                    } else {
-                        $wizardElements = $wizardGroup['elements.'];
-                    }
-                    if (is_array($wizardElements)) {
-                        foreach ($wizardElements as $itemKey => $itemConf) {
-                            $itemKey = rtrim($itemKey, '.');
-                            if ($showAll || in_array($itemKey, $showItems)) {
-                                $tmpItem = $this->getWizardItem($groupKey, $itemKey, $itemConf);
-                                if ($tmpItem) {
-                                    $groupItems[$groupKey . '_' . $itemKey] = $tmpItem;
-                                }
+            foreach ($wizards as $groupKey => $wizardGroup) {
+                $groupKey = rtrim($groupKey, '.');
+                $showItems = GeneralUtility::trimExplode(',', $wizardGroup['show'], true);
+                $showAll = in_array('*', $showItems, true);
+                $groupItems = [];
+                if (is_array($appendWizards[$groupKey . '.']['elements.'])) {
+                    $wizardElements = array_merge((array)$wizardGroup['elements.'], $appendWizards[$groupKey . '.']['elements.']);
+                } else {
+                    $wizardElements = $wizardGroup['elements.'];
+                }
+                if (is_array($wizardElements)) {
+                    foreach ($wizardElements as $itemKey => $itemConf) {
+                        $itemKey = rtrim($itemKey, '.');
+                        if ($showAll || in_array($itemKey, $showItems)) {
+                            $tmpItem = $this->getWizardItem($itemConf);
+                            if ($tmpItem) {
+                                $groupItems[$groupKey . '_' . $itemKey] = $tmpItem;
                             }
                         }
                     }
-                    if (!empty($groupItems)) {
-                        $wizardItems[$groupKey] = $this->getWizardGroupHeader($wizardGroup);
-                        $wizardItems = array_merge($wizardItems, $groupItems);
-                    }
+                }
+                if (!empty($groupItems)) {
+                    $wizardItems[$groupKey] = $this->getWizardGroupHeader($wizardGroup);
+                    $wizardItems = array_merge($wizardItems, $groupItems);
                 }
             }
         }
@@ -526,9 +401,6 @@ class NewContentElementController
      */
     protected function getAppendWizards(array $wizardElements): array
     {
-        if (!is_array($wizardElements)) {
-            $wizardElements = [];
-        }
         if (is_array($GLOBALS['TBE_MODULES_EXT']['xMOD_db_new_content_el']['addElClasses'])) {
             foreach ($GLOBALS['TBE_MODULES_EXT']['xMOD_db_new_content_el']['addElClasses'] as $class => $path) {
                 if (!class_exists($class) && file_exists($path)) {
@@ -550,15 +422,14 @@ class NewContentElementController
     }
 
     /**
-     * @param string $groupKey Not used
-     * @param string $itemKey Not used
      * @param array $itemConf
      * @return array
      */
-    protected function getWizardItem(string $groupKey, string $itemKey, array $itemConf): array
+    protected function getWizardItem(array $itemConf): array
     {
         $itemConf['title'] = $this->getLanguageService()->sL($itemConf['title']);
         $itemConf['description'] = $this->getLanguageService()->sL($itemConf['description']);
+        $itemConf['saveAndClose'] = (bool)$itemConf['saveAndClose'];
         $itemConf['tt_content_defValues'] = $itemConf['tt_content_defValues.'];
         unset($itemConf['tt_content_defValues.']);
         return $itemConf;
@@ -585,9 +456,10 @@ class NewContentElementController
      */
     protected function removeInvalidWizardItems(array &$wizardItems): void
     {
+        $removeItems = [];
+        $keepItems = [];
         // Get TCEFORM from TSconfig of current page
-        $row = ['pid' => $this->id];
-        $TCEFORM_TSconfig = BackendUtility::getTCEFORM_TSconfig('tt_content', $row);
+        $TCEFORM_TSconfig = BackendUtility::getTCEFORM_TSconfig('tt_content', ['pid' => $this->id]);
         $headersUsed = [];
         // Traverse wizard items:
         foreach ($wizardItems as $key => $cfg) {
@@ -695,7 +567,6 @@ class NewContentElementController
      */
     protected function getFluidTemplateObject(string $filename = 'Main.html'): StandaloneView
     {
-        /** @var StandaloneView $view */
         $view = GeneralUtility::makeInstance(StandaloneView::class);
         $view->setTemplatePathAndFilename(GeneralUtility::getFileAbsFileName('EXT:backend/Resources/Private/Templates/NewContentElement/' . $filename));
         $view->getRequest()->setControllerExtensionName('Backend');
