@@ -15,8 +15,9 @@ namespace TYPO3\CMS\Backend\Controller;
  */
 
 use Psr\Http\Message\ResponseInterface;
-use TYPO3\CMS\Backend\Template\DocumentTemplate;
+use TYPO3\CMS\Backend\Template\ModuleTemplate;
 use TYPO3\CMS\Core\Http\HtmlResponse;
+use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
  * '/empty' routing target returns dummy content.
@@ -31,18 +32,9 @@ class DummyController
      */
     public function mainAction(): ResponseInterface
     {
-        $documentTemplate = $this->getDocumentTemplate();
-        $content = $documentTemplate->startPage('Dummy document') . $documentTemplate->endPage();
-        return new HtmlResponse($content);
-    }
-
-    /**
-     * Returns an instance of DocumentTemplate
-     *
-     * @return DocumentTemplate
-     */
-    protected function getDocumentTemplate()
-    {
-        return $GLOBALS['TBE_TEMPLATE'];
+        $moduleTemplate = GeneralUtility::makeInstance(ModuleTemplate::class);
+        $moduleTemplate->setTitle('Blank');
+        $moduleTemplate->getDocHeaderComponent()->disable();
+        return new HtmlResponse($moduleTemplate->renderContent());
     }
 }
