@@ -143,30 +143,14 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
             'data' => '',
             'stylesheetFiles' => [],
             'scriptCall' => [],
+            'compilerInput' => [
+                'uid' => $childData['databaseRow']['uid'],
+                'childChildUid' => $childChildUid,
+                'parentConfig' => $parentConfig,
+            ],
         ];
 
-        // The HTML-object-id's prefix of the dynamically created record
-        $objectName = $inlineStackProcessor->getCurrentStructureDomObjectIdPrefix($inlineFirstPid);
-        $objectPrefix = $objectName . '-' . $child['table'];
-        $objectId = $objectPrefix . '-' . $childData['databaseRow']['uid'];
-        $expandSingle = $parentConfig['appearance']['expandSingle'];
-        if (!$child['uid']) {
-            $jsonArray['scriptCall'][] = 'inline.domAddNewRecord(\'bottom\',' . GeneralUtility::quoteJSvalue($objectName . '_records') . ',' . GeneralUtility::quoteJSvalue($objectPrefix) . ',json.data);';
-            $jsonArray['scriptCall'][] = 'inline.memorizeAddRecord(' . GeneralUtility::quoteJSvalue($objectPrefix) . ',' . GeneralUtility::quoteJSvalue($childData['databaseRow']['uid']) . ',null,' . GeneralUtility::quoteJSvalue($childChildUid) . ');';
-        } else {
-            $jsonArray['scriptCall'][] = 'inline.domAddNewRecord(\'after\',' . GeneralUtility::quoteJSvalue($domObjectId . '_div') . ',' . GeneralUtility::quoteJSvalue($objectPrefix) . ',json.data);';
-            $jsonArray['scriptCall'][] = 'inline.memorizeAddRecord(' . GeneralUtility::quoteJSvalue($objectPrefix) . ',' . GeneralUtility::quoteJSvalue($childData['databaseRow']['uid']) . ',' . GeneralUtility::quoteJSvalue($child['uid']) . ',' . GeneralUtility::quoteJSvalue($childChildUid) . ');';
-        }
         $jsonArray = $this->mergeChildResultIntoJsonResult($jsonArray, $childResult);
-        if ($parentConfig['appearance']['useSortable']) {
-            $inlineObjectName = $inlineStackProcessor->getCurrentStructureDomObjectIdPrefix($inlineFirstPid);
-            $jsonArray['scriptCall'][] = 'inline.createDragAndDropSorting(' . GeneralUtility::quoteJSvalue($inlineObjectName . '_records') . ');';
-        }
-        if (!$parentConfig['appearance']['collapseAll'] && $expandSingle) {
-            $jsonArray['scriptCall'][] = 'inline.collapseAllRecords(' . GeneralUtility::quoteJSvalue($objectId) . ',' . GeneralUtility::quoteJSvalue($objectPrefix) . ',' . GeneralUtility::quoteJSvalue($childData['databaseRow']['uid']) . ');';
-        }
-        // Fade out and fade in the new record in the browser view to catch the user's eye
-        $jsonArray['scriptCall'][] = 'inline.fadeOutFadeIn(' . GeneralUtility::quoteJSvalue($objectId . '_div') . ');';
 
         return new JsonResponse($jsonArray);
     }
@@ -232,22 +216,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
             'scriptCall' => [],
         ];
 
-        // The HTML-object-id's prefix of the dynamically created record
-        $objectPrefix = $inlineStackProcessor->getCurrentStructureDomObjectIdPrefix($inlineFirstPid) . '-' . $child['table'];
-        $objectId = $objectPrefix . '-' . (int)$child['uid'];
-        $expandSingle = $parentConfig['appearance']['expandSingle'];
-        $jsonArray['scriptCall'][] = 'inline.domAddRecordDetails(' . GeneralUtility::quoteJSvalue($domObjectId) . ',' . GeneralUtility::quoteJSvalue($objectPrefix) . ',' . ($expandSingle ? '1' : '0') . ',json.data);';
-        if ($parentConfig['foreign_unique']) {
-            $jsonArray['scriptCall'][] = 'inline.removeUsed(' . GeneralUtility::quoteJSvalue($objectPrefix) . ',\'' . (int)$child['uid'] . '\');';
-        }
         $jsonArray = $this->mergeChildResultIntoJsonResult($jsonArray, $childResult);
-        if ($parentConfig['appearance']['useSortable']) {
-            $inlineObjectName = $inlineStackProcessor->getCurrentStructureDomObjectIdPrefix($inlineFirstPid);
-            $jsonArray['scriptCall'][] = 'inline.createDragAndDropSorting(' . GeneralUtility::quoteJSvalue($inlineObjectName . '_records') . ');';
-        }
-        if (!$parentConfig['appearance']['collapseAll'] && $expandSingle) {
-            $jsonArray['scriptCall'][] = 'inline.collapseAllRecords(' . GeneralUtility::quoteJSvalue($objectId) . ',' . GeneralUtility::quoteJSvalue($objectPrefix) . ',\'' . (int)$child['uid'] . '\');';
-        }
 
         return new JsonResponse($jsonArray);
     }
@@ -276,7 +245,9 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         $jsonArray = [
             'data' => '',
             'stylesheetFiles' => [],
-            'scriptCall' => [],
+            'compilerInput' => [
+                'localize' => [],
+            ],
         ];
         if ($type === 'localize' || $type === 'synchronize' || MathUtility::canBeInterpretedAsInteger($type)) {
             // Parent, this table embeds the child table
@@ -286,9 +257,6 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
             $processedTca = $GLOBALS['TCA'][$parent['table']];
             $processedTca['columns'][$parentFieldName]['config'] = $parentConfig;
 
-            // Child, a record from this table should be rendered
-            $child = $inlineStackProcessor->getUnstableStructure();
-
             $formDataCompilerInputForParent = [
                 'vanillaUid' => (int)$parent['uid'],
                 'command' => 'edit',
@@ -343,9 +311,6 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
 
             $newItemList = $tce->registerDBList[$parent['table']][$parent['uid']][$parentFieldName];
 
-            $nameObject = $inlineStackProcessor->getCurrentStructureDomObjectIdPrefix($inlineFirstPid);
-            $nameObjectForeignTable = $nameObject . '-' . $child['table'];
-
             $oldItems = $this->getInlineRelatedRecordsUidArray($oldItemList);
             $newItems = $this->getInlineRelatedRecordsUidArray($newItemList);
 
@@ -368,12 +333,10 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
 
             // Set the items that should be removed in the forms view:
             $removedItems = array_diff($oldItems, $newItems);
-            foreach ($removedItems as $childUid) {
-                $jsonArray['scriptCall'][] = 'inline.deleteRecord(' . GeneralUtility::quoteJSvalue($nameObjectForeignTable . '-' . $childUid) . ', {forceDirectRemoval: true});';
-            }
+            $jsonArray['compilerInput']['delete'] = $removedItems;
 
             $localizedItems = array_diff($newItems, $oldItems);
-            foreach ($localizedItems as $childUid) {
+            foreach ($localizedItems as $i => $childUid) {
                 $childData = $this->compileChild($parentData, $parentFieldName, (int)$childUid, $inlineStackProcessor->getStructure());
 
                 $childData['inlineParentUid'] = (int)$parent['uid'];
@@ -385,27 +348,26 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
 
                 // Get the name of the field used as foreign selector (if any):
                 $foreignSelector = isset($parentConfig['foreign_selector']) && $parentConfig['foreign_selector'] ? $parentConfig['foreign_selector'] : false;
-                $selectedValue = $foreignSelector ? GeneralUtility::quoteJSvalue($childData['databaseRow'][$foreignSelector]) : 'null';
+                $selectedValue = $foreignSelector ? GeneralUtility::quoteJSvalue($childData['databaseRow'][$foreignSelector]) : null;
                 if (is_array($selectedValue)) {
                     $selectedValue = $selectedValue[0];
                 }
-                $jsonArray['scriptCall'][] = 'inline.memorizeAddRecord(' . GeneralUtility::quoteJSvalue($nameObjectForeignTable) . ', ' . GeneralUtility::quoteJSvalue($childUid) . ', null, ' . $selectedValue . ');';
+
+                $jsonArray['compilerInput']['localize'][$i] = [
+                    'uid' => $childUid,
+                    'selectedValue' => $selectedValue,
+                ];
+
                 // Remove possible virtual records in the form which showed that a child records could be localized:
                 $transOrigPointerFieldName = $childData['processedTca']['ctrl']['transOrigPointerField'];
                 if (isset($childData['databaseRow'][$transOrigPointerFieldName]) && $childData['databaseRow'][$transOrigPointerFieldName]) {
-                    $transOrigPointerField = $childData['databaseRow'][$transOrigPointerFieldName];
-                    if (is_array($transOrigPointerField)) {
-                        $transOrigPointerField = $transOrigPointerField[0];
+                    $transOrigPointerFieldValue = $childData['databaseRow'][$transOrigPointerFieldName];
+                    if (is_array($transOrigPointerFieldValue)) {
+                        $transOrigPointerFieldValue = $transOrigPointerFieldValue[0];
                     }
-                    $jsonArray['scriptCall'][] = 'inline.fadeAndRemove(' . GeneralUtility::quoteJSvalue($nameObjectForeignTable . '-' . $transOrigPointerField . '_div') . ');';
+                    $jsonArray['compilerInput']['localize'][$i]['remove'] = $transOrigPointerFieldValue;
                 }
             }
-            // Tell JS to add new HTML of one or multiple (localize all) records to DOM
-            if (!empty($jsonArray['data'])) {
-                $jsonArray['scriptCall'][] = 'inline.domAddNewRecord(\'bottom\', ' . GeneralUtility::quoteJSvalue($nameObject . '_records')
-                . ', ' . GeneralUtility::quoteJSvalue($nameObjectForeignTable)
-                . ', json.data);';
-            }
         }
         return new JsonResponse($jsonArray);
     }
@@ -419,14 +381,12 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
     public function expandOrCollapseAction(ServerRequestInterface $request): ResponseInterface
     {
         $ajaxArguments = $request->getParsedBody()['ajax'] ?? $request->getQueryParams()['ajax'];
-        $domObjectId = $ajaxArguments[0];
+        [$domObjectId, $expand, $collapse] = $ajaxArguments;
 
         /** @var InlineStackProcessor $inlineStackProcessor */
         $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
         // Parse the DOM identifier (string), add the levels to the structure stack (array), don't load TCA config
         $inlineStackProcessor->initializeByParsingDomObjectIdString($domObjectId);
-        $expand = $ajaxArguments[1];
-        $collapse = $ajaxArguments[2];
 
         $backendUser = $this->getBackendUserAuthentication();
         // The current table - for this table we should add/import records
@@ -452,7 +412,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
             // Save states back to database
             if (is_array($inlineView[$topTable][$topUid][$currentTable])) {
                 $inlineView[$topTable][$topUid][$currentTable] = array_unique($inlineView[$topTable][$topUid][$currentTable]);
-                $backendUser->uc['inlineView'] = serialize($inlineView);
+                $backendUser->uc['inlineView'] = json_encode($inlineView);
                 $backendUser->writeUC();
             }
         }
@@ -573,12 +533,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
             $jsonResult['stylesheetFiles'][] = $this->getRelativePathToStylesheetFile($stylesheetFile);
         }
         if (!empty($childResult['inlineData'])) {
-            $jsonResult['scriptCall'][] = 'inline.addToDataArray(' . json_encode($childResult['inlineData']) . ');';
-        }
-        if (!empty($childResult['additionalJavaScriptSubmit'])) {
-            $additionalJavaScriptSubmit = implode('', $childResult['additionalJavaScriptSubmit']);
-            $additionalJavaScriptSubmit = str_replace([CR, LF], '', $additionalJavaScriptSubmit);
-            $jsonResult['scriptCall'][] = 'TBE_EDITOR.addActionChecks("submit", "' . addslashes($additionalJavaScriptSubmit) . '");';
+            $jsonResult['inlineData'] = $childResult['inlineData'];
         }
         foreach ($childResult['additionalJavaScriptPost'] as $singleAdditionalJavaScriptPost) {
             $jsonResult['scriptCall'][] = $singleAdditionalJavaScriptPost;
@@ -604,8 +559,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
 
             $jsonResult['scriptCall'][] = implode(LF, $javaScriptCode);
         }
-        $requireJsModule = $this->createExecutableStringRepresentationOfRegisteredRequireJsModules($childResult);
-        $jsonResult['scriptCall'] = array_merge($requireJsModule, $jsonResult['scriptCall']);
+        $jsonResult['requireJsModules'] = $this->createExecutableStringRepresentationOfRegisteredRequireJsModules($childResult);
 
         return $jsonResult;
     }
@@ -661,7 +615,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
             return [];
         }
 
-        $inlineView = unserialize($backendUser->uc['inlineView']);
+        $inlineView = json_decode($backendUser->uc['inlineView'], true);
         if (!is_array($inlineView)) {
             $inlineView = [];
         }
@@ -671,7 +625,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
 
     /**
      * Method to check whether the backend user has the property inline view for the current IRRE item.
-     * In existing or old IRRE items the attribute may not exist, then the unserialize will fail.
+     * In existing or old IRRE items the attribute may not exist, then the json_decode will fail.
      *
      * @param BackendUserAuthentication $backendUser
      * @return bool
@@ -725,7 +679,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         // Substitute FlexForm addition and make parsing a bit easier
         $domObjectId = str_replace('---', ':', $domObjectId);
         // The starting pattern of an object identifier (e.g. "data-<firstPidValue>-<anything>)
-        $pattern = '/^data' . '-' . '(.+?)' . '-' . '(.+)$/';
+        $pattern = '/^data-(.+?)-(.+)$/';
         if (preg_match($pattern, $domObjectId, $match)) {
             return $match[1];
         }
