@@ -36,6 +36,7 @@ class LinkBrowserController extends AbstractLinkBrowserController
     protected function initCurrentUrl()
     {
         $currentLink = isset($this->parameters['currentValue']) ? trim($this->parameters['currentValue']) : '';
+        /** @var array<string,string> $currentLinkParts */
         $currentLinkParts = GeneralUtility::makeInstance(TypoLinkCodecService::class)->decode($currentLink);
         $currentLinkParts['params'] = $currentLinkParts['additionalParams'];
         unset($currentLinkParts['additionalParams']);
@@ -53,9 +54,6 @@ class LinkBrowserController extends AbstractLinkBrowserController
         parent::initCurrentUrl();
     }
 
-    /**
-     * Initialize document template object
-     */
     protected function initDocumentTemplate()
     {
         parent::initDocumentTemplate();
@@ -66,7 +64,7 @@ class LinkBrowserController extends AbstractLinkBrowserController
         unset($this->parameters['fieldChangeFunc']['alert']);
         $update = [];
         foreach ($this->parameters['fieldChangeFunc'] as $v) {
-            $update[] = 'parent.opener.' . $v;
+            $update[] = 'FormEngineLinkBrowserAdapter.getParent().' . $v;
         }
         $inlineJS = implode(LF, $update);
 
@@ -132,6 +130,7 @@ class LinkBrowserController extends AbstractLinkBrowserController
      */
     protected function getBodyTagAttributes()
     {
+        $formEngineParameters = [];
         $parameters = parent::getBodyTagAttributes();
 
         $formEngineParameters['fieldChangeFunc'] = $this->parameters['fieldChangeFunc'];
