@@ -15,16 +15,16 @@ namespace TYPO3\CMS\Backend\Controller;
  * The TYPO3 project - inspiring people to share!
  */
 
+use Psr\EventDispatcher\EventDispatcherInterface;
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
 use Psr\Log\LoggerAwareInterface;
 use Psr\Log\LoggerAwareTrait;
 use Symfony\Component\HttpFoundation\Cookie;
-use TYPO3\CMS\Backend\Exception;
+use TYPO3\CMS\Backend\LoginProvider\Event\ModifyPageLayoutOnLoginProviderSelectionEvent;
 use TYPO3\CMS\Backend\LoginProvider\LoginProviderInterface;
 use TYPO3\CMS\Backend\Routing\UriBuilder;
-use TYPO3\CMS\Backend\Template\DocumentTemplate;
-use TYPO3\CMS\Backend\Utility\BackendUtility;
+use TYPO3\CMS\Backend\Template\ModuleTemplate;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
 use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
 use TYPO3\CMS\Core\Database\ConnectionPool;
@@ -32,6 +32,7 @@ use TYPO3\CMS\Core\FormProtection\BackendFormProtection;
 use TYPO3\CMS\Core\FormProtection\FormProtectionFactory;
 use TYPO3\CMS\Core\Http\HtmlResponse;
 use TYPO3\CMS\Core\Http\NormalizedParams;
+use TYPO3\CMS\Core\Information\Typo3Information;
 use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Localization\Locales;
 use TYPO3\CMS\Core\Page\PageRenderer;
@@ -41,7 +42,7 @@ use TYPO3\CMS\Core\Utility\PathUtility;
 use TYPO3\CMS\Fluid\View\StandaloneView;
 
 /**
- * Script Class for rendering the login form
+ * Controller responsible for rendering the TYPO3 Backend login form
  * @internal This class is a specific Backend controller implementation and is not considered part of the Public TYPO3 API.
  */
 class LoginController implements LoggerAwareInterface
@@ -98,53 +99,35 @@ class LoginController implements LoggerAwareInterface
     protected $view;
 
     /**
-     * Initialize the login box. Will also react on a &L=OUT flag and exit.
+     * @var ModuleTemplate
      */
-    public function __construct()
+    protected $moduleTemplate;
+
+    /**
+     * @var EventDispatcherInterface
+     */
+    protected $eventDispatcher;
+
+    /**
+     * @var Typo3Information
+     */
+    protected $typo3Information;
+
+    /**
+     * @var UriBuilder
+     */
+    protected $uriBuilder;
+
+    /**
+     * @var PageRenderer
+     */
+    protected $pageRenderer;
+
+    public function __construct(Typo3Information $typo3Information, EventDispatcherInterface $eventDispatcher, UriBuilder $uriBuilder)
     {
-        // @deprecated since TYPO3 v9, will be obsolete in TYPO3 v10.0
-        $request = $GLOBALS['TYPO3_REQUEST'];
-        $parsedBody = $request->getParsedBody();
-        $queryParams = $request->getQueryParams();
-        $this->validateAndSortLoginProviders();
-
-        $this->redirectUrl = GeneralUtility::sanitizeLocalUrl($parsedBody['redirect_url'] ?? $queryParams['redirect_url'] ?? null);
-        $this->loginProviderIdentifier = $this->detectLoginProvider($request);
-
-        $this->loginRefresh = (bool)($parsedBody['loginRefresh'] ?? $queryParams['loginRefresh'] ?? false);
-        // Value of "Login" button. If set, the login button was pressed.
-        $this->submitValue = $parsedBody['commandLI'] ?? $queryParams['commandLI'] ?? null;
-        // Try to get the preferred browser language
-        /** @var Locales $locales */
-        $locales = GeneralUtility::makeInstance(Locales::class);
-        $httpAcceptLanguage = $request->getServerParams()['HTTP_ACCEPT_LANGUAGE'];
-        $preferredBrowserLanguage = $locales
-            ->getPreferredClientLanguage($httpAcceptLanguage);
-
-        // If we found a $preferredBrowserLanguage and it is not the default language and no be_user is logged in
-        // initialize $this->getLanguageService() again with $preferredBrowserLanguage
-        if ($preferredBrowserLanguage !== 'default' && empty($this->getBackendUserAuthentication()->user['uid'])) {
-            $this->getLanguageService()->init($preferredBrowserLanguage);
-            GeneralUtility::makeInstance(PageRenderer::class)->setLanguage($preferredBrowserLanguage);
-        }
-
-        $this->getLanguageService()->includeLLFile('EXT:backend/Resources/Private/Language/locallang_login.xlf');
-
-        // Setting the redirect URL to "index.php?M=main" if no alternative input is given
-        if ($this->redirectUrl) {
-            $this->redirectToURL = $this->redirectUrl;
-        } else {
-            $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
-            $this->redirectToURL = (string)$uriBuilder->buildUriFromRoute('main');
-        }
-
-        // If "L" is "OUT", then any logged in is logged out. If redirect_url is given, we redirect to it
-        if (($parsedBody['L'] ?? $queryParams['L'] ?? null) === 'OUT' && is_object($this->getBackendUserAuthentication())) {
-            $this->getBackendUserAuthentication()->logoff();
-            $this->redirectToUrl();
-        }
-
-        $this->view = $this->getFluidTemplateObject();
+        $this->typo3Information = $typo3Information;
+        $this->eventDispatcher = $eventDispatcher;
+        $this->uriBuilder = $uriBuilder;
     }
 
     /**
@@ -156,6 +139,7 @@ class LoginController implements LoggerAwareInterface
      */
     public function formAction(ServerRequestInterface $request): ResponseInterface
     {
+        $this->init($request);
         return new HtmlResponse($this->createLoginLogoutForm($request));
     }
 
@@ -167,40 +151,73 @@ class LoginController implements LoggerAwareInterface
      */
     public function refreshAction(ServerRequestInterface $request): ResponseInterface
     {
+        $this->init($request);
         $this->loginRefresh = true;
         return new HtmlResponse($this->createLoginLogoutForm($request));
     }
 
     /**
-     * Main function - creating the login/logout form
+     * This can be called by single login providers, they receive an instance of $this
      *
-     * @throws Exception
-     * @return string The content to output
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
+     * @return string
      */
-    public function main(): string
+    public function getLoginProviderIdentifier()
     {
-        trigger_error('LoginController->main() will be replaced by protected method createLoginLogoutForm() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        return $this->createLoginLogoutForm($GLOBALS['TYPO3_REQUEST']);
+        return $this->loginProviderIdentifier;
     }
 
     /**
-     * Main function - creating the login/logout form
+     * Initialize the login box. Will also react on a &L=OUT flag and exit.
      *
-     * @param ServerRequestInterface $request
-     * @return string $content
-     * @throws Exception
+     * @param ServerRequestInterface $request the current request
      */
-    protected function createLoginLogoutForm(ServerRequestInterface $request): string
+    protected function init(ServerRequestInterface $request): void
     {
-        /** @var PageRenderer $pageRenderer */
-        $pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
-        $pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/Login');
+        $this->moduleTemplate = GeneralUtility::makeInstance(ModuleTemplate::class);
+        $this->moduleTemplate->setTitle('TYPO3 CMS Login: ' . $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename']);
+        $this->pageRenderer = $this->moduleTemplate->getPageRenderer();
+        $parsedBody = $request->getParsedBody();
+        $queryParams = $request->getQueryParams();
+        $this->validateAndSortLoginProviders();
 
-        // Checking, if we should make a redirect.
-        // Might set JavaScript in the header to close window.
-        $this->checkRedirect($request);
+        $this->redirectUrl = GeneralUtility::sanitizeLocalUrl($parsedBody['redirect_url'] ?? $queryParams['redirect_url'] ?? null);
+        $this->loginProviderIdentifier = $this->detectLoginProvider($request);
 
+        $this->loginRefresh = (bool)($parsedBody['loginRefresh'] ?? $queryParams['loginRefresh'] ?? false);
+        // Value of "Login" button. If set, the login button was pressed.
+        $this->submitValue = $parsedBody['commandLI'] ?? $queryParams['commandLI'] ?? null;
+        // Try to get the preferred browser language
+        $httpAcceptLanguage = $request->getServerParams()['HTTP_ACCEPT_LANGUAGE'];
+        $preferredBrowserLanguage = GeneralUtility::makeInstance(Locales::class)->getPreferredClientLanguage($httpAcceptLanguage);
+
+        // If we found a $preferredBrowserLanguage and it is not the default language and no be_user is logged in
+        // initialize $this->getLanguageService() again with $preferredBrowserLanguage
+        if ($preferredBrowserLanguage !== 'default' && empty($this->getBackendUserAuthentication()->user['uid'])) {
+            $this->getLanguageService()->init($preferredBrowserLanguage);
+            $this->pageRenderer->setLanguage($preferredBrowserLanguage);
+        }
+
+        $this->getLanguageService()->includeLLFile('EXT:backend/Resources/Private/Language/locallang_login.xlf');
+
+        // Setting the redirect URL to "index.php?M=main" if no alternative input is given
+        if ($this->redirectUrl) {
+            $this->redirectToURL = $this->redirectUrl;
+        } else {
+            $this->redirectToURL = (string)$this->uriBuilder->buildUriFromRoute('main');
+        }
+
+        // If "L" is "OUT", then any logged in is logged out. If redirect_url is given, we redirect to it
+        if (($parsedBody['L'] ?? $queryParams['L'] ?? null) === 'OUT' && is_object($this->getBackendUserAuthentication())) {
+            $this->getBackendUserAuthentication()->logoff();
+            $this->redirectToUrl();
+        }
+
+        $this->view = $this->moduleTemplate->getView();
+        $this->view->getRequest()->setControllerExtensionName('Backend');
+    }
+
+    protected function provideCustomLoginStyling()
+    {
         // Extension Configuration
         $extConf = GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('backend');
 
@@ -213,13 +230,13 @@ class LoginController implements LoggerAwareInterface
                     '" can\'t be resolved. Please check if the file exists and the extension is activated.'
                 );
             }
-            $this->getDocumentTemplate()->inDocStylesArray[] = '
+            $this->pageRenderer->addCssInlineBlock('loginBackgroundImage', '
 				.typo3-login-carousel-control.right,
 				.typo3-login-carousel-control.left,
 				.panel-login { border: 0; }
 				.typo3-login { background-image: url("' . $backgroundImage . '"); }
 				.typo3-login-footnote { background-color: #000000; color: #ffffff; opacity: 0.5; }
-			';
+			');
         }
 
         // Login Footnote
@@ -229,7 +246,7 @@ class LoginController implements LoggerAwareInterface
 
         // Add additional css to use the highlight color in the login screen
         if (!empty($extConf['loginHighlightColor'])) {
-            $this->getDocumentTemplate()->inDocStylesArray[] = '
+            $this->pageRenderer->addCssInlineBlock('loginHighlightColor', '
 				.btn-login.disabled, .btn-login[disabled], fieldset[disabled] .btn-login,
 				.btn-login.disabled:hover, .btn-login[disabled]:hover, fieldset[disabled] .btn-login:hover,
 				.btn-login.disabled:focus, .btn-login[disabled]:focus, fieldset[disabled] .btn-login:focus,
@@ -240,7 +257,7 @@ class LoginController implements LoggerAwareInterface
 				.btn-login:active:hover, .btn-login:active:focus,
 				.btn-login { background-color: ' . $extConf['loginHighlightColor'] . '; }
 				.panel-login .panel-body { border-color: ' . $extConf['loginHighlightColor'] . '; }
-			';
+			');
         }
 
         // Logo
@@ -259,11 +276,36 @@ class LoginController implements LoggerAwareInterface
             } else {
                 $logo = 'EXT:backend/Resources/Public/Images/typo3_orange.svg';
             }
-            $this->getDocumentTemplate()->inDocStylesArray[] = '
+            $this->pageRenderer->addCssInlineBlock('loginLogo', '
 				.typo3-login-logo .typo3-login-image { max-width: 150px; height:100%;}
-			';
+			');
         }
-        $logo = $this->getUriForFileName($logo);
+        $this->view->assignMultiple([
+            'logo' => $this->getUriForFileName($logo),
+            'images' => [
+                'capslock' => $this->getUriForFileName('EXT:backend/Resources/Public/Images/icon_capslock.svg'),
+                'typo3' => $this->getUriForFileName('EXT:backend/Resources/Public/Images/typo3_orange.svg'),
+            ],
+            'copyright' => $this->typo3Information->getCopyrightNotice(),
+            'loginNewsItems' => $this->getSystemNews(),
+        ]);
+    }
+
+    /**
+     * Main function - creating the login/logout form
+     *
+     * @param ServerRequestInterface $request
+     * @return string $content
+     */
+    protected function createLoginLogoutForm(ServerRequestInterface $request): string
+    {
+        $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/Login');
+
+        // Checking, if we should make a redirect.
+        // Might set JavaScript in the header to close window.
+        $this->checkRedirect($request);
+
+        $this->provideCustomLoginStyling();
 
         // Start form
         $formType = empty($this->getBackendUserAuthentication()->user['uid']) ? 'LoginForm' : 'LogoutForm';
@@ -271,31 +313,32 @@ class LoginController implements LoggerAwareInterface
             'backendUser' => $this->getBackendUserAuthentication()->user,
             'hasLoginError' => $this->isLoginInProgress($request),
             'formType' => $formType,
-            'logo' => $logo,
-            'images' => [
-                'capslock' => $this->getUriForFileName('EXT:backend/Resources/Public/Images/icon_capslock.svg'),
-                'typo3' => $this->getUriForFileName('EXT:backend/Resources/Public/Images/typo3_orange.svg'),
-            ],
-            'copyright' => BackendUtility::TYPO3_copyRightNotice(),
             'redirectUrl' => $this->redirectUrl,
             'loginRefresh' => $this->loginRefresh,
-            'loginNewsItems' => $this->getSystemNews(),
             'loginProviderIdentifier' => $this->loginProviderIdentifier,
             'loginProviders' => $this->loginProviders
         ]);
 
         // Initialize interface selectors:
         $this->makeInterfaceSelector($request);
+        $this->renderHtmlViaLoginProvider();
 
+        $this->moduleTemplate->setContent($this->view->render());
+        return $this->moduleTemplate->renderContent();
+    }
+
+    protected function renderHtmlViaLoginProvider(): void
+    {
         /** @var LoginProviderInterface $loginProvider */
         $loginProvider = GeneralUtility::makeInstance($this->loginProviders[$this->loginProviderIdentifier]['provider']);
-        $loginProvider->render($this->view, $pageRenderer, $this);
-
-        $content = $this->getDocumentTemplate()->startPage('TYPO3 CMS Login: ' . $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename']);
-        $content .= $this->view->render();
-        $content .= $this->getDocumentTemplate()->endPage();
-
-        return $content;
+        $this->eventDispatcher->dispatch(
+            new ModifyPageLayoutOnLoginProviderSelectionEvent(
+                $this,
+                $this->view,
+                $this->pageRenderer
+            )
+        );
+        $loginProvider->render($this->view, $this->pageRenderer, $this);
     }
 
     /**
@@ -322,11 +365,7 @@ class LoginController implements LoggerAwareInterface
          */
         if (!isset($_COOKIE[BackendUserAuthentication::getCookieName()])) {
             if ($this->submitValue === 'setCookie') {
-                /*
-                 * we tried it a second time but still no cookie
-                 * 26/4 2005: This does not work anymore, because the saving of challenge values
-                 * in $_SESSION means the system will act as if the password was wrong.
-                 */
+                // we tried it a second time but still no cookie
                 throw new \RuntimeException('Login-error: Yeah, that\'s a classic. No cookies, no TYPO3. ' .
                     'Please accept cookies from TYPO3 - otherwise you\'ll not be able to use the system.', 1294586846);
             }
@@ -344,8 +383,7 @@ class LoginController implements LoggerAwareInterface
                     $this->redirectToURL = '../';
                     break;
                 case 'backend':
-                    $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
-                    $this->redirectToURL = (string)$uriBuilder->buildUriFromRoute('main');
+                    $this->redirectToURL = (string)$this->uriBuilder->buildUriFromRoute('main');
                     break;
             }
         } else {
@@ -363,7 +401,7 @@ class LoginController implements LoggerAwareInterface
         if ($this->loginRefresh) {
             $formProtection->setSessionTokenFromRegistry();
             $formProtection->persistSessionToken();
-            $this->getDocumentTemplate()->JScode .= GeneralUtility::wrapJS('
+            $this->pageRenderer->addJsInlineCode('loginRefresh', '
 				if (window.opener && window.opener.TYPO3 && window.opener.TYPO3.LoginRefresh) {
 					window.opener.TYPO3.LoginRefresh.startTask();
 					window.close();
@@ -375,17 +413,6 @@ class LoginController implements LoggerAwareInterface
         }
     }
 
-    /**
-     * Making interface selector
-     *
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
-     */
-    public function makeInterfaceSelectorBox(): void
-    {
-        trigger_error('LoginController->makeInterfaceSelectorBox() will be replaced by protected method makeInterfaceSelector() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        $this->makeInterfaceSelector($GLOBALS['TYPO3_REQUEST']);
-    }
-
     /**
      * Making interface selector
      * @param ServerRequestInterface $request
@@ -397,11 +424,10 @@ class LoginController implements LoggerAwareInterface
             $parts = GeneralUtility::trimExplode(',', $GLOBALS['TYPO3_CONF_VARS']['BE']['interfaces']);
             if (count($parts) > 1) {
                 // Only if more than one interface is defined we will show the selector
-                $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
                 $interfaces = [
                     'backend' => [
                         'label' => $this->getLanguageService()->getLL('interface.backend'),
-                        'jumpScript' => (string)$uriBuilder->buildUriFromRoute('main'),
+                        'jumpScript' => (string)$this->uriBuilder->buildUriFromRoute('main'),
                         'interface' => 'backend'
                     ],
                     'frontend' => [
@@ -486,23 +512,6 @@ class LoginController implements LoggerAwareInterface
         return !empty($username) || !empty($this->submitValue);
     }
 
-    /**
-     * returns a new standalone view, shorthand function
-     *
-     * @return StandaloneView
-     */
-    protected function getFluidTemplateObject()
-    {
-        /** @var StandaloneView $view */
-        $view = GeneralUtility::makeInstance(StandaloneView::class);
-        $view->setLayoutRootPaths([GeneralUtility::getFileAbsFileName('EXT:backend/Resources/Private/Layouts')]);
-        $view->setPartialRootPaths([GeneralUtility::getFileAbsFileName('EXT:backend/Resources/Private/Partials')]);
-        $view->setTemplateRootPaths([GeneralUtility::getFileAbsFileName('EXT:backend/Resources/Private/Templates')]);
-
-        $view->getRequest()->setControllerExtensionName('Backend');
-        return $view;
-    }
-
     /**
      * Wrapper method to redirect to configured redirect URL
      */
@@ -586,14 +595,6 @@ class LoginController implements LoggerAwareInterface
         return (string)$loginProvider;
     }
 
-    /**
-     * @return string
-     */
-    public function getLoginProviderIdentifier()
-    {
-        return $this->loginProviderIdentifier;
-    }
-
     /**
      * Returns LanguageService
      *
@@ -611,14 +612,4 @@ class LoginController implements LoggerAwareInterface
     {
         return $GLOBALS['BE_USER'];
     }
-
-    /**
-     * Returns an instance of DocumentTemplate
-     *
-     * @return DocumentTemplate
-     */
-    protected function getDocumentTemplate(): DocumentTemplate
-    {
-        return $GLOBALS['TBE_TEMPLATE'];
-    }
 }
