@@ -24,9 +24,9 @@ use TYPO3\CMS\Backend\Tree\View\NewRecordPageTreeView;
 use TYPO3\CMS\Backend\Tree\View\PagePositionMap;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
-use TYPO3\CMS\Core\Compatibility\PublicPropertyDeprecationTrait;
 use TYPO3\CMS\Core\Database\ConnectionPool;
 use TYPO3\CMS\Core\Database\Query\Restriction\DeletedRestriction;
+use TYPO3\CMS\Core\Domain\Repository\PageRepository;
 use TYPO3\CMS\Core\Http\HtmlResponse;
 use TYPO3\CMS\Core\Http\RedirectResponse;
 use TYPO3\CMS\Core\Imaging\Icon;
@@ -34,9 +34,7 @@ use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Type\Bitmask\Permission;
 use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Core\Utility\HttpUtility;
 use TYPO3\CMS\Core\Utility\PathUtility;
-use TYPO3\CMS\Frontend\Page\PageRepository;
 
 /**
  * Script class for 'db_new'
@@ -44,32 +42,6 @@ use TYPO3\CMS\Frontend\Page\PageRepository;
  */
 class NewRecordController
 {
-    use PublicPropertyDeprecationTrait;
-
-    /**
-     * @var array
-     */
-    protected $deprecatedPublicProperties = [
-        'pageinfo' => 'Using $pageinfo of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'pidInfo' => 'Using $pidInfo of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'newPagesInto' => 'Using $newPagesInto of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'newContentInto' => 'Using $newContentInto of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'newPagesAfter' => 'Using $newPagesAfter of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'web_list_modTSconfig' => 'Using $web_list_modTSconfig of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'allowedNewTables' => 'Using $allowedNewTables of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'deniedNewTables' => 'Using $deniedNewTables of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'web_list_modTSconfig_pid' => 'Using $web_list_modTSconfig_pid of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'allowedNewTables_pid' => 'Using $allowedNewTables_pid of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'deniedNewTables_pid' => 'Using $deniedNewTables_pid of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'code' => 'Using $code of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'R_URI' => 'Using $R_URI of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'returnUrl' => 'Using $returnUrl of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'pagesOnly' => 'Using $pagesOnly of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'perms_clause' => 'Using $perms_clause of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'content' => 'Using $content of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-        'tRows' => 'Using $tRows of class NewRecordController from outside is discouraged as this variable is only used for internal storage.',
-    ];
-
     /**
      * @var array
      */
@@ -201,9 +173,6 @@ class NewRecordController
 
         // @see \TYPO3\CMS\Backend\Tree\View\NewRecordPageTreeView::expandNext()
         $GLOBALS['SOBE'] = $this;
-
-        // @deprecated since TYPO3 v9, will be moved out of __construct() in TYPO3 v10.0
-        $this->init($GLOBALS['TYPO3_REQUEST']);
     }
 
     /**
@@ -215,6 +184,7 @@ class NewRecordController
      */
     public function mainAction(ServerRequestInterface $request): ResponseInterface
     {
+        $this->init($request);
         $response = $this->renderContent($request);
 
         if (empty($response)) {
@@ -224,109 +194,6 @@ class NewRecordController
         return $response;
     }
 
-    /**
-     * Main processing, creating the list of new record tables to select from
-     *
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
-     */
-    public function main()
-    {
-        trigger_error('NewRecordController->main() will be replaced by protected method renderContent() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-
-        $response = $this->renderContent($GLOBALS['TYPO3_REQUEST']);
-
-        if ($response instanceof RedirectResponse) {
-            HttpUtility::redirect($response->getHeaders()['location'][0]);
-        }
-    }
-
-    /**
-     * Creates the position map for pages wizard
-     *
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
-     */
-    public function pagesOnly()
-    {
-        trigger_error('NewRecordController->pagesOnly() will be replaced by protected method renderPositionTree() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        $this->renderPositionTree();
-    }
-
-    /**
-     * Create a regular new element (pages and records)
-     *
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
-     */
-    public function regularNew()
-    {
-        trigger_error('NewRecordController->regularNew() will be replaced by protected method renderNewRecordControls() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        $this->renderNewRecordControls($GLOBALS['TYPO3_REQUEST']);
-    }
-
-    /**
-     * User array sort function used by renderNewRecordControls
-     *
-     * @param string $a First array element for compare
-     * @param string $b First array element for compare
-     * @return int -1 for lower, 0 for equal, 1 for greater
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
-     */
-    public function sortNewRecordsByConfig($a, $b)
-    {
-        trigger_error('NewRecordController->sortNewRecordsByConfig() will be replaced by protected method sortTableRows() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        return $this->sortTableRows($a, $b);
-    }
-
-    /**
-     * Links the string $code to a create-new form for a record in $table created on page $pid
-     *
-     * @param string $linkText Link text
-     * @param string $table Table name (in which to create new record)
-     * @param int $pid PID value for the "&edit['.$table.']['.$pid.']=new" command (positive/negative)
-     * @param bool $addContentTable If $addContentTable is set, then a new tt_content record is created together with pages
-     * @return string The link.
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
-     */
-    public function linkWrap($linkText, $table, $pid, $addContentTable = false)
-    {
-        trigger_error('NewRecordController->linkWrap() will be replaced by protected method renderLink() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        return $this->renderLink($linkText, $table, $pid, $addContentTable);
-    }
-
-    /**
-     * Returns TRUE if the tablename $checkTable is allowed to be created on the page with record $pid_row
-     *
-     * @param array $pid_row Record for parent page.
-     * @param string $checkTable Table name to check
-     * @return bool Returns TRUE if the tablename $checkTable is allowed to be created on the page with record $pid_row
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
-     */
-    public function isTableAllowedForThisPage($pid_row, $checkTable)
-    {
-        trigger_error('NewRecordController->isTableAllowedForThisPage() will be replaced by protected method isTableAllowedOnPage() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        return $this->isTableAllowedOnPage($checkTable, $pid_row);
-    }
-
-    /**
-     * Returns TRUE if:
-     * - $allowedNewTables and $deniedNewTables are empty
-     * - the table is not found in $deniedNewTables and $allowedNewTables is not set or the $table tablename is found in
-     *   $allowedNewTables
-     *
-     * If $table tablename is found in $allowedNewTables and $deniedNewTables, $deniedNewTables
-     * has priority over $allowedNewTables.
-     *
-     * @param string $table Table name to test if in allowedTables
-     * @param array $allowedNewTables Array of new tables that are allowed.
-     * @param array $deniedNewTables Array of new tables that are not allowed.
-     * @return bool Returns TRUE if a link for creating new records should be displayed for $table
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0
-     */
-    public function showNewRecLink($table, array $allowedNewTables = [], array $deniedNewTables = [])
-    {
-        trigger_error('NewRecordController->showNewRecLink() will be replaced by protected method isRecordCreationAllowedForTable() in TYPO3 v10.0. Do not call from other extension.', E_USER_DEPRECATED);
-        return $this->isRecordCreationAllowedForTable($table, $allowedNewTables, $deniedNewTables);
-    }
-
     /**
      * Constructor function for the class
      *
@@ -461,6 +328,7 @@ class NewRecordController
      */
     protected function getButtons(): void
     {
+        $buttons = [];
         $lang = $this->getLanguageService();
         $buttonBar = $this->moduleTemplate->getDocHeaderComponent()->getButtonBar();
         // Regular new element:
@@ -613,7 +481,7 @@ class NewRecordController
         if ($displayNewPagesIntoLink
             && $this->isTableAllowedOnPage('pages', $this->pageinfo)
             && $this->getBackendUserAuthentication()->check('tables_modify', 'pages')
-            && $this->getBackendUserAuthentication()->workspaceCreateNewRecord(($this->pageinfo['_ORIG_uid'] ?: $this->id), 'pages')
+            && $this->getBackendUserAuthentication()->workspaceCanCreateNewRecord('pages')
         ) {
             // Create link to new page inside:
             $recordIcon = $this->moduleTemplate->getIconFactory()->getIconForRecord($table, [], Icon::SIZE_SMALL)->render();
@@ -627,7 +495,7 @@ class NewRecordController
         if ($displayNewPagesAfterLink
             && $this->isTableAllowedOnPage('pages', $this->pidInfo)
             && $this->getBackendUserAuthentication()->check('tables_modify', 'pages')
-            && $this->getBackendUserAuthentication()->workspaceCreateNewRecord($this->pidInfo['uid'], 'pages')
+            && $this->getBackendUserAuthentication()->workspaceCanCreateNewRecord('pages')
         ) {
             $newPageLinks[] = $this->renderLink(
                 $pageIcon . htmlspecialchars($lang->sL($v['ctrl']['title'])) . ' (' . htmlspecialchars($lang->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:db_new.php.after')) . ')',
@@ -669,7 +537,7 @@ class NewRecordController
                         && $this->isTableAllowedOnPage($table, $this->pageinfo)
                         && $this->getBackendUserAuthentication()->check('tables_modify', $table)
                         && ($rootLevelConfiguration === -1 || ($this->id xor $rootLevelConfiguration))
-                        && $this->getBackendUserAuthentication()->workspaceCreateNewRecord(($this->pageinfo['_ORIG_uid'] ? $this->pageinfo['_ORIG_uid'] : $this->id), $table)
+                        && $this->getBackendUserAuthentication()->workspaceCanCreateNewRecord($table)
                     ) {
                         $newRecordIcon = $this->moduleTemplate->getIconFactory()->getIconForRecord($table, [], Icon::SIZE_SMALL)->render();
                         $rowContent = '';
@@ -692,9 +560,10 @@ class NewRecordController
                             /** @var \TYPO3\CMS\Core\Http\NormalizedParams */
                             $normalizedParams = $request->getAttribute('normalizedParams');
                             $url = (string)$uriBuilder->buildUriFromRoute($moduleName, ['id' => $this->id, 'returnUrl' => $normalizedParams->getRequestUri()]);
+                            $title = htmlspecialchars($this->getLanguageService()->getLL('newContentElement'));
                             $rowContent .= '<li>' . $newLink . ' ' . BackendUtility::wrapInHelp($table, '') . '</li>'
                                 . '<li>'
-                                . '<a href="' . htmlspecialchars($url) . '" data-title="' . htmlspecialchars($this->getLanguageService()->getLL('newContentElement')) . '" class="t3js-toggle-new-content-element-wizard">'
+                                . '<a href="' . htmlspecialchars($url) . '" title="' . $title . '" data-title="' . $title . '" class="t3js-toggle-new-content-element-wizard">'
                                 . $newContentIcon . htmlspecialchars($lang->getLL('clickForWizard'))
                                 . '</a>'
                                 . '</li>'
@@ -729,7 +598,7 @@ class NewRecordController
                                             include $extEmConfFile;
                                             $thisTitle = $EM_CONF[$_EXTKEY]['title'];
                                         }
-                                        $iconFile[$_EXTKEY] = '<img src="' . PathUtility::getAbsoluteWebPath(ExtensionManagementUtility::getExtensionIcon($extPath, true)) . '" ' . 'width="16" height="16" ' . 'alt="' . $thisTitle . '" />';
+                                        $iconFile[$_EXTKEY] = '<img src="' . PathUtility::getAbsoluteWebPath(ExtensionManagementUtility::getExtensionIcon($extPath, true)) . '" width="16" height="16" alt="' . $thisTitle . '" />';
                                     }
                                 }
                                 if (empty($thisTitle)) {
@@ -865,7 +734,7 @@ class NewRecordController
             $allowedTableList = $GLOBALS['PAGES_TYPES']['default']['allowedTables'];
         }
         // If all tables or the table is listed as an allowed type, return TRUE
-        if (strstr($allowedTableList, '*') || GeneralUtility::inList($allowedTableList, $table)) {
+        if (strpos($allowedTableList, '*') !== false || GeneralUtility::inList($allowedTableList, $table)) {
             return true;
         }
 
@@ -904,25 +773,6 @@ class NewRecordController
         return !in_array($table, $deniedNewTables) && (empty($allowedNewTables) || in_array($table, $allowedNewTables));
     }
 
-    /**
-     * Checks if sys_language records are present
-     *
-     * @return bool
-     */
-    protected function checkIfLanguagesExist(): bool
-    {
-        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
-            ->getQueryBuilderForTable('sys_language');
-        $queryBuilder->getRestrictions()->removeAll();
-
-        $count = $queryBuilder
-            ->count('uid')
-            ->from('sys_language')
-            ->execute()
-            ->fetchColumn(0);
-        return (bool)$count;
-    }
-
     /**
      * @return LanguageService
      */
