@@ -28,7 +28,6 @@ use TYPO3\CMS\Core\Http\JsonResponse;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
 use TYPO3\CMS\Core\Localization\LanguageService;
-use TYPO3\CMS\Core\Site\PseudoSiteFinder;
 use TYPO3\CMS\Core\Site\SiteFinder;
 use TYPO3\CMS\Core\Type\Bitmask\JsConfirmation;
 use TYPO3\CMS\Core\Type\Bitmask\Permission;
@@ -291,11 +290,10 @@ class TreeController
             'icon' => $icon->getIdentifier(),
             'name' => $visibleText,
             'nameSourceField' => $nameSourceField,
-            'alias' => htmlspecialchars($page['alias'] ?? ''),
-            'selectable' => true,
-            'checked' => false,
             'mountPoint' => $entryPoint,
             'workspaceId' => !empty($page['t3ver_oid']) ? $page['t3ver_oid'] : $pageId,
+            'siblingsCount' => $page['siblingsCount'] ?? 1,
+            'siblingsPosition' => $page['siblingsPosition'] ?? 1,
         ];
 
         if (!empty($page['_children'])) {
@@ -335,8 +333,12 @@ class TreeController
         }
 
         $items[] = $item;
-        if (!$stopPageTree) {
+        if (!$stopPageTree && is_array($page['_children'])) {
+            $siblingsCount = count($page['_children']);
+            $siblingsPosition = 0;
             foreach ($page['_children'] as $child) {
+                $child['siblingsCount'] = $siblingsCount;
+                $child['siblingsPosition'] = ++$siblingsPosition;
                 $items = array_merge($items, $this->pagesToFlatArray($child, $entryPoint, $depth + 1, ['backgroundColor' => $backgroundColor]));
             }
         }
@@ -443,14 +445,7 @@ class TreeController
             $site = $siteFinder->getSiteByRootPageId($pageId);
             $domain = (string)$site->getBase();
         } catch (SiteNotFoundException $e) {
-            // No site found, let's see if it is a legacy-pseudo-site
-            $pseudoSiteFinder = GeneralUtility::makeInstance(PseudoSiteFinder::class);
-            try {
-                $site = $pseudoSiteFinder->getSiteByRootPageId($pageId);
-                $domain = trim((string)$site->getBase(), '/');
-            } catch (SiteNotFoundException $e) {
-                // No pseudo-site found either
-            }
+            // No site found
         }
 
         return $domain;
