@@ -26,19 +26,6 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
  */
 class SystemInformationController
 {
-    /**
-     * @var SystemInformationToolbarItem
-     */
-    protected $toolbarItem;
-
-    /**
-     * Set up dependencies
-     */
-    public function __construct()
-    {
-        $this->toolbarItem = GeneralUtility::makeInstance(SystemInformationToolbarItem::class);
-    }
-
     /**
      * Renders the menu for AJAX calls
      *
@@ -46,6 +33,7 @@ class SystemInformationController
      */
     public function renderMenuAction(): ResponseInterface
     {
-        return new HtmlResponse($this->toolbarItem->getDropDown());
+        $toolbarItem = GeneralUtility::makeInstance(SystemInformationToolbarItem::class);
+        return new HtmlResponse($toolbarItem->getDropDown());
     }
 }
