@@ -194,7 +194,7 @@ class BackendModuleRepository implements \TYPO3\CMS\Core\SingletonInterface
     protected function createMenuEntriesForTbeModulesExt()
     {
         foreach ($GLOBALS['TBE_MODULES_EXT'] ?? [] as $mainModule => $tbeModuleExt) {
-            list($main) = explode('_', $mainModule);
+            [$main] = explode('_', $mainModule);
             $mainEntry = $this->findByModuleName($main);
             if ($mainEntry === false) {
                 continue;
@@ -348,7 +348,7 @@ class BackendModuleRepository implements \TYPO3\CMS\Core\SingletonInterface
         $iconRegistry = GeneralUtility::makeInstance(IconRegistry::class);
         if ($iconRegistry->isRegistered($iconIdentifier)) {
             $iconFactory = GeneralUtility::makeInstance(IconFactory::class);
-            return $iconFactory->getIcon($iconIdentifier)->render();
+            return $iconFactory->getIcon($iconIdentifier)->render('inline');
         }
         return '';
     }
