@@ -94,7 +94,6 @@ abstract class AbstractNode implements NodeInterface, LoggerAwareInterface
     {
         return [
             'additionalJavaScriptPost' => [],
-            'additionalJavaScriptSubmit' => [],
             'additionalHiddenFields' => [],
             'additionalInlineLanguageLabelFiles' => [],
             'stylesheetFiles' => [],
@@ -125,9 +124,6 @@ abstract class AbstractNode implements NodeInterface, LoggerAwareInterface
         foreach ($childReturn['additionalJavaScriptPost'] ?? [] as $value) {
             $existing['additionalJavaScriptPost'][] = $value;
         }
-        foreach ($childReturn['additionalJavaScriptSubmit'] ?? [] as $value) {
-            $existing['additionalJavaScriptSubmit'][] = $value;
-        }
         foreach ($childReturn['additionalHiddenFields'] ?? [] as $value) {
             $existing['additionalHiddenFields'][] = $value;
         }
