@@ -14,7 +14,6 @@ namespace TYPO3\CMS\Backend\Form\Element;
  * The TYPO3 project - inspiring people to share!
  */
 
-use TYPO3\CMS\Backend\Form\InlineStackProcessor;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Localization\LanguageService;
@@ -77,25 +76,13 @@ class GroupElement extends AbstractFormElement
         'tableList' => [
             'renderType' => 'tableList',
         ],
-        'fileTypeList' => [
-            'renderType' => 'fileTypeList',
-            'after' => [ 'tableList' ],
-        ],
-        'fileThumbnails' => [
-            'renderType' => 'fileThumbnails',
-            'after' => [ 'fileTypeList' ],
-        ],
         'recordsOverview' => [
             'renderType' => 'recordsOverview',
-            'after' => [ 'fileThumbnails' ],
-        ],
-        'fileUpload' => [
-            'renderType' => 'fileUpload',
-            'after' => [ 'recordsOverview' ],
+            'after' => [ 'tableList' ],
         ],
         'localizationStateSelector' => [
             'renderType' => 'localizationStateSelector',
-            'after' => [ 'fileUpload' ],
+            'after' => [ 'recordsOverview' ],
         ],
         'otherLanguageContent' => [
             'renderType' => 'otherLanguageContent',
@@ -145,19 +132,7 @@ class GroupElement extends AbstractFormElement
 
         $listOfSelectedValues = [];
         $selectorOptionsHtml = [];
-        if ($internalType === 'file_reference' || $internalType === 'file') {
-            // @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0. Deprecation logged by TcaMigration class.
-            foreach ($selectedItems as $selectedItem) {
-                $uidOrPath = $selectedItem['uidOrPath'];
-                $listOfSelectedValues[] = $uidOrPath;
-                $title = $selectedItem['title'];
-                $shortenedTitle = GeneralUtility::fixed_lgd_cs($title, $maxTitleLength);
-                $selectorOptionsHtml[] =
-                    '<option value="' . htmlspecialchars($uidOrPath) . '" title="' . htmlspecialchars($title) . '">'
-                        . htmlspecialchars($this->appendValueToLabelInDebugMode($shortenedTitle, $uidOrPath))
-                    . '</option>';
-            }
-        } elseif ($internalType === 'folder') {
+        if ($internalType === 'folder') {
             foreach ($selectedItems as $selectedItem) {
                 $folder = $selectedItem['folder'];
                 $listOfSelectedValues[] = $folder;
@@ -265,24 +240,13 @@ class GroupElement extends AbstractFormElement
             $showDeleteControl = false;
         }
 
-        // Check against inline uniqueness - Create some onclick js for delete control and element browser
-        // to override record selection in some FAL scenarios - See 'appearance' docs of group element
-        $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
-        $inlineStackProcessor->initializeByGivenStructure($this->data['inlineStructure']);
-        $deleteControlOnClick = '';
-        if ($this->data['isInlineChild']
-            && $this->data['inlineParentUid']
-            && $this->data['inlineParentConfig']['foreign_table'] === $table
-            && $this->data['inlineParentConfig']['foreign_unique'] === $fieldName
-        ) {
-            $objectPrefix = $inlineStackProcessor->getCurrentStructureDomObjectIdPrefix($this->data['inlineFirstPid']) . '-' . $table;
-            $deleteControlOnClick = 'inline.revertUnique(' . GeneralUtility::quoteJSvalue($objectPrefix) . ',null,' . GeneralUtility::quoteJSvalue($row['uid']) . ');';
-        }
+        $fieldId = StringUtility::getUniqueId('tceforms-multiselect-');
 
         $selectorAttributes = [
-            'id' => StringUtility::getUniqueId('tceforms-multiselect-'),
+            'id' => $fieldId,
             'data-formengine-input-name' => htmlspecialchars($elementName),
             'data-formengine-validation-rules' => $this->getValidationDataAsJsonString($config),
+            'data-maxitems' => $maxItems,
             'size' => $size,
         ];
         $selectorClasses = [
@@ -345,7 +309,7 @@ class GroupElement extends AbstractFormElement
         $html[] =           '<div class="btn-group-vertical">';
         if ($maxItems > 1 && $size >=5 && $showMoveIcons) {
             $html[] =           '<a href="#"';
-            $html[] =               ' class="btn btn-default t3js-btn-moveoption-top"';
+            $html[] =               ' class="btn btn-default t3js-btn-option t3js-btn-moveoption-top"';
             $html[] =               ' data-fieldname="' . htmlspecialchars($elementName) . '"';
             $html[] =               ' title="' . htmlspecialchars($languageService->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.move_to_top')) . '"';
             $html[] =           '>';
@@ -354,14 +318,14 @@ class GroupElement extends AbstractFormElement
         }
         if ($maxItems > 1 && $size > 1 && $showMoveIcons) {
             $html[] =           '<a href="#"';
-            $html[] =               ' class="btn btn-default t3js-btn-moveoption-up"';
+            $html[] =               ' class="btn btn-default t3js-btn-option t3js-btn-moveoption-up"';
             $html[] =               ' data-fieldname="' . htmlspecialchars($elementName) . '"';
             $html[] =               ' title="' . htmlspecialchars($languageService->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.move_up')) . '"';
             $html[] =           '>';
             $html[] =               $this->iconFactory->getIcon('actions-move-up', Icon::SIZE_SMALL)->render();
             $html[] =           '</a>';
             $html[] =           '<a href="#"';
-            $html[] =               ' class="btn btn-default t3js-btn-moveoption-down"';
+            $html[] =               ' class="btn btn-default t3js-btn-option t3js-btn-moveoption-down"';
             $html[] =               ' data-fieldname="' . htmlspecialchars($elementName) . '"';
             $html[] =               ' title="' . htmlspecialchars($languageService->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.move_down')) . '"';
             $html[] =           '>';
@@ -370,7 +334,7 @@ class GroupElement extends AbstractFormElement
         }
         if ($maxItems > 1 && $size >= 5 && $showMoveIcons) {
             $html[] =           '<a href="#"';
-            $html[] =               ' class="btn btn-default t3js-btn-moveoption-bottom"';
+            $html[] =               ' class="btn btn-default t3js-btn-option t3js-btn-moveoption-bottom"';
             $html[] =               ' data-fieldname="' . htmlspecialchars($elementName) . '"';
             $html[] =               ' title="' . htmlspecialchars($languageService->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.move_to_bottom')) . '"';
             $html[] =           '>';
@@ -379,10 +343,10 @@ class GroupElement extends AbstractFormElement
         }
         if ($showDeleteControl) {
             $html[] =           '<a href="#"';
-            $html[] =               ' class="btn btn-default t3js-btn-removeoption"';
+            $html[] =               ' class="btn btn-default t3js-btn-option t3js-btn-removeoption t3js-revert-unique"';
             $html[] =               ' data-fieldname="' . htmlspecialchars($elementName) . '"';
+            $html[] =               ' data-uid="' . htmlspecialchars($row['uid']) . '"';
             $html[] =               ' title="' . htmlspecialchars($languageService->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.remove_selected')) . '"';
-            $html[] =               ' onClick="' . $deleteControlOnClick . '"';
             $html[] =           '>';
             $html[] =               $this->iconFactory->getIcon('actions-selection-delete', Icon::SIZE_SMALL)->render();
             $html[] =           '</a>';
@@ -403,6 +367,12 @@ class GroupElement extends AbstractFormElement
         $html[] =   '<input type="hidden" name="' . htmlspecialchars($elementName) . '" value="' . htmlspecialchars(implode(',', $listOfSelectedValues)) . '" />';
         $html[] = '</div>';
 
+        $resultArray['requireJsModules'][] = ['TYPO3/CMS/Backend/FormEngine/Element/GroupElement' => '
+            function(GroupElement) {
+                new GroupElement(' . GeneralUtility::quoteJSvalue($fieldId) . ');
+            }'
+        ];
+
         $resultArray['html'] = implode(LF, $html);
         return $resultArray;
     }
