@@ -224,9 +224,8 @@ class ImageManipulationElement extends AbstractFormElement
         }
         if (MathUtility::canBeInterpretedAsInteger($fileUid)) {
             try {
-                $file = ResourceFactory::getInstance()->getFileObject($fileUid);
-            } catch (FileDoesNotExistException $e) {
-            } catch (\InvalidArgumentException $e) {
+                $file = GeneralUtility::makeInstance(ResourceFactory::class)->getFileObject($fileUid);
+            } catch (FileDoesNotExistException|\InvalidArgumentException $e) {
             }
         }
         return $file;
@@ -328,6 +327,7 @@ class ImageManipulationElement extends AbstractFormElement
      */
     protected function getWizardPayload(array $cropVariants, File $image): array
     {
+        $uriArguments = [];
         $arguments = [
             'cropVariants' => $cropVariants,
             'image' => $image->getUid(),
