@@ -32,6 +32,7 @@ use TYPO3\CMS\Core\Messaging\FlashMessage;
 use TYPO3\CMS\Core\Messaging\FlashMessageQueue;
 use TYPO3\CMS\Core\Messaging\FlashMessageService;
 use TYPO3\CMS\Core\Resource\FileRepository;
+use TYPO3\CMS\Core\Site\SiteFinder;
 use TYPO3\CMS\Core\Type\Bitmask\Permission;
 use TYPO3\CMS\Core\Utility\ArrayUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
@@ -274,15 +275,46 @@ abstract class AbstractItemProvider
                 }
                 break;
             case $special === 'languages':
-                foreach ($result['systemLanguageRows'] as $language) {
-                    if ($language['uid'] !== -1) {
-                        $items[] = [
-                            0 => $language['title'],
-                            1 => $language['uid'],
-                            2 => $language['flagIconIdentifier']
-                        ];
+                $allLanguages = [];
+                if (($result['effectivePid'] ?? 0) === 0) {
+                    // This provides a list of all languages available for ALL sites
+                    // Due to the nature of the "sys_language_uid" field having no meaning currently,
+                    // We preserve the language ID and make a list of all languages
+                    $sites = $this->getAllSites();
+                    foreach ($sites as $site) {
+                        foreach ($site->getAllLanguages() as $language) {
+                            $languageId = $language->getLanguageId();
+                            if (isset($allLanguages[$languageId])) {
+                                // Language already provided by another site, just add the label separately
+                                $allLanguages[$languageId][0] .= ', ' . $language->getTitle() . ' [Site: ' . $site->getIdentifier() . ']';
+                            } else {
+                                $allLanguages[$languageId] = [
+                                    0 => $language->getTitle() . ' [Site: ' . $site->getIdentifier() . ']',
+                                    1 => $languageId,
+                                    2 => $language->getFlagIdentifier()
+                                ];
+                            }
+                        }
+                    }
+                    ksort($allLanguages);
+                }
+                if (!empty($allLanguages)) {
+                    foreach ($allLanguages as $item) {
+                        $items[] = $item;
+                    }
+                } else {
+                    // Happens for non-pid=0 records (e.g. "tt_content"), or when no site was configured
+                    foreach ($result['systemLanguageRows'] as $language) {
+                        if ($language['uid'] !== -1) {
+                            $items[] = [
+                                0 => $language['title'],
+                                1 => $language['uid'],
+                                2 => $language['flagIconIdentifier']
+                            ];
+                        }
                     }
                 }
+
                 break;
             case $special === 'custom':
                 $customOptions = $GLOBALS['TYPO3_CONF_VARS']['BE']['customPermOptions'];
@@ -434,6 +466,8 @@ abstract class AbstractItemProvider
      */
     protected function addItemsFromForeignTable(array $result, $fieldName, array $items)
     {
+        $databaseError = null;
+        $queryResult = null;
         // Guard
         if (empty($result['processedTca']['columns'][$fieldName]['config']['foreign_table'])
             || !is_string($result['processedTca']['columns'][$fieldName]['config']['foreign_table'])
@@ -481,7 +515,6 @@ abstract class AbstractItemProvider
             $labelPrefix = $languageService->sL($labelPrefix);
         }
 
-        $iconFactory = GeneralUtility::makeInstance(IconFactory::class);
         $fileRepository = GeneralUtility::makeInstance(FileRepository::class);
 
         while ($foreignRow = $queryResult->fetch()) {
@@ -507,19 +540,6 @@ abstract class AbstractItemProvider
                         $icon = reset($references);
                         $icon = $icon->getPublicUrl();
                     }
-                } else {
-                    $iconPath = '';
-                    if (!empty($GLOBALS['TCA'][$foreignTable]['ctrl']['selicon_field_path'])) {
-                        $iconPath = $GLOBALS['TCA'][$foreignTable]['ctrl']['selicon_field_path'];
-                    }
-                    if ($iconFieldName && $iconPath && $foreignRow[$iconFieldName]) {
-                        // Prepare the row icon if available
-                        $iParts = GeneralUtility::trimExplode(',', $foreignRow[$iconFieldName], true);
-                        $icon = $iconPath . '/' . trim($iParts[0]);
-                    } else {
-                        // Else, determine icon based on record type, or a generic fallback
-                        $icon = $iconFactory->mapRecordTypeToIconIdentifier($foreignTable, $foreignRow);
-                    }
                 }
                 // Add the item
                 $items[] = [
@@ -981,7 +1001,7 @@ abstract class AbstractItemProvider
 
         if (!empty($foreignTableClauseArray['ORDERBY'])) {
             foreach ($foreignTableClauseArray['ORDERBY'] as $orderPair) {
-                list($fieldName, $order) = $orderPair;
+                [$fieldName, $order] = $orderPair;
                 $queryBuilder->addOrderBy($fieldName, $order);
             }
         } elseif (!empty($GLOBALS['TCA'][$foreignTableName]['ctrl']['default_sortby'])) {
@@ -1069,7 +1089,7 @@ abstract class AbstractItemProvider
         ) {
             $foreignTableClause = $result['processedTca']['columns'][$localFieldName]['config']['foreign_table_where'];
             // Replace possible markers in query
-            if (strstr($foreignTableClause, '###REC_FIELD_')) {
+            if (strpos($foreignTableClause, '###REC_FIELD_') !== false) {
                 // " AND table.field='###REC_FIELD_field1###' AND ..." -> array(" AND table.field='", "field1###' AND ...")
                 $whereClauseParts = explode('###REC_FIELD_', $foreignTableClause);
                 foreach ($whereClauseParts as $key => $value) {
@@ -1295,9 +1315,6 @@ abstract class AbstractItemProvider
         $languageService = $this->getLanguageService();
 
         foreach ($itemArray as $key => $item) {
-            if (!isset($dynamicItems[$key])) {
-                $staticValues[$item[1]] = $item;
-            }
             if (isset($result['pageTsConfig']['TCEFORM.'][$table . '.'][$fieldName . '.']['altLabels.'][$item[1]])
                 && !empty($result['pageTsConfig']['TCEFORM.'][$table . '.'][$fieldName . '.']['altLabels.'][$item[1]])
             ) {
@@ -1367,39 +1384,15 @@ abstract class AbstractItemProvider
         $table = $result['tableName'];
         $row = $result['databaseRow'];
         $uid = $row['uid'];
-        if (!empty($result['processedTca']['ctrl']['versioningWS'])
-            && $result['pid'] === -1
-        ) {
-            if (empty($row['t3ver_oid'])) {
-                throw new \UnexpectedValueException(
-                    'No t3ver_oid found for record ' . $row['uid'] . ' on table ' . $table,
-                    1440066481
-                );
-            }
+        if (BackendUtility::isTableWorkspaceEnabled($table) && (int)$row['t3ver_oid'] > 0) {
             $uid = $row['t3ver_oid'];
         }
         return $uid;
     }
 
-    /**
-     * Determine the static values in the item array
-     *
-     * Used by TcaSelectItems and TcaSelectTreeItems data providers
-     *
-     * @param array $itemArray All item records for the select field
-     * @param array $dynamicItemArray Item records from dynamic sources
-     * @return array
-     * @todo: Check method usage, it's probably bogus in select context and was removed from select tree already.
-     */
-    protected function getStaticValues($itemArray, $dynamicItemArray)
+    protected function getAllSites(): array
     {
-        $staticValues = [];
-        foreach ($itemArray as $key => $item) {
-            if (!isset($dynamicItemArray[$key])) {
-                $staticValues[$item[1]] = $item;
-            }
-        }
-        return $staticValues;
+        return GeneralUtility::makeInstance(SiteFinder::class)->getAllSites();
     }
 
     /**
