@@ -307,7 +307,7 @@ class EvaluateDisplayConditions implements FormDataProviderInterface
                     }
                     $namedConditionArray['operand'] = (int)$operand;
                 } elseif ($namedConditionArray['operator'] === '-' || $namedConditionArray['operator'] === '!-') {
-                    list($minimum, $maximum) = GeneralUtility::trimExplode('-', $operand);
+                    [$minimum, $maximum] = GeneralUtility::trimExplode('-', $operand);
                     if (!MathUtility::canBeInterpretedAsInteger($minimum) || !MathUtility::canBeInterpretedAsInteger($maximum)) {
                         throw new \RuntimeException(
                             'Field condition "' . $conditionString . '" with comparison operator ' . $namedConditionArray['operator']
@@ -393,6 +393,9 @@ class EvaluateDisplayConditions implements FormDataProviderInterface
                     );
                 }
                 $namedConditionArray['uid'] = $databaseRow['uid'];
+                if (array_key_exists('t3ver_oid', $databaseRow)) {
+                    $namedConditionArray['t3ver_oid'] = $databaseRow['t3ver_oid'];
+                }
                 if (array_key_exists('pid', $databaseRow)) {
                     $namedConditionArray['pid'] = $databaseRow['pid'];
                 }
@@ -890,9 +893,7 @@ class EvaluateDisplayConditions implements FormDataProviderInterface
         $isNewRecord = !((int)$condition['uid'] > 0);
         // Detection of version can be done by detecting the workspace of the user
         $isUserInWorkspace = $this->getBackendUser()->workspace > 0;
-        if ((array_key_exists('pid', $condition) && (int)$condition['pid'] === -1)
-            || (array_key_exists('_ORIG_pid', $condition) && (int)$condition['_ORIG_pid'] === -1)
-        ) {
+        if ((int)($condition['t3ver_oid'] ?? 0) > 0) {
             $isRecordDetectedAsVersion = true;
         } else {
             $isRecordDetectedAsVersion = false;
@@ -900,7 +901,7 @@ class EvaluateDisplayConditions implements FormDataProviderInterface
         // New records in a workspace are not handled as a version record
         // if it's no new version, we detect versions like this:
         // * if user is in workspace: always TRUE
-        // * if editor is in live ws: only TRUE if pid == -1
+        // * if editor is in live ws: only TRUE if t3ver_oid > 0
         $result = ($isUserInWorkspace || $isRecordDetectedAsVersion) && !$isNewRecord;
         if (!$condition['isVersion']) {
             $result = !$result;
