@@ -32,17 +32,13 @@ class TcaInlineExpandCollapseState implements FormDataProviderInterface
     public function addData(array $result)
     {
         if (empty($result['inlineExpandCollapseStateArray'])) {
+            $fullInlineState = json_decode($this->getBackendUser()->uc['inlineView'] ?? null, true);
+            if (!is_array($fullInlineState)) {
+                $fullInlineState = [];
+            }
+            $inlineStateForTable = [];
             if (!empty($result['inlineTopMostParentUid']) && !empty($result['inlineTopMostParentTableName'])) {
                 // Happens in inline ajax context, top parent uid and top parent table are set
-                if (isset($this->getBackendUser()->uc['inlineView'])) {
-                    $fullInlineState = unserialize($this->getBackendUser()->uc['inlineView']);
-                    if (!is_array($fullInlineState)) {
-                        $fullInlineState = [];
-                    }
-                } else {
-                    $fullInlineState = [];
-                }
-                $inlineStateForTable = [];
                 if ($result['command'] !== 'new') {
                     $table = $result['inlineTopMostParentTableName'];
                     $uid = $result['inlineTopMostParentUid'];
@@ -50,14 +46,8 @@ class TcaInlineExpandCollapseState implements FormDataProviderInterface
                         $inlineStateForTable = $fullInlineState[$table][$uid];
                     }
                 }
-                $result['inlineExpandCollapseStateArray'] = $inlineStateForTable;
             } else {
                 // Default case for a single record
-                $fullInlineState = !empty($this->getBackendUser()->uc['inlineView']) ? unserialize($this->getBackendUser()->uc['inlineView']) : [];
-                if (!is_array($fullInlineState)) {
-                    $fullInlineState = [];
-                }
-                $inlineStateForTable = [];
                 if ($result['command'] !== 'new') {
                     $table = $result['tableName'];
                     $uid = $result['databaseRow']['uid'];
@@ -65,8 +55,8 @@ class TcaInlineExpandCollapseState implements FormDataProviderInterface
                         $inlineStateForTable = $fullInlineState[$table][$uid];
                     }
                 }
-                $result['inlineExpandCollapseStateArray'] = $inlineStateForTable;
             }
+            $result['inlineExpandCollapseStateArray'] = $inlineStateForTable;
         }
 
         if (!$result['isInlineChildExpanded']) {
