@@ -17,6 +17,7 @@ namespace TYPO3\CMS\Backend\Http;
 
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
+use Psr\Http\Server\RequestHandlerInterface;
 use TYPO3\CMS\Core\Configuration\ConfigurationManager;
 use TYPO3\CMS\Core\Context\Context;
 use TYPO3\CMS\Core\Context\DateTimeAspect;
@@ -24,40 +25,32 @@ use TYPO3\CMS\Core\Context\VisibilityAspect;
 use TYPO3\CMS\Core\Core\Environment;
 use TYPO3\CMS\Core\Http\AbstractApplication;
 use TYPO3\CMS\Core\Http\RedirectResponse;
-use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
  * Entry point for the TYPO3 Backend (HTTP requests)
  */
 class Application extends AbstractApplication
 {
-    /**
-     * @var string
-     */
-    protected $requestHandler = RequestHandler::class;
-
-    /**
-     * @var string
-     */
-    protected $middlewareStack = 'backend';
-
     /**
      * @var ConfigurationManager
      */
     protected $configurationManager;
 
     /**
-     * @param ConfigurationManager $configurationManager
+     * @var Context
      */
-    public function __construct(ConfigurationManager $configurationManager)
-    {
+    protected $context;
+
+    public function __construct(
+        RequestHandlerInterface $requestHandler,
+        ConfigurationManager $configurationManager,
+        Context $context
+    ) {
+        $this->requestHandler = $requestHandler;
         $this->configurationManager = $configurationManager;
+        $this->context = $context;
     }
 
-    /**
-     * @param ServerRequestInterface $request
-     * @return ResponseInterface
-     */
     protected function handle(ServerRequestInterface $request): ResponseInterface
     {
         if (!$this->checkIfEssentialConfigurationExists()) {
@@ -91,13 +84,10 @@ class Application extends AbstractApplication
 
     /**
      * Initializes the Context used for accessing data and finding out the current state of the application
-     * Will be moved to a DI-like concept once introduced, for now, this is a singleton
      */
-    protected function initializeContext()
+    protected function initializeContext(): void
     {
-        GeneralUtility::makeInstance(Context::class, [
-            'date' => new DateTimeAspect(new \DateTimeImmutable('@' . $GLOBALS['EXEC_TIME'])),
-            'visibility' => new VisibilityAspect(true, true)
-        ]);
+        $this->context->setAspect('date', new DateTimeAspect(new \DateTimeImmutable('@' . $GLOBALS['EXEC_TIME'])));
+        $this->context->setAspect('visibility', new VisibilityAspect(true, true));
     }
 }
