@@ -26,6 +26,7 @@ use TYPO3\CMS\Core\Database\Query\Restriction\HiddenRestriction;
 use TYPO3\CMS\Core\Database\Query\Restriction\StartTimeRestriction;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
+use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Type\Bitmask\Permission;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
@@ -197,7 +198,7 @@ class LiveSearch
 
             $orderBy = $GLOBALS['TCA'][$tableName]['ctrl']['sortby'] ?: $GLOBALS['TCA'][$tableName]['ctrl']['default_sortby'];
             foreach (QueryHelper::parseOrderBy((string)$orderBy) as $orderPair) {
-                list($fieldName, $order) = $orderPair;
+                [$fieldName, $order] = $orderPair;
                 $queryBuilder->addOrderBy($fieldName, $order);
             }
 
@@ -279,7 +280,7 @@ class LiveSearch
      */
     protected function getTitleOfCurrentRecordType($tableName)
     {
-        return $GLOBALS['LANG']->sL($GLOBALS['TCA'][$tableName]['ctrl']['title']);
+        return $this->getLanguageService()->sL($GLOBALS['TCA'][$tableName]['ctrl']['title']);
     }
 
     /**
@@ -481,4 +482,12 @@ class LiveSearch
         $tree->ids[] = -1;
         return implode(',', $tree->ids);
     }
+
+    /**
+     * @return LanguageService|null
+     */
+    protected function getLanguageService(): ?LanguageService
+    {
+        return $GLOBALS['LANG'] ?? null;
+    }
 }
