@@ -20,10 +20,11 @@ use TYPO3\CMS\Backend\Backend\Shortcut\ShortcutRepository;
 use TYPO3\CMS\Backend\Routing\UriBuilder;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
-use TYPO3\CMS\Core\Compatibility\PublicPropertyDeprecationTrait;
 use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
+use TYPO3\CMS\Core\Information\Typo3Information;
+use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Page\PageRenderer;
 use TYPO3\CMS\Core\Service\MarkerBasedTemplateService;
 use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
@@ -43,18 +44,12 @@ use TYPO3\CMS\Core\Utility\PathUtility;
  * After this file $LANG and $TBE_TEMPLATE are global variables / instances of their respective classes.
  *
  * Please refer to Inside TYPO3 for a discussion of how to use this API.
+ *
+ * @deprecated will be removed in TYPO3 v11.0. Use ModuleTemplate and PageRenderer instead.
  */
 class DocumentTemplate implements LoggerAwareInterface
 {
     use LoggerAwareTrait;
-    use PublicPropertyDeprecationTrait;
-
-    /**
-     * @var array
-     */
-    protected $deprecatedPublicProperties = [
-        'hasDocheader' => 'Using $hasDocheader of class DocumentTemplate is discouraged. The property is not evaluated in the TYPO3 core anymore and will be removed in TYPO3 v10.0.'
-    ];
 
     // Vars you typically might want to/should set from outside after making instance of this class:
     /**
@@ -66,7 +61,7 @@ class DocumentTemplate implements LoggerAwareInterface
     public $form = '';
 
     /**
-     * Additional header code (eg. a JavaScript section) could be accommulated in this var. It will be directly outputted in the header.
+     * Additional header code (eg. a JavaScript section) could be accumulated in this var. It will be directly outputted in the header.
      *
      * @var string
      */
@@ -79,6 +74,7 @@ class DocumentTemplate implements LoggerAwareInterface
      */
     public $JScodeArray = ['jumpToUrl' => '
 function jumpToUrl(URL) {
+	console.warn(\'jumpToUrl() has been marked as deprecated.\');
 	window.location.href = URL;
 	return false;
 }
@@ -180,28 +176,6 @@ function jumpToUrl(URL) {
         'visual' => 'Resources/Public/Css/visual/'
     ];
 
-    /**
-     * JavaScript files loaded for every page in the Backend
-     *
-     * @var array
-     */
-    protected $jsFiles = [];
-
-    /**
-     * JavaScript files loaded for every page in the Backend, but explicitly excluded from concatenation (useful for libraries etc.)
-     *
-     * @var array
-     */
-    protected $jsFilesNoConcatenation = [];
-
-    /**
-     * Indicates if a <div>-output section is open
-     *
-     * @var int
-     * @internal will be removed in TYPO3 v9
-     */
-    public $sectionFlag = 0;
-
     /**
      * (Default) Class for wrapping <DIV>-tag of page. Is set in class extensions.
      *
@@ -219,12 +193,6 @@ function jumpToUrl(URL) {
      */
     public $endOfPageJsBlock = '';
 
-    /**
-     * @var bool
-     * @deprecated since TYPO3 v9.4, will be removed in TYPO3 v10.0
-     */
-    protected $hasDocheader = true;
-
     /**
      * @var PageRenderer
      */
@@ -259,8 +227,8 @@ function jumpToUrl(URL) {
      */
     public function __construct()
     {
-        // Initializes the page rendering object:
         $this->initPageRenderer();
+        trigger_error(__CLASS__ . ' will be removed in TYPO3 v11. Use ModuleTemplate API instead.', E_USER_DEPRECATED);
 
         $this->iconFactory = GeneralUtility::makeInstance(IconFactory::class);
 
@@ -303,31 +271,13 @@ function jumpToUrl(URL) {
      */
     protected function initPageRenderer()
     {
-        if ($this->pageRenderer !== null) {
-            return;
-        }
         $this->pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
-        $this->pageRenderer->setLanguage($GLOBALS['LANG']->lang);
+        $this->pageRenderer->setLanguage($this->getLanguageService()->lang);
         $this->pageRenderer->enableConcatenateCss();
         $this->pageRenderer->enableConcatenateJavascript();
         $this->pageRenderer->enableCompressCss();
         $this->pageRenderer->enableCompressJavascript();
-        // Add all JavaScript files defined in $this->jsFiles to the PageRenderer
-        foreach ($this->jsFilesNoConcatenation as $file) {
-            $this->pageRenderer->addJsFile(
-                $file,
-                'text/javascript',
-                true,
-                false,
-                '',
-                true
-            );
-        }
-        // Add all JavaScript files defined in $this->jsFiles to the PageRenderer
-        foreach ($this->jsFiles as $file) {
-            $this->pageRenderer->addJsFile($file);
-        }
-        if ((int)$GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] === 1) {
+        if ($GLOBALS['TYPO3_CONF_VARS']['BE']['debug']) {
             $this->pageRenderer->enableDebugMode();
         }
     }
@@ -354,13 +304,6 @@ function jumpToUrl(URL) {
         $gvList = 'route,' . $gvList;
         $storeUrl = $this->makeShortcutUrl($gvList, $setList);
         $pathInfo = parse_url(GeneralUtility::getIndpEnv('REQUEST_URI'));
-        // Fallback for alt_mod. We still pass in the old xMOD... stuff, but TBE_MODULES only knows about "record_edit".
-        // We still need to pass the xMOD name to createShortcut below, since this is used for icons.
-        $moduleName = $modName === 'xMOD_alt_doc.php' ? 'record_edit' : $modName;
-        // Add the module identifier automatically if typo3/index.php is used:
-        if (GeneralUtility::_GET('M') !== null) {
-            $storeUrl = '&M=' . $moduleName . $storeUrl;
-        }
         if ((int)$motherModName === 1) {
             $motherModule = 'top.currentModuleLoaded';
         } elseif ($motherModName) {
@@ -368,7 +311,7 @@ function jumpToUrl(URL) {
         } else {
             $motherModule = '\'\'';
         }
-        $confirmationText = GeneralUtility::quoteJSvalue($GLOBALS['LANG']->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.makeBookmark'));
+        $confirmationText = GeneralUtility::quoteJSvalue($this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.makeBookmark'));
 
         $shortcutUrl = $pathInfo['path'] . '?' . $storeUrl;
         $shortcutRepository = GeneralUtility::makeInstance(ShortcutRepository::class);
@@ -383,7 +326,7 @@ function jumpToUrl(URL) {
             ', ' . $url . ', ' . $confirmationText . ', ' . $motherModule . ', this);return false;';
 
         return '<a href="#" class="' . htmlspecialchars($classes) . '" onclick="' . htmlspecialchars($onClick) . '" title="' .
-        htmlspecialchars($GLOBALS['LANG']->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.makeBookmark')) . '">' .
+        htmlspecialchars($this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.makeBookmark')) . '">' .
         $this->iconFactory->getIcon('actions-system-shortcut-new', Icon::SIZE_SMALL)->render() . '</a>';
     }
 
@@ -407,41 +350,25 @@ function jumpToUrl(URL) {
         return HttpUtility::buildQueryString($storeArray, '&');
     }
 
-    /**
-     * Returns <input> attributes to set the width of an text-type input field.
-     * For client browsers with no CSS support the cols/size attribute is returned.
-     * For CSS compliant browsers (recommended) a ' style="width: ...px;"' is returned.
-     *
-     * @param int $size A relative number which multiplied with approx. 10 will lead to the width in pixels
-     * @param bool $textarea A flag you can set for textareas - DEPRECATED as there is no difference any more between the two
-     * @param string $styleOverride A string which will be returned as attribute-value for style="" instead of the calculated width (if CSS is enabled)
-     * @return string Tag attributes for an <input> tag (regarding width)
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0.
-     */
-    public function formWidth($size = 48, $textarea = false, $styleOverride = '')
-    {
-        trigger_error('DocumentTemplate->formWidth() will be removed in TYPO3 10.0 - use responsive code or direct inline styles to format your input fields instead.', E_USER_DEPRECATED);
-        return ' style="' . ($styleOverride ?: 'width:' . ceil($size * 9.58) . 'px;') . '"';
-    }
-
     /**
      * Returns JavaScript variables setting the returnUrl and thisScript location for use by JavaScript on the page.
      * Used in fx. db_list.php (Web>List)
      *
      * @param string $thisLocation URL to "this location" / current script
      * @return string Urls are returned as JavaScript variables T3_RETURN_URL and T3_THIS_LOCATION
-     * @see typo3/db_list.php
      */
     public function redirectUrls($thisLocation = '')
     {
-        $thisLocation = $thisLocation ? $thisLocation : GeneralUtility::linkThisScript([
+        $thisLocation = $thisLocation ?: GeneralUtility::linkThisScript([
             'CB' => '',
             'SET' => '',
             'cmd' => '',
             'popViewId' => ''
         ]);
         $out = '
+	// @deprecated
 	var T3_RETURN_URL = ' . GeneralUtility::quoteJSvalue(str_replace('%20', '', rawurlencode(GeneralUtility::sanitizeLocalUrl(GeneralUtility::_GP('returnUrl'))))) . ';
+	// @deprecated
 	var T3_THIS_LOCATION = ' . GeneralUtility::quoteJSvalue(str_replace('%20', '', rawurlencode($thisLocation))) . '
 		';
         return $out;
@@ -492,12 +419,10 @@ function jumpToUrl(URL) {
         $this->pageRenderer->setRenderXhtml(false);
 
         $languageCode = $this->pageRenderer->getLanguage() === 'default' ? 'en' : $this->pageRenderer->getLanguage();
-        $this->pageRenderer->setHtmlTag('<html lang="' . $languageCode . '">');
+        $this->pageRenderer->setHtmlTag('<html lang="' . htmlspecialchars($languageCode) . '">');
 
         $headerStart = '<!DOCTYPE html>';
         $this->pageRenderer->setXmlPrologAndDocType($headerStart);
-        $this->pageRenderer->setHeadTag('<head>' . LF . '<!-- TYPO3 Script ID: ' . htmlspecialchars($this->scriptID) . ' -->');
-        header('Content-Type:text/html;charset=utf-8');
         $this->pageRenderer->setCharSet('utf-8');
         $this->pageRenderer->setMetaTag('name', 'generator', $this->generator());
         $this->pageRenderer->setMetaTag('name', 'robots', 'noindex,follow');
@@ -631,22 +556,6 @@ function jumpToUrl(URL) {
         }
     }
 
-    /**
-     * Insert additional style sheet link
-     *
-     * @param string $key some key identifying the style sheet
-     * @param string $href uri to the style sheet file
-     * @param string $title value for the title attribute of the link element
-     * @param string $relation value for the rel attribute of the link element
-     * @deprecated since TYPO3 v9.4, will be removed in TYPO3 v10.0
-     * @see PageRenderer::addCssFile()
-     */
-    public function addStyleSheet($key, $href, $title = '', $relation = 'stylesheet')
-    {
-        trigger_error('DocumentTemplate->->addStyleSheet() will be removed in TYPO3 v10.0.', E_USER_DEPRECATED);
-        $this->pageRenderer->addCssFile($href, $relation, 'screen', $title);
-    }
-
     /**
      * Add all *.css files of the directory $path to the stylesheets
      *
@@ -656,15 +565,16 @@ function jumpToUrl(URL) {
     {
         $path = GeneralUtility::getFileAbsFileName($path);
         // Read all files in directory and sort them alphabetically
-        $cssFiles = GeneralUtility::getFilesInDir($path, 'css');
-        foreach ($cssFiles as $cssFile) {
-            $this->pageRenderer->addCssFile(PathUtility::getRelativePathTo($path) . $cssFile);
+        foreach (GeneralUtility::getFilesInDir($path, 'css', true) as $cssFile) {
+            $this->pageRenderer->addCssFile($cssFile);
         }
     }
 
     /**
      * Insert post rendering document style into already rendered content
-     * This is needed for extobjbase
+     *
+     * @todo: Review this, it may be obsolete or could be done differently?
+     * @todo: Previous comment was: "This is needed for extobjbase" (AbstractFunctionModule)
      *
      * @param string $content style-content to insert.
      * @return string content with inserted styles
@@ -695,7 +605,7 @@ function jumpToUrl(URL) {
             foreach ($GLOBALS['TBE_STYLES']['skins'] as $skinExtKey => $skin) {
                 $skinStylesheetDirs = $this->stylesheetsSkins;
                 // Skins can add custom stylesheetDirectories using
-                // $GLOBALS['TBE_STYLES']['skins'][$_EXTKEY]['stylesheetDirectories']
+                // $GLOBALS['TBE_STYLES']['skins']['your_extension_key']['stylesheetDirectories']
                 if (is_array($skin['stylesheetDirectories'])) {
                     $skinStylesheetDirs = array_merge($skinStylesheetDirs, $skin['stylesheetDirectories']);
                 }
@@ -703,7 +613,7 @@ function jumpToUrl(URL) {
                 foreach ($skinStylesheetDirs as $stylesheetDir) {
                     // for EXT:myskin/stylesheets/ syntax
                     if (strpos($stylesheetDir, 'EXT:') === 0) {
-                        list($extKey, $path) = explode('/', substr($stylesheetDir, 4), 2);
+                        [$extKey, $path] = explode('/', substr($stylesheetDir, 4), 2);
                         if (!empty($extKey) && ExtensionManagementUtility::isLoaded($extKey) && !empty($path)) {
                             $stylesheetDirectories[] = ExtensionManagementUtility::extPath($extKey) . $path;
                         }
@@ -724,20 +634,7 @@ function jumpToUrl(URL) {
      */
     public function generator()
     {
-        return 'TYPO3 CMS, ' . TYPO3_URL_GENERAL . ', &#169; Kasper Sk&#229;rh&#248;j ' . TYPO3_copyright_year . ', extensions are copyright of their respective owners.';
-    }
-
-    /**
-     * Returns X-UA-Compatible meta tag
-     * @deprecated since TYPO3 v9, will be removed in TYPO3 v10.0.
-     *
-     * @param string $content Content of the compatible tag (default: IE-8)
-     * @return string <meta http-equiv="X-UA-Compatible" content="???" />
-     */
-    public function xUaCompatible($content = 'IE=8')
-    {
-        trigger_error('DocumentTemplate->xUaCompatible() will be removed with TYPO3 v10.0. Use PageRenderer->setMetaTag() instead.', E_USER_DEPRECATED);
-        return '<meta http-equiv="X-UA-Compatible" content="' . $content . '" />';
+        return GeneralUtility::makeInstance(Typo3Information::class)->getHtmlGeneratorTagContent();
     }
 
     /*****************************************
@@ -908,7 +805,7 @@ function jumpToUrl(URL) {
             $title = '';
         }
         // Setting the path of the page
-        $pagePath = htmlspecialchars($GLOBALS['LANG']->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.path')) . ': <span class="typo3-docheader-pagePath">';
+        $pagePath = htmlspecialchars($this->getLanguageService()->sL('LLL:EXT:core/Resources/Private/Language/locallang_core.xlf:labels.path')) . ': <span class="typo3-docheader-pagePath">';
         // crop the title to title limit (or 50, if not defined)
         $cropLength = empty($GLOBALS['BE_USER']->uc['titleLen']) ? 50 : $GLOBALS['BE_USER']->uc['titleLen'];
         $croppedTitle = GeneralUtility::fixed_lgd_cs($title, -$cropLength);
@@ -1004,4 +901,12 @@ function jumpToUrl(URL) {
     {
         return $GLOBALS['BE_USER'] ?? null;
     }
+
+    /**
+     * @return LanguageService|null
+     */
+    protected function getLanguageService(): ?LanguageService
+    {
+        return $GLOBALS['LANG'] ?? null;
+    }
 }
