@@ -18,54 +18,27 @@ use TYPO3\CMS\Backend\Backend\Shortcut\ShortcutRepository;
 use TYPO3\CMS\Backend\Template\Components\DocHeaderComponent;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
+use TYPO3\CMS\Core\Configuration\ExtensionConfiguration;
 use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
 use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Messaging\AbstractMessage;
+use TYPO3\CMS\Core\Messaging\FlashMessageQueue;
 use TYPO3\CMS\Core\Messaging\FlashMessageService;
 use TYPO3\CMS\Core\Page\PageRenderer;
 use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\HttpUtility;
-use TYPO3\CMS\Fluid\View\Exception\InvalidTemplateResourceException;
+use TYPO3\CMS\Core\Utility\PathUtility;
 use TYPO3\CMS\Fluid\View\StandaloneView;
+use TYPO3Fluid\Fluid\View\Exception\InvalidTemplateResourceException;
 
 /**
  * A class taking care of the "outer" HTML of a module, especially
  * the doc header and other related parts.
- *
- * @internal This API is not yet carved in stone and may be adapted later.
  */
 class ModuleTemplate
 {
-    /**
-     * Error Icon Constant
-     *
-     * @internal
-     */
-    const STATUS_ICON_ERROR = 3;
-
-    /**
-     * Warning Icon Constant
-     *
-     * @internal
-     */
-    const STATUS_ICON_WARNING = 2;
-
-    /**
-     * Notification Icon Constant
-     *
-     * @internal
-     */
-    const STATUS_ICON_NOTIFICATION = 1;
-
-    /**
-     * OK Icon Constant
-     *
-     * @internal
-     */
-    const STATUS_ICON_OK = -1;
-
     /**
      * DocHeaderComponent
      *
@@ -142,6 +115,11 @@ class ModuleTemplate
      */
     protected $iconFactory;
 
+    /**
+     * @var FlashMessageService
+     */
+    protected $flashMessageService;
+
     /**
      * Module ID
      *
@@ -173,7 +151,7 @@ class ModuleTemplate
     /**
      * Flash message queue
      *
-     * @var \TYPO3\CMS\Core\Messaging\FlashMessageQueue
+     * @var FlashMessageQueue
      */
     protected $flashMessageQueue;
 
@@ -247,18 +225,25 @@ class ModuleTemplate
      * Class constructor
      * Sets up view and property objects
      *
+     * @param PageRenderer $pageRenderer
+     * @param IconFactory $iconFactory
+     * @param FlashMessageService $flashMessageService
      * @throws InvalidTemplateResourceException In case a template is invalid
      */
-    public function __construct()
-    {
+    public function __construct(
+        PageRenderer $pageRenderer,
+        IconFactory $iconFactory,
+        FlashMessageService $flashMessageService
+    ) {
         $this->view = GeneralUtility::makeInstance(StandaloneView::class);
         $this->view->setPartialRootPaths($this->partialRootPaths);
         $this->view->setTemplateRootPaths($this->templateRootPaths);
         $this->view->setLayoutRootPaths($this->layoutRootPaths);
         $this->view->setTemplate($this->templateFile);
-        $this->pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
+        $this->pageRenderer = $pageRenderer;
+        $this->iconFactory = $iconFactory;
+        $this->flashMessageService = $flashMessageService;
         $this->docHeaderComponent = GeneralUtility::makeInstance(DocHeaderComponent::class);
-        $this->iconFactory = GeneralUtility::makeInstance(IconFactory::class);
     }
 
     /**
@@ -269,7 +254,6 @@ class ModuleTemplate
         $this->pageRenderer->loadRequireJsModule('bootstrap');
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/ContextHelp');
         $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/DocumentHeader');
-        $this->pageRenderer->loadRequireJsModule('TYPO3/CMS/Backend/SplitButtons');
     }
 
     /**
@@ -283,6 +267,30 @@ class ModuleTemplate
         if (!empty($GLOBALS['TBE_STYLES']['stylesheet2'])) {
             $this->pageRenderer->addCssFile($GLOBALS['TBE_STYLES']['stylesheet2']);
         }
+        // Add all *.css files of the directory $path to the stylesheets
+        foreach ($this->getRegisteredStylesheetFolders() as $folder) {
+            // Read all files in directory and sort them alphabetically
+            foreach (GeneralUtility::getFilesInDir($folder, 'css', true) as $cssFile) {
+                $this->pageRenderer->addCssFile($cssFile);
+            }
+        }
+    }
+
+    /**
+     * Returns an array of all stylesheet directories registered via $TBE_STYLES['skins']
+     */
+    protected function getRegisteredStylesheetFolders(): array
+    {
+        $stylesheetDirectories = [];
+        foreach ($GLOBALS['TBE_STYLES']['skins'] ?? [] as $skin) {
+            foreach ($skin['stylesheetDirectories'] ?? [] as $stylesheetDir) {
+                $directory = GeneralUtility::getFileAbsFileName($stylesheetDir);
+                if (!empty($directory)) {
+                    $stylesheetDirectories[] = $directory;
+                }
+            }
+        }
+        return $stylesheetDirectories;
     }
 
     /**
@@ -293,8 +301,18 @@ class ModuleTemplate
         // Yes, hardcoded on purpose
         $this->pageRenderer->setXmlPrologAndDocType('<!DOCTYPE html>');
         $this->pageRenderer->setCharSet('utf-8');
-        $this->pageRenderer->setLanguage($GLOBALS['LANG']->lang);
+        $this->pageRenderer->setLanguage($this->getLanguageService()->lang);
         $this->pageRenderer->setMetaTag('name', 'viewport', 'width=device-width, initial-scale=1');
+        $this->pageRenderer->setFavIcon($this->getBackendFavicon());
+        $this->pageRenderer->enableConcatenateCss();
+        $this->pageRenderer->enableConcatenateJavascript();
+        $this->pageRenderer->enableCompressCss();
+        $this->pageRenderer->enableCompressJavascript();
+        $languageCode = $this->pageRenderer->getLanguage() === 'default' ? 'en' : $this->pageRenderer->getLanguage();
+        $this->pageRenderer->setHtmlTag('<html lang="' . htmlspecialchars($languageCode) . '">');
+        if ($GLOBALS['TYPO3_CONF_VARS']['BE']['debug']) {
+            $this->pageRenderer->enableDebugMode();
+        }
     }
 
     /**
@@ -479,24 +497,6 @@ class ModuleTemplate
      * be prepared to change this later again.
      *******************************************/
 
-    /**
-     * Includes a javascript library that exists in the core /typo3/ directory
-     *
-     * @param string $lib Library name. Call it with the full path like
-     * "sysext/core/Resources/Public/JavaScript/QueryGenerator.js" to load it
-     *
-     * @internal
-     * @return self
-     * @deprecated since TYPO3 v9.4, will be removed in TYPO3 v10.0, use PageRenderer directly.
-     * @see \TYPO3\CMS\Core\Page\PageRenderer::addJsFile
-     */
-    public function loadJavascriptLib($lib): self
-    {
-        trigger_error('ModuleTemplate->loadJavascriptLib() should not be used any longer, as this method will be removed in TYPO3 v10.0.', E_USER_DEPRECATED);
-        $this->pageRenderer->addJsFile($lib);
-        return $this;
-    }
-
     /**
      * Returns a linked shortcut-icon which will call the shortcut frame and set a
      * shortcut there back to the calling page/module
@@ -586,6 +586,48 @@ class ModuleTemplate
         return HttpUtility::buildQueryString($storeArray, '&');
     }
 
+    /**
+     * Retrieves configured favicon for backend (with fallback)
+     *
+     * @return string
+     */
+    protected function getBackendFavicon()
+    {
+        $backendFavicon = GeneralUtility::makeInstance(ExtensionConfiguration::class)->get('backend', 'backendFavicon');
+        if (!empty($backendFavicon)) {
+            $path = $this->getUriForFileName($backendFavicon);
+        } else {
+            $path = ExtensionManagementUtility::extPath('backend') . 'Resources/Public/Icons/favicon.ico';
+        }
+        return PathUtility::getAbsoluteWebPath($path);
+    }
+
+    /**
+     * Returns the uri of a relative reference, resolves the "EXT:" prefix
+     * (way of referring to files inside extensions) and checks that the file is inside
+     * the project root of the TYPO3 installation
+     *
+     * @param string $filename The input filename/filepath to evaluate
+     * @return string Returns the filename of $filename if valid, otherwise blank string.
+     */
+    protected function getUriForFileName($filename)
+    {
+        if (strpos($filename, '://')) {
+            return $filename;
+        }
+        $urlPrefix = '';
+        if (strpos($filename, 'EXT:') === 0) {
+            $absoluteFilename = GeneralUtility::getFileAbsFileName($filename);
+            $filename = '';
+            if ($absoluteFilename !== '') {
+                $filename = PathUtility::getAbsoluteWebPath($absoluteFilename);
+            }
+        } elseif (strpos($filename, '/') !== 0) {
+            $urlPrefix = GeneralUtility::getIndpEnv('TYPO3_SITE_PATH');
+        }
+        return $urlPrefix . $filename;
+    }
+
     /**
      * Returns the BE USER Object
      *
@@ -606,66 +648,26 @@ class ModuleTemplate
         return $GLOBALS['LANG'];
     }
 
-    /**
-     * Returns an image-tag with an 18x16 icon of the following types:
-     *
-     * $type:
-     * -1:»   OK icon (Check-mark)
-     * 1:»   Notice (Speach-bubble)
-     * 2:»   Warning (Yellow triangle)
-     * 3:»   Fatal error (Red stop sign)
-     *
-     * @param int $type See description
-     *
-     * @return string HTML image tag (if applicable)
-     * @internal
-     * @deprecated since TYPO3 v9.4, will be removed in TYPO3 v10.0
-     */
-    public function icons($type)
-    {
-        trigger_error('ModuleTemplate->icons() should not be used any longer, as this will be removed in TYPO3 v10.0.', E_USER_DEPRECATED);
-        $icon = '';
-        switch ($type) {
-            case self::STATUS_ICON_ERROR:
-                $icon = 'status-dialog-error';
-                break;
-            case self::STATUS_ICON_WARNING:
-                $icon = 'status-dialog-warning';
-                break;
-            case self::STATUS_ICON_NOTIFICATION:
-                $icon = 'status-dialog-notification';
-                break;
-            case self::STATUS_ICON_OK:
-                $icon = 'status-dialog-ok';
-                break;
-            default:
-                // Do nothing
-        }
-        if ($icon != '') {
-            return $this->iconFactory->getIcon($icon, Icon::SIZE_SMALL)->render();
-        }
-        return '';
-    }
-
     /**
      * Returns JavaScript variables setting the returnUrl and thisScript location for use by JavaScript on the page.
      * Used in fx. db_list.php (Web>List)
      *
      * @param string $thisLocation URL to "this location" / current script
      * @return string Urls are returned as JavaScript variables T3_RETURN_URL and T3_THIS_LOCATION
-     * @see typo3/db_list.php
      * @internal
      */
     public function redirectUrls($thisLocation = '')
     {
-        $thisLocation = $thisLocation ? $thisLocation : GeneralUtility::linkThisScript([
+        $thisLocation = $thisLocation ?: GeneralUtility::linkThisScript([
             'CB' => '',
             'SET' => '',
             'cmd' => '',
             'popViewId' => ''
         ]);
         $out = '
+	// @deprecated
 	var T3_RETURN_URL = ' . GeneralUtility::quoteJSvalue(str_replace('%20', '', rawurlencode(GeneralUtility::sanitizeLocalUrl(GeneralUtility::_GP('returnUrl'))))) . ';
+	// @deprecated
 	var T3_THIS_LOCATION = ' . GeneralUtility::quoteJSvalue(str_replace('%20', '', rawurlencode($thisLocation))) . '
 		';
         return $out;
@@ -716,7 +718,7 @@ class ModuleTemplate
     }
 
     /**
-     * @param \TYPO3\CMS\Core\Messaging\FlashMessageQueue $flashMessageQueue
+     * @param FlashMessageQueue $flashMessageQueue
      * @return self
      */
     public function setFlashMessageQueue($flashMessageQueue): self
@@ -726,14 +728,12 @@ class ModuleTemplate
     }
 
     /**
-     * @return \TYPO3\CMS\Core\Messaging\FlashMessageQueue
+     * @return FlashMessageQueue
      */
     protected function getFlashMessageQueue()
     {
         if (!isset($this->flashMessageQueue)) {
-            /** @var FlashMessageService $service */
-            $service = GeneralUtility::makeInstance(FlashMessageService::class);
-            $this->flashMessageQueue = $service->getMessageQueueByIdentifier();
+            $this->flashMessageQueue = $this->flashMessageService->getMessageQueueByIdentifier();
         }
         return $this->flashMessageQueue;
     }
