@@ -10,4 +10,4 @@
  *
  * The TYPO3 project - inspiring people to share!
  */
-define(["require","exports"],function(e,E){"use strict";var r;Object.defineProperty(E,"__esModule",{value:!0}),(r=E.KeyTypesEnum||(E.KeyTypesEnum={}))[r.ENTER=13]="ENTER",r[r.ESCAPE=27]="ESCAPE"});
\ No newline at end of file
+define(["require","exports"],(function(E,e){"use strict";Object.defineProperty(e,"__esModule",{value:!0}),function(E){E[E.ENTER=13]="ENTER",E[E.ESCAPE=27]="ESCAPE",E[E.SPACE=32]="SPACE",E[E.END=35]="END",E[E.HOME=36]="HOME",E[E.LEFT=37]="LEFT",E[E.UP=38]="UP",E[E.RIGHT=39]="RIGHT",E[E.DOWN=40]="DOWN"}(e.KeyTypesEnum||(e.KeyTypesEnum={}))}));
\ No newline at end of file
