@@ -21,22 +21,24 @@
  */
 
 // add legacy functions to be accessible in the global scope
-var setFormValueOpenBrowser,
-  setFormValueFromBrowseWin,
-  setHiddenFromList,
-  setFormValueManipulate,
-  setFormValue_getFObj;
+var setFormValueOpenBrowser, // @deprecated
+  setFormValueFromBrowseWin, // @deprecated
+  setHiddenFromList, // @deprecated
+  setFormValueManipulate, // @deprecated
+  setFormValue_getFObj; // @deprecated
 
 /**
  * Module: TYPO3/CMS/Backend/FormEngine
  */
 define(['jquery',
   'TYPO3/CMS/Backend/FormEngineValidation',
+  'TYPO3/CMS/Backend/DocumentSaveActions',
   'TYPO3/CMS/Backend/Modal',
+  'TYPO3/CMS/Backend/Utility/MessageUtility',
   'TYPO3/CMS/Backend/Severity',
   'TYPO3/CMS/Backend/BackendException',
   'TYPO3/CMS/Backend/Event/InteractionRequestMap'
-], function($, FormEngineValidation, Modal, Severity, BackendException, InteractionRequestMap) {
+], function($, FormEngineValidation, DocumentSaveActions, Modal, MessageUtility, Severity, BackendException, InteractionRequestMap) {
 
   /**
    * @param {InteractionRequest} interactionRequest
@@ -65,16 +67,26 @@ define(['jquery',
     browserUrl: ''
   };
 
-  // functions to connect the db/file browser with this document and the formfields on it!
+  /**
+   * Opens a popup window with the element browser (browser.php)
+   *
+   * @param {string} mode can be "db" or "file"
+   * @param {string} params additional params for the browser window
+   * @deprecated since TYPO3 v10.2, will be removed in TYPO3 v11.0. Use FormEngine.openPopupWindow() instead.
+   */
+  setFormValueOpenBrowser = function(mode, params) {
+    console.warn('setFormValueOpenBrowser() has been marked as deprecated and will be removed in TYPO3 v11. Use FormEngine.openPopupWindow instead.');
+    return FormEngine.openPopupWindow(mode, params);
+  };
 
   /**
    * Opens a popup window with the element browser (browser.php)
    *
-   * @param {String} mode can be "db" or "file"
-   * @param {String} params additional params for the browser window
+   * @param {string} mode can be "db" or "file"
+   * @param {string} params additional params for the browser window
    */
-  FormEngine.openPopupWindow = setFormValueOpenBrowser = function(mode, params) {
-    Modal.advanced({
+  FormEngine.openPopupWindow = function(mode, params) {
+    return Modal.advanced({
       type: Modal.types.iframe,
       content: FormEngine.browserUrl + '&mode=' + mode + '&bparams=' + params,
       size: Modal.sizes.large
@@ -86,14 +98,35 @@ define(['jquery',
    * or from a multi-select (two selects side-by-side)
    * previously known as "setFormValueFromBrowseWin"
    *
-   * @param {String} fieldName Formerly known as "fName" name of the field, like [tt_content][2387][header]
-   * @param {(String|Number)} value The value to fill in (could be an integer)
-   * @param {String} label The visible name in the selector
-   * @param {String} title The title when hovering over it
-   * @param {String} exclusiveValues If the select field has exclusive options that are not combine-able
+   * @param {string} fieldName Formerly known as "fsetFormValueFromBrowseWinName" name of the field, like [tt_content][2387][header]
+   * @param {string|number} value The value to fill in (could be an integer)
+   * @param {string} label The visible name in the selector
+   * @param {string} title The title when hovering over it
+   * @param {string} exclusiveValues If the select field has exclusive options that are not combine-able
+   * @param {$} $optionEl The jQuery object of the selected <option> tag
+   * @deprecated since TYPO3 v10.2, will be removed in TYPO3 v11.0. Use FormEngine.setSelectOptionFromExternalSource() instead.
+   */
+  setFormValueFromBrowseWin = function(fieldName, value, label, title, exclusiveValues, $optionEl) {
+    console.warn(
+      'setFormValueFromBrowseWin() has been marked as deprecated and will be removed in TYPO3 v11. '
+      + 'Use FormEngine.setSelectOptionFromExternalSource() or send a message instead.'
+    );
+    FormEngine.setSelectOptionFromExternalSource(fieldName, value, label, title, exclusiveValues, $optionEl);
+  };
+
+  /**
+   * properly fills the select field from the popup window (element browser, link browser)
+   * or from a multi-select (two selects side-by-side)
+   * previously known as "setFormValueFromBrowseWin"
+   *
+   * @param {string} fieldName Formerly known as "fsetFormValueFromBrowseWinName" name of the field, like [tt_content][2387][header]
+   * @param {string|number} value The value to fill in (could be an integer)
+   * @param {string} label The visible name in the selector
+   * @param {string} title The title when hovering over it
+   * @param {string} exclusiveValues If the select field has exclusive options that are not combine-able
    * @param {$} $optionEl The jQuery object of the selected <option> tag
    */
-  FormEngine.setSelectOptionFromExternalSource = setFormValueFromBrowseWin = function(fieldName, value, label, title, exclusiveValues, $optionEl) {
+  FormEngine.setSelectOptionFromExternalSource = function(fieldName, value, label, title, exclusiveValues, $optionEl) {
     exclusiveValues = String(exclusiveValues);
 
     var $fieldEl,
@@ -225,8 +258,21 @@ define(['jquery',
    *
    * @param {HTMLElement} selectFieldEl the select field
    * @param {HTMLElement} originalFieldEl the hidden form field
+   * @deprecated since TYPO3 v10.2, will be removed in TYPO3 v11.0. Use FormEngine.updateHiddenFieldValueFromSelect() instead.
    */
-  FormEngine.updateHiddenFieldValueFromSelect = setHiddenFromList = function(selectFieldEl, originalFieldEl) {
+  setHiddenFromList = function(selectFieldEl, originalFieldEl) {
+    console.warn('setHiddenFromList() has been marked as deprecated and will be removed in TYPO3 v11. Use FormEngine.updateHiddenFieldValueFromSelect() instead.');
+    FormEngine.updateHiddenFieldValueFromSelect(selectFieldEl, originalFieldEl);
+  };
+
+  /**
+   * sets the value of the hidden field, from the select list, always executed after the select field was updated
+   * previously known as global function setHiddenFromList()
+   *
+   * @param {HTMLElement} selectFieldEl the select field
+   * @param {HTMLElement} originalFieldEl the hidden form field
+   */
+  FormEngine.updateHiddenFieldValueFromSelect = function(selectFieldEl, originalFieldEl) {
     var selectedValues = [];
     $(selectFieldEl).find('option').each(function() {
       selectedValues.push($(this).prop('value'));
@@ -243,8 +289,10 @@ define(['jquery',
    * @param {String} fName
    * @param {String} type
    * @param {Number} maxLength
+   * @deprecated since TYPO3 v10.2, will be removed in TYPO3 v11.0.
    */
   setFormValueManipulate = function(fName, type, maxLength) {
+    console.warn('setFormValueManipulate() has been marked as deprecated and will be removed in TYPO3 v11.');
     var $formEl = FormEngine.getFormElement(fName);
     if ($formEl.length > 0) {
       var formObj = $formEl.get(0);
@@ -418,8 +466,10 @@ define(['jquery',
    *
    * @param {String} fieldName the name of the field name
    * @returns {*|HTMLElement}
+   * @deprecated since TYPO3 v10.2, will be removed in TYPO3 v11.0. Use FormEngine.getFormElement() instead.
    */
   setFormValue_getFObj = function(fieldName) {
+    console.warn('setFormValue_getFObj() has been marked as deprecated and will be removed in TYPO3 v11. Use FormEngine.getFormElement() instead.');
     var $formEl = FormEngine.getFormElement(fieldName);
     if ($formEl.length > 0) {
       // return the DOM element of the form object
@@ -470,7 +520,7 @@ define(['jquery',
    * @returns {*|HTMLElement}
    */
   FormEngine.getFieldElement = function(fieldName, appendix, noFallback) {
-    var $formEl = FormEngine.getFormElement();
+    var $formEl = $('form[name="' + FormEngine.formName + '"]:first');
 
     // if an appendix is set, return the field with the appendix (like _mul or _list)
     if (appendix) {
@@ -495,106 +545,6 @@ define(['jquery',
     return $(':input[name="' + fieldName + '"]', $formEl);
   };
 
-
-  /**************************************************
-   * manipulate existing options in a select field
-   **************************************************/
-
-  /**
-   * Moves currently selected options from a select field to the very top,
-   * can be multiple entries as well
-   *
-   * @param {Object} $fieldEl a jQuery object, containing the select field
-   */
-  FormEngine.moveOptionToTop = function($fieldEl) {
-    // remove the selected options
-    var selectedOptions = $fieldEl.find(':selected').detach();
-    // and add them on first position again
-    $fieldEl.prepend(selectedOptions);
-  };
-
-
-  /**
-   * moves currently selected options from a select field up by one position,
-   * can be multiple entries as well
-   *
-   * @param {Object} $fieldEl a jQuery object, containing the select field
-   */
-  FormEngine.moveOptionUp = function($fieldEl) {
-    // remove the selected options and add it before the previous sibling
-    $.each($fieldEl.find(':selected'), function(k, optionEl) {
-      var $optionEl = $(optionEl)
-        , $optionBefore = $optionEl.prev();
-
-      // stop if first option to move is already the first one
-      if (k == 0 && $optionBefore.length === 0) {
-        return false;
-      }
-
-      $optionBefore.before($optionEl.detach());
-    });
-  };
-
-
-  /**
-   * moves currently selected options from a select field down one position,
-   * can be multiple entries as well
-   *
-   * @param {Object} $fieldEl a jQuery object, containing the select field
-   */
-  FormEngine.moveOptionDown = function($fieldEl) {
-    // remove the selected options and add it after the next sibling
-    // however, this time, we need to go from the last to the first
-    var selectedOptions = $fieldEl.find(':selected');
-    selectedOptions = $.makeArray(selectedOptions);
-    selectedOptions.reverse();
-    $.each(selectedOptions, function(k, optionEl) {
-      var $optionEl = $(optionEl)
-        , $optionAfter = $optionEl.next();
-
-      // stop if first option to move is already the last one
-      if (k == 0 && $optionAfter.length === 0) {
-        return false;
-      }
-
-      $optionAfter.after($optionEl.detach());
-    });
-  };
-
-
-  /**
-   * moves currently selected options from a select field as the very last entries
-   *
-   * @param {Object} $fieldEl a jQuery object, containing the select field
-   */
-  FormEngine.moveOptionToBottom = function($fieldEl) {
-    // remove the selected options
-    var selectedOptions = $fieldEl.find(':selected').detach();
-    // and add them on last position again
-    $fieldEl.append(selectedOptions);
-  };
-
-  /**
-   * removes currently selected options from a select field
-   *
-   * @param {Object} $fieldEl a jQuery object, containing the select field
-   * @param {Object} $availableFieldEl a jQuery object, containing all available value
-   */
-  FormEngine.removeOption = function($fieldEl, $availableFieldEl) {
-    var $selected = $fieldEl.find(':selected');
-
-    $selected.each(function() {
-      $availableFieldEl
-        .find('option[value="' + $.escapeSelector($(this).attr('value')) + '"]')
-        .removeClass('hidden')
-        .prop('disabled', false);
-    });
-
-    // remove the selected options
-    $selected.remove();
-  };
-
-
   /**
    * Initialize events for all form engine relevant tasks.
    * This function only needs to be called once on page load,
@@ -607,51 +557,7 @@ define(['jquery',
         top.TYPO3.Backend.consumerScope.detach(FormEngine);
       });
     }
-    $(document).on('click', '.t3js-btn-moveoption-top, .t3js-btn-moveoption-up, .t3js-btn-moveoption-down, .t3js-btn-moveoption-bottom, .t3js-btn-removeoption', function(evt) {
-      evt.preventDefault();
-
-      // track the arrows "Up", "Down", "Clear" etc in multi-select boxes
-      var $el = $(this)
-        , fieldName = $el.data('fieldname')
-        , $listFieldEl = FormEngine.getFieldElement(fieldName, '_list');
-
-      if ($listFieldEl.length > 0) {
-
-        if ($el.hasClass('t3js-btn-moveoption-top')) {
-          FormEngine.moveOptionToTop($listFieldEl);
-        } else if ($el.hasClass('t3js-btn-moveoption-up')) {
-          FormEngine.moveOptionUp($listFieldEl);
-        } else if ($el.hasClass('t3js-btn-moveoption-down')) {
-          FormEngine.moveOptionDown($listFieldEl);
-        } else if ($el.hasClass('t3js-btn-moveoption-bottom')) {
-          FormEngine.moveOptionToBottom($listFieldEl);
-        } else if ($el.hasClass('t3js-btn-removeoption')) {
-          var $availableFieldEl = FormEngine.getFieldElement(fieldName, '_avail');
-          FormEngine.removeOption($listFieldEl, $availableFieldEl);
-        }
-
-        // make sure to update the hidden field value when modifying the select value
-        FormEngine.updateHiddenFieldValueFromSelect($listFieldEl, FormEngine.getFieldElement(fieldName));
-        FormEngine.legacyFieldChangedCb();
-        if (typeof FormEngine.Validation !== 'undefined' && typeof FormEngine.Validation.validate === 'function') {
-          FormEngine.Validation.markFieldAsChanged($listFieldEl);
-          FormEngine.Validation.validate();
-        }
-      }
-    }).on('click', '.t3js-formengine-select-itemstoselect', function(evt) {
-      // in multi-select environments with two (e.g. "Access"), on click the item from the right should go to the left
-      var $el = $(this)
-        , fieldName = $el.data('relatedfieldname')
-        , exclusiveValues = $el.data('exclusivevalues');
-
-      if (fieldName) {
-        // try to add each selected field to the "left" select field
-        $el.find(':selected').each(function() {
-          var $optionEl = $(this);
-          FormEngine.setSelectOptionFromExternalSource(fieldName, $optionEl.prop('value'), $optionEl.text(), $optionEl.prop('title'), exclusiveValues, $optionEl);
-        });
-      }
-    }).on('click', '.t3js-editform-close', function(e) {
+    $(document).on('click', '.t3js-editform-close', function(e) {
         e.preventDefault();
         FormEngine.preventExitIfNotSaved(
             FormEngine.preventExitIfNotSavedCallback
@@ -668,35 +574,7 @@ define(['jquery',
     }).on('click', '.t3js-editform-delete-record', function(e) {
       e.preventDefault();
       FormEngine.deleteAction(e, FormEngine.deleteActionCallback);
-    }).on('click', '.t3js-editform-delete-inline-record', function(e) {
-      e.preventDefault();
-      var title = TYPO3.lang['label.confirm.delete_record.title'] || 'Delete this record?';
-      var content = TYPO3.lang['label.confirm.delete_record.content'] || 'Are you sure you want to delete this record?';
-      var $anchorElement = $(this);
-      var $modal = Modal.confirm(title, content, Severity.warning, [
-        {
-          text: TYPO3.lang['buttons.confirm.delete_record.no'] || 'Cancel',
-          active: true,
-          btnClass: 'btn-default',
-          name: 'no'
-        },
-        {
-          text: TYPO3.lang['buttons.confirm.delete_record.yes'] || 'Yes, delete this record',
-          btnClass: 'btn-warning',
-          name: 'yes'
-        }
-      ]);
-      $modal.on('button.clicked', function(e) {
-        if (e.target.name === 'no') {
-          Modal.dismiss();
-        } else if (e.target.name === 'yes') {
-          var objectId = $anchorElement.data('objectid');
-          inline.deleteRecord(objectId);
-          Modal.dismiss();
-        }
-      });
     }).on('click', '.t3js-editform-submitButton', function(event) {
-      // remember the clicked submit button. we need to know that in TBE_EDITOR.submitForm();
       var $me = $(this),
         name = $me.data('name') || this.name,
         $elem = $('<input />').attr('type', 'hidden').attr('name', name).attr('value', '1');
@@ -707,41 +585,20 @@ define(['jquery',
       $(this).closest('.t3js-formengine-field-item').toggleClass('disabled');
     }).on('change', '.t3js-form-field-eval-null-placeholder-checkbox input[type="checkbox"]', function(e) {
       FormEngine.toggleCheckboxField($(this));
-    }).on('change', '.t3js-l10n-state-container input[type=radio]', function(event) {
-      // Change handler for "l10n_state" field changes
-      var $me = $(this);
-      var $input = $me.closest('.t3js-formengine-field-item').find('[data-formengine-input-name]');
-
-      if ($input.length > 0) {
-        var lastState = $input.data('last-l10n-state') || false,
-          currentState = $(this).val();
-
-        if (lastState && currentState === lastState) {
-          return;
-        }
-
-        if (currentState === 'custom') {
-          if (lastState) {
-            $(this).attr('data-original-language-value', $input.val());
-          }
-          $input.attr('disabled', false);
-        } else {
-          if (lastState === 'custom') {
-            $(this).closest('.t3js-l10n-state-container').find('.t3js-l10n-state-custom').attr('data-original-language-value', $input.val());
-          }
-          $input.attr('disabled', 'disabled');
-        }
-
-        $input.val($(this).attr('data-original-language-value')).trigger('change');
-        $input.data('last-l10n-state', $(this).val());
-      }
-    }).on('formengine.dp.change', function(event, $field) {
-      FormEngine.Validation.validate();
-      FormEngine.Validation.markFieldAsChanged($field);
-      $('.module-docheader-bar .btn').removeClass('disabled').prop('disabled', false);
     }).on('change', function(event) {
       $('.module-docheader-bar .btn').removeClass('disabled').prop('disabled', false);
+    }).on('click', '.t3js-element-browser', function(e) {
+      e.preventDefault();
+      e.stopPropagation();
+
+      const $me = $(e.currentTarget);
+      const mode = $me.data('mode');
+      const params = $me.data('params');
+
+      FormEngine.openPopupWindow(mode, params);
     });
+
+    window.addEventListener('message', FormEngine.handlePostMessage);
   };
 
   /**
@@ -783,6 +640,28 @@ define(['jquery',
     }
   };
 
+  FormEngine.handlePostMessage = function (e) {
+    if (!MessageUtility.MessageUtility.verifyOrigin(e.origin)) {
+      throw 'Denied message sent by ' + e.origin;
+    }
+
+    if (e.data.actionName === 'typo3:elementBrowser:elementAdded') {
+      if (typeof e.data.fieldName === 'undefined') {
+        throw 'fieldName not defined in message';
+      }
+
+      if (typeof e.data.value === 'undefined') {
+        throw 'value not defined in message';
+      }
+
+      const label = e.data.label || e.data.value;
+      const title = e.data.title || label;
+      const exclusiveValues = e.data.exclusiveValues || '';
+
+      FormEngine.setSelectOptionFromExternalSource(e.data.fieldName, e.data.value, label, title, exclusiveValues);
+    }
+  };
+
   /**
    * Initializes the remaining character views based on the fields' maxlength attribute
    */
@@ -818,44 +697,6 @@ define(['jquery',
     });
   };
 
-  /**
-   * Initialize select checkbox element checkboxes
-   */
-  FormEngine.initializeSelectCheckboxes = function() {
-    $('.t3js-toggle-checkboxes').each(function() {
-      var $checkbox = $(this);
-      var $table = $checkbox.closest('table');
-      var $checkboxes = $table.find('.t3js-checkbox');
-      var checkIt = $checkboxes.length === $table.find('.t3js-checkbox:checked').length;
-      $checkbox.prop('checked', checkIt);
-    });
-    $(document).on('change', '.t3js-toggle-checkboxes', function(e) {
-      e.preventDefault();
-      var $checkbox = $(this);
-      var $table = $checkbox.closest('table');
-      var $checkboxes = $table.find('.t3js-checkbox');
-      var checkIt = $checkboxes.length !== $table.find('.t3js-checkbox:checked').length;
-      $checkboxes.prop('checked', checkIt);
-      $checkbox.prop('checked', checkIt);
-      FormEngine.Validation.markFieldAsChanged($checkbox);
-    });
-    $(document).on('change', '.t3js-checkbox', function(e) {
-      FormEngine.updateCheckboxState(this);
-    });
-  };
-
-  /**
-   *
-   * @param {HTMLElement} source
-   */
-  FormEngine.updateCheckboxState = function(source) {
-    var $sourceElement = $(source);
-    var $table = $sourceElement.closest('table');
-    var $checkboxes = $table.find('.t3js-checkbox');
-    var checkIt = $checkboxes.length === $table.find('.t3js-checkbox:checked').length;
-    $table.find('.t3js-toggle-checkboxes').prop('checked', checkIt);
-  };
-
   /**
    * Get the properties required for proper rendering of the character counter
    *
@@ -885,99 +726,6 @@ define(['jquery',
     };
   };
 
-  /**
-   * Select field filter functions, see TCA option "enableMultiSelectFilterTextfield"
-   * and "multiSelectFilterItems"
-   */
-  FormEngine.SelectBoxFilter = {
-    options: {
-      fieldContainerSelector: '.t3js-formengine-field-group',
-      filterContainerSelector: '.t3js-formengine-multiselect-filter-container',
-      filterTextFieldSelector: '.t3js-formengine-multiselect-filter-textfield',
-      filterSelectFieldSelector: '.t3js-formengine-multiselect-filter-dropdown',
-      itemsToSelectElementSelector: '.t3js-formengine-select-itemstoselect'
-    }
-  };
-
-  /**
-   * Make sure that all selectors and input filters are recognized
-   * note: this also works on elements that are loaded asynchronously via AJAX, no need to call this method
-   * after an AJAX load.
-   */
-  FormEngine.SelectBoxFilter.initializeEvents = function() {
-    $(document).on('keyup', FormEngine.SelectBoxFilter.options.filterTextFieldSelector, function() {
-      var $selectElement = FormEngine.SelectBoxFilter.getSelectElement($(this));
-      FormEngine.SelectBoxFilter.filter($selectElement, $(this).val());
-    }).on('change', FormEngine.SelectBoxFilter.options.filterSelectFieldSelector, function() {
-      var $selectElement = FormEngine.SelectBoxFilter.getSelectElement($(this));
-      FormEngine.SelectBoxFilter.filter($selectElement, $(this).val());
-    });
-  };
-
-  /**
-   * Fetch the "itemstoselect" select element where a filter item is attached to
-   *
-   * @param {Object} $relativeElement
-   * @returns {*}
-   */
-  FormEngine.SelectBoxFilter.getSelectElement = function($relativeElement) {
-    var $containerElement = $relativeElement.closest(FormEngine.SelectBoxFilter.options.fieldContainerSelector);
-    return $containerElement.find(FormEngine.SelectBoxFilter.options.itemsToSelectElementSelector);
-  };
-
-  /**
-   * Filter the actual items
-   *
-   * @param {Object} $selectElement
-   * @param {String} filterText
-   */
-  FormEngine.SelectBoxFilter.filter = function($selectElement, filterText) {
-    var $allOptionElements;
-    if (!$selectElement.data('alloptions')) {
-      $allOptionElements = $selectElement.find('option').clone();
-      $selectElement.data('alloptions', $allOptionElements);
-    } else {
-      $allOptionElements = $selectElement.data('alloptions');
-    }
-
-    if (filterText.length > 0) {
-      var matchFilter = new RegExp(filterText, 'i');
-      $selectElement.html('');
-      $allOptionElements.each(function() {
-        var $item = $(this);
-        if ($item.text().match(matchFilter)) {
-          $selectElement.append($item.clone());
-        }
-      });
-    } else {
-      $selectElement.html($allOptionElements);
-    }
-  };
-
-  /**
-   * convert all textareas so they grow when it is typed in.
-   */
-  FormEngine.convertTextareasResizable = function() {
-    var $elements = $('.t3js-formengine-textarea');
-    if (TYPO3.settings.Textarea && TYPO3.settings.Textarea.autosize && $elements.length) {
-      require(['autosize'], function(autosize) {
-        autosize($elements);
-      });
-    }
-  };
-
-  /**
-   * convert all textareas to enable tab
-   */
-  FormEngine.convertTextareasEnableTab = function() {
-    var $elements = $('.t3js-enable-tab');
-    if ($elements.length) {
-      require(['taboverride'], function(taboverride) {
-        taboverride.set($elements);
-      });
-    }
-  };
-
   /**
    * Initialize input / text field "null" checkbox CSS overlay if no placeholder is set.
    */
@@ -1026,30 +774,18 @@ define(['jquery',
    */
   FormEngine.reinitialize = function() {
     // Apply "close" button to all input / datetime fields
-    if ($('.t3js-clearable').length) {
-      require(['TYPO3/CMS/Backend/jquery.clearable'], function() {
-        $('.t3js-clearable').clearable();
-      });
-    }
-    if ($('.t3-form-suggest').length) {
-      require(['TYPO3/CMS/Backend/FormEngineSuggest'], function(Suggest) {
-        $('.t3-form-suggest').each(function(index, suggestElement) {
-          new Suggest(suggestElement);
-        });
-      });
-    }
-    // Apply DatePicker to all date time fields
-    if ($('.t3js-datetimepicker').length) {
-      require(['TYPO3/CMS/Backend/DateTimePicker'], function(DateTimePicker) {
-        DateTimePicker.initialize();
+    const clearables = Array.from(document.querySelectorAll('.t3js-clearable')).filter(inputElement => {
+      // Filter input fields being a date time picker and a color picker
+      return !inputElement.classList.contains('t3js-datetimepicker') && !inputElement.classList.contains('t3js-color-picker');
+    });
+    if (clearables.length > 0) {
+      require(['TYPO3/CMS/Backend/Input/Clearable'], function() {
+        clearables.forEach(clearableField => clearableField.clearable());
       });
     }
 
-    FormEngine.convertTextareasResizable();
-    FormEngine.convertTextareasEnableTab();
     FormEngine.initializeNullNoPlaceholderCheckboxes();
     FormEngine.initializeNullWithPlaceholderCheckboxes();
-    FormEngine.initializeInputLinkToggle();
     FormEngine.initializeLocalizationStateSelector();
     FormEngine.initializeRemainingCharacterViews();
   };
@@ -1067,60 +803,6 @@ define(['jquery',
     });
   };
 
-  /**
-   * Toggle for input link explanation
-   */
-  FormEngine.initializeInputLinkToggle = function() {
-    var toggleClass = '.t3js-form-field-inputlink-explanation-toggle',
-      inputFieldClass = '.t3js-form-field-inputlink-input',
-      explanationClass = '.t3js-form-field-inputlink-explanation',
-      iconClass = '.t3js-form-field-inputlink-icon';
-
-    // if empty, show input field
-    $(explanationClass).filter(function() {
-      return !$.trim($(this).val());
-    }).each(function() {
-      var $group = $(this).closest('.t3js-form-field-inputlink'),
-        $inputField = $group.find(inputFieldClass),
-        $explanationField = $group.find(explanationClass);
-      $explanationField.toggleClass('hidden', true);
-      $inputField.toggleClass('hidden', false);
-      $group.find('.form-control-clearable button.close').toggleClass('hidden', false)
-    });
-
-    $(document).on('click', toggleClass, function(e) {
-      e.preventDefault();
-
-      var $group = $(this).closest('.t3js-form-field-inputlink'),
-        $inputField = $group.find(inputFieldClass),
-        $explanationField = $group.find(explanationClass),
-        explanationShown;
-
-      explanationShown = !$explanationField.hasClass('hidden');
-      $explanationField.toggleClass('hidden', explanationShown);
-      $inputField.toggleClass('hidden', !explanationShown);
-      $group.find('.form-control-clearable button.close').toggleClass('hidden', !explanationShown);
-    });
-
-    $(inputFieldClass).on('change', function() {
-      var $group = $(this).closest('.t3js-form-field-inputlink'),
-        $inputField = $group.find(inputFieldClass),
-        $explanationField = $group.find(explanationClass),
-        explanationShown;
-
-      if (!$explanationField.hasClass('hidden')) {
-
-        explanationShown = !$explanationField.hasClass('hidden');
-        $explanationField.toggleClass('hidden', explanationShown);
-        $inputField.toggleClass('hidden', !explanationShown);
-        $group.find('.form-control-clearable button.close').toggleClass('hidden', !explanationShown)
-      }
-
-      $group.find(toggleClass).addClass('disabled').attr('disabled', 'disabled');
-      $group.find(iconClass).empty();
-    });
-  };
-
   /**
    * @return {boolean}
    */
@@ -1194,9 +876,8 @@ define(['jquery',
           callback.call(null, true);
         } else if (e.target.name === 'save') {
           $('form[name=' + FormEngine.formName + ']').append($elem);
-          $('input[name=doSave]').val(1);
           Modal.dismiss();
-          document.editform.submit();
+          FormEngine.saveDocument();
         }
       });
     } else {
@@ -1228,6 +909,28 @@ define(['jquery',
     return true;
   };
 
+  FormEngine.requestConfirmationOnFieldChange = function(fieldName, showConfirmation) {
+    const $field = FormEngine.getFieldElement(fieldName);
+    $field.on('change', function() {
+      if (showConfirmation) {
+        const $modal = Modal.confirm(
+          TYPO3.lang['FormEngine.refreshRequiredTitle'],
+          TYPO3.lang['FormEngine.refreshRequiredContent']
+        );
+
+        $modal.on('button.clicked', function(e) {
+          if (e.target.name === 'ok') {
+            FormEngine.saveDocument();
+          }
+
+          Modal.dismiss();
+        });
+      } else {
+        FormEngine.saveDocument();
+      }
+    });
+  };
+
   /**
    * Preview action
    *
@@ -1272,9 +975,8 @@ define(['jquery',
         break;
       case 'save':
         $('form[name=' + FormEngine.formName + ']').append($actionElement);
-        $('input[name=doSave]').val(1);
         window.open('', 'newTYPO3frontendWindow');
-        document.editform.submit();
+        FormEngine.saveDocument();
         break;
     }
   };
@@ -1372,8 +1074,7 @@ define(['jquery',
         break;
       case 'yes':
         $form.append($actionElement);
-        $('input[name=doSave]').val(1);
-        document.editform.submit();
+        FormEngine.saveDocument();
         break;
     }
   };
@@ -1465,8 +1166,7 @@ define(['jquery',
         break;
       case 'yes':
         $form.append($actionElement);
-        $('input[name=doSave]').val(1);
-        document.editform.submit();
+        FormEngine.saveDocument();
         break;
     }
   };
@@ -1545,10 +1245,8 @@ define(['jquery',
    */
   FormEngine.deleteActionCallback = function(modalButtonName, $anchorElement) {
     Modal.dismiss();
-    switch(modalButtonName) {
-      case 'yes':
-        deleteRecord($anchorElement.data('table'), $anchorElement.data('uid'), $anchorElement.data('return-url'));
-        break;
+    if (modalButtonName === 'yes') {
+        FormEngine.invokeRecordDeletion($anchorElement);
     }
   };
 
@@ -1596,6 +1294,11 @@ define(['jquery',
     document.editform.submit();
   };
 
+  FormEngine.saveDocument = function() {
+    document.editform.doSave.value = 1;
+    document.editform.submit();
+  };
+
   /**
    * Main init function called from outside
    *
@@ -1605,19 +1308,28 @@ define(['jquery',
    * @param {Number} mode
    */
   FormEngine.initialize = function(browserUrl, mode) {
+    DocumentSaveActions.getInstance().addPreSubmitCallback(function() {
+      $('[data-active-password]:not([type="password"])').each(function(index, element) {
+        element.setAttribute('type', 'password');
+        element.blur();
+      });
+    });
+
     FormEngine.browserUrl = browserUrl;
     FormEngine.Validation.setUsMode(mode);
 
     $(function() {
       FormEngine.initializeEvents();
-      FormEngine.SelectBoxFilter.initializeEvents();
-      FormEngine.initializeSelectCheckboxes();
       FormEngine.Validation.initialize();
       FormEngine.reinitialize();
       $('#t3js-ui-block').remove();
     });
   };
 
+  FormEngine.invokeRecordDeletion = function ($anchorElement) {
+    window.location.href = $anchorElement.attr('href');
+  };
+
   // load required modules to hook in the post initialize function
   if (undefined !== TYPO3.settings.RequireJS && undefined !== TYPO3.settings.RequireJS.PostInitializationModules['TYPO3/CMS/Backend/FormEngine']) {
     $.each(TYPO3.settings.RequireJS.PostInitializationModules['TYPO3/CMS/Backend/FormEngine'], function(pos, moduleName) {
